@@ -108,9 +108,7 @@ void stat2fsal_attributes(const struct stat *buffstat,
 	fsalattr->ctime = posix2fsal_time(buffstat->st_ctime, 0);
 	fsalattr->mtime = posix2fsal_time(buffstat->st_mtime, 0);
 
-	fsalattr->chgtime = posix2fsal_time(MAX(buffstat->st_mtime,
-						buffstat->st_ctime), 0);
-	fsalattr->change = fsalattr->chgtime.tv_sec;
+	fsalattr->change = MAX(buffstat->st_mtime, buffstat->st_ctime);
 
 	fsalattr->spaceused = buffstat->st_blocks * S_BLKSIZE;
 
@@ -167,53 +165,15 @@ void gluster_cleanup_vars(struct glfs_object *glhandle)
 	}
 }
 
-/* fs_specific_has() parses the fs_specific string for a particular key,
- * returns true if found, and optionally returns a val if the string is
- * of the form key=val.
- *
- * The fs_specific string is a comma (,) separated options where each option
- * can be of the form key=value or just key. Example:
- *	FS_specific = "foo=baz,enable_A";
- */
-bool fs_specific_has(const char *fs_specific, const char *key, char *val,
-		     int *max_val_bytes)
-{
-	char *next_comma, *option;
-	bool ret;
-	char *fso_dup = NULL;
-
-	if (!fs_specific || !fs_specific[0])
-		return false;
-
-	fso_dup = gsh_strdup(fs_specific);
-
-	for (option = strtok_r(fso_dup, ",", &next_comma); option;
-	     option = strtok_r(NULL, ",", &next_comma)) {
-		char *k = option;
-		char *v = k;
-
-		strsep(&v, "=");
-		if (strcmp(k, key) == 0) {
-			if (val)
-				strncpy(val, v, *max_val_bytes);
-			if (max_val_bytes)
-				*max_val_bytes = strlen(v) + 1;
-			ret = true;
-			goto cleanup;
-		}
-	}
-
-	ret = false;
- cleanup:
-	gsh_free(fso_dup);
-	return ret;
-}
-
 void setglustercreds(struct glusterfs_export *glfs_export, uid_t *uid,
 		     gid_t *gid, unsigned int ngrps, gid_t *groups,
+		     char *client_addr, unsigned int client_addr_len,
 		     char *file, int line, char *function)
 {
 	int rc = 0;
+#ifdef USE_GLUSTER_DELEGATION
+	char lease_id[GLAPI_LEASE_ID_SIZE];
+#endif
 
 	if (uid) {
 		if (*uid != glfs_export->saveduid)
@@ -238,12 +198,21 @@ void setglustercreds(struct glusterfs_export *glfs_export, uid_t *uid,
 	else
 		rc = glfs_setfsgroups(0, NULL);
 
+#ifdef USE_GLUSTER_DELEGATION
+	if ((client_addr_len <= GLAPI_LEASE_ID_SIZE) && client_addr) {
+		memset(lease_id, 0, GLFS_LEASE_ID_SIZE);
+		memcpy(lease_id, client_addr, client_addr_len);
+		rc = glfs_setfsleaseid(lease_id);
+	} else
+		rc = glfs_setfsleaseid(NULL);
+#endif
  out:
 	if (rc != 0) {
 		DisplayLogComponentLevel(COMPONENT_FSAL, file, line, function,
 			 NIV_FATAL,
 			"Could not set Gluster credentials - uid(%d), gid(%d)",
-			*uid, *gid);
+			uid ? *uid : glfs_export->saveduid,
+			gid ? *gid : glfs_export->savedgid);
 	}
 }
 
