@@ -772,25 +772,6 @@ find_fd(int *fd, struct fsal_obj_handle *obj_hdl, bool bypass,
 	return status;
 }
 
-static fsal_status_t
-gpfs_write_plus_fd(int my_fd, uint64_t offset,
-		   size_t buffer_size, void *buffer, size_t *write_amount,
-		   bool *fsal_stable, struct io_info *info, int expfd)
-{
-	switch (info->io_content.what) {
-	case NFS4_CONTENT_DATA:
-		return GPFSFSAL_write(my_fd, offset, buffer_size, buffer,
-				     write_amount, fsal_stable, op_ctx,
-				     expfd);
-	case NFS4_CONTENT_DEALLOCATE:
-		return GPFSFSAL_alloc(my_fd, offset, buffer_size, false);
-	case NFS4_CONTENT_ALLOCATE:
-		return GPFSFSAL_alloc(my_fd, offset, buffer_size, true);
-	default:
-		return fsalstat(ERR_FSAL_UNION_NOTSUPP, 0);
-	}
-}
-
 /**
  * @brief Read data from a file
  *
@@ -964,20 +945,12 @@ gpfs_write2(struct fsal_obj_handle *obj_hdl,
 		return;
 	}
 
-	if (write_arg->info)
-		status = gpfs_write_plus_fd(my_fd, write_arg->offset,
-					    write_arg->iov[0].iov_len,
-					    write_arg->iov[0].iov_base,
-					    &write_arg->io_amount,
-					    &write_arg->fsal_stable,
-					    write_arg->info, export_fd);
-	else
-		status = GPFSFSAL_write(my_fd, write_arg->offset,
-					write_arg->iov[0].iov_len,
-					write_arg->iov[0].iov_base,
-					&write_arg->io_amount,
-					&write_arg->fsal_stable,
-					op_ctx, export_fd);
+	status = GPFSFSAL_write(my_fd, write_arg->offset,
+				write_arg->iov[0].iov_len,
+				write_arg->iov[0].iov_base,
+				&write_arg->io_amount,
+				&write_arg->fsal_stable,
+				op_ctx, export_fd);
 
 	if (gpfs_fd)
 		PTHREAD_RWLOCK_unlock(&gpfs_fd->fdlock);
