@@ -31,6 +31,7 @@
 #include <unistd.h>
 #include <utime.h>
 #include <sys/time.h>
+#include <urcu-bp.h>
 
 /* Setup up_vector. File system's upvector_mutex must be held */
 static bool setup_up_vector(struct gpfs_filesystem *gpfs_fs)
@@ -82,6 +83,8 @@ void *GPFSFSAL_UP_Thread(void *Arg)
 	int errsv = 0;
 	fsal_status_t fsal_status = {0,};
 
+	rcu_register_thread();
+
 #ifdef _VALGRIND_MEMCHECK
 		memset(&handle, 0, sizeof(handle));
 		memset(&buf, 0, sizeof(buf));
@@ -89,9 +92,9 @@ void *GPFSFSAL_UP_Thread(void *Arg)
 		memset(&devid, 0, sizeof(devid));
 #endif
 
-	snprintf(thr_name, sizeof(thr_name),
-		 "fsal_up_%"PRIu64".%"PRIu64,
-		 gpfs_fs->fs->dev.major, gpfs_fs->fs->dev.minor);
+	(void) snprintf(thr_name, sizeof(thr_name),
+			"fsal_up_%"PRIu64".%"PRIu64,
+			gpfs_fs->fs->dev.major, gpfs_fs->fs->dev.minor);
 	SetNameFunction(thr_name);
 
 	LogFullDebug(COMPONENT_FSAL_UP,
@@ -134,7 +137,7 @@ void *GPFSFSAL_UP_Thread(void *Arg)
 				LogFatal(COMPONENT_FSAL_UP,
 					 "Ganesha version %d mismatch GPFS version %d.",
 					 callback.interface_version, rc);
-				return NULL;
+				goto out;
 			}
 
 			if (errsv == EINTR)
@@ -205,7 +208,7 @@ void *GPFSFSAL_UP_Thread(void *Arg)
 		PTHREAD_MUTEX_lock(&gpfs_fs->upvector_mutex);
 		if (!setup_up_vector(gpfs_fs)) {
 			PTHREAD_MUTEX_unlock(&gpfs_fs->upvector_mutex);
-			return NULL;
+			goto out;
 		}
 		event_func = gpfs_fs->up_vector;
 
@@ -368,11 +371,11 @@ void *GPFSFSAL_UP_Thread(void *Arg)
 					upflags = 0;
 					if (flags & UP_SIZE)
 						attr.valid_mask |=
-						   ATTR_CHGTIME | ATTR_CHANGE |
+						   ATTR_CHANGE |
 						   ATTR_SIZE | ATTR_SPACEUSED;
 					if (flags & UP_SIZE_BIG) {
 						attr.valid_mask |=
-						   ATTR_CHGTIME | ATTR_CHANGE |
+						   ATTR_CHANGE |
 						   ATTR_SIZE | ATTR_SPACEUSED;
 						upflags |=
 						   fsal_up_update_filesize_inc |
@@ -380,21 +383,21 @@ void *GPFSFSAL_UP_Thread(void *Arg)
 					}
 					if (flags & UP_MODE)
 						attr.valid_mask |=
-						   ATTR_CHGTIME | ATTR_CHANGE |
+						   ATTR_CHANGE |
 						   ATTR_MODE;
 					if (flags & UP_OWN)
 						attr.valid_mask |=
-						   ATTR_CHGTIME | ATTR_CHANGE |
+						   ATTR_CHANGE |
 						   ATTR_OWNER | ATTR_GROUP |
 						   ATTR_MODE;
 					if (flags & UP_TIMES)
 						attr.valid_mask |=
-						   ATTR_CHGTIME | ATTR_CHANGE |
+						   ATTR_CHANGE |
 						   ATTR_ATIME | ATTR_CTIME |
 						    ATTR_MTIME;
 					if (flags & UP_ATIME)
 						attr.valid_mask |=
-						   ATTR_CHGTIME | ATTR_CHANGE |
+						   ATTR_CHANGE |
 						   ATTR_ATIME;
 					if (flags & UP_NLINK)
 						attr.valid_mask |=
@@ -429,7 +432,7 @@ void *GPFSFSAL_UP_Thread(void *Arg)
 				"Terminating the GPFS up call thread for %d",
 				gpfs_fs->root_fd);
 			PTHREAD_MUTEX_unlock(&gpfs_fs->upvector_mutex);
-			return NULL;
+			goto out;
 
 		case INODE_INVALIDATE:
 			LogMidDebug(COMPONENT_FSAL_UP,
@@ -468,6 +471,7 @@ void *GPFSFSAL_UP_Thread(void *Arg)
 				fsal_err_txt(fsal_status));
 		}
 	}
-
+out:
+	rcu_unregister_thread();
 	return NULL;
 }				/* GPFSFSAL_UP_Thread */
