@@ -66,7 +66,7 @@ struct gpfs_fsal_obj_handle *alloc_handle(struct gpfs_file_handle *fh,
 
 	hdl->handle = (struct gpfs_file_handle *)&hdl[1];
 	hdl->obj_handle.fs = fs;
-	memcpy(hdl->handle, fh, sizeof(struct gpfs_file_handle));
+	memcpy(hdl->handle, fh, fh->handle_size);
 	hdl->obj_handle.type = attributes->type;
 	if (hdl->obj_handle.type == REGULAR_FILE) {
 		hdl->u.file.fd.fd = -1;	/* no open on this yet */
@@ -92,18 +92,6 @@ struct gpfs_fsal_obj_handle *alloc_handle(struct gpfs_file_handle *fh,
 	return hdl;
 }
 
-uint64_t get_handle2inode(struct gpfs_file_handle *gfh)
-{
-	struct f_handle {
-		char unused1[8];
-		uint64_t inode;  /* inode for file */
-		char unused2[8];
-		uint64_t pinode; /* inode for parent */
-	} *f_handle = (struct f_handle *)gfh->f_handle;
-
-	return f_handle->inode;
-}
-
 /* lookup
  * deprecated NULL parent && NULL path implies root handle
  */
@@ -638,9 +626,13 @@ static fsal_status_t getxattrs(struct fsal_obj_handle *obj_hdl,
 			return fsalstat(ERR_FSAL_NOENT, 0);
 		return fsalstat(posix2fsal_error(errsv), errsv);
 	}
+
+	/* Make sure utf8string is NUL terminated */
+	xa_value->utf8string_val[gxarg.value_len] = '\0';
+
 	LogDebug(COMPONENT_FSAL,
-		"GETXATTRS returned value %.*s len %d rc %d",
-		gxarg.value_len, (char *)gxarg.value, gxarg.value_len, rc);
+		"GETXATTRS returned value %s len %d rc %d",
+		(char *)gxarg.value, gxarg.value_len, rc);
 
 	xa_value->utf8string_len = gxarg.value_len;
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
@@ -778,7 +770,7 @@ static fsal_status_t listxattrs(struct fsal_obj_handle *obj_hdl,
 		next += 1;
 
 		LogDebug(COMPONENT_FSAL,
-		"nameP %s at offset %td", name, (next - name));
+			 "nameP %s at offset %td", name, (next - name));
 
 		if (entryCount >= *la_cookie) {
 			if ((((char *)entry - (char *)lr_names->entries) +
@@ -804,6 +796,7 @@ static fsal_status_t listxattrs(struct fsal_obj_handle *obj_hdl,
 			entry->utf8string_val = val;
 			memcpy(entry->utf8string_val, name,
 						entry->utf8string_len);
+			entry->utf8string_val[entry->utf8string_len] = '\0';
 
 			LogFullDebug(COMPONENT_FSAL,
 				"entry %d val %p at %p len %d at %p name %s",
@@ -1038,8 +1031,8 @@ fsal_status_t gpfs_lookup_path(struct fsal_export *exp_hdl,
 		attributes.request_mask |= attrs_out->request_mask;
 
 	if (dir_fd < 0) {
-		LogCrit(COMPONENT_FSAL,
-			"Could not open directory for path %s", path);
+		LogDebug(COMPONENT_FSAL,
+			 "Could not open directory for path %s", path);
 		fsal_status = fsalstat(posix2fsal_error(-dir_fd), retval);
 		goto errout;
 	}
@@ -1196,7 +1189,7 @@ fsal_status_t gpfs_create_handle(struct fsal_export *exp_hdl,
 		return fsalstat(ERR_FSAL_FAULT, 0);
 
 	fh = alloca(hdl_desc->len);
-	memcpy(fh, hdl_desc->addr, hdl_desc->len); /* struct aligned copy */
+	memcpy(fh, hdl_desc->addr, hdl_desc->len);
 
 	gpfs_extract_fsid(fh, &fsid);
 
