@@ -48,7 +48,6 @@ struct flock
 
 #define GPFS_DEVNAMEX "/dev/ss0"  /* Must be the same as GPFS_DEVNAME */
 #define kGanesha 140             /* Must be the same as Ganesha in enum kxOps */
-#define kLWEvent 169             /* Must be the same as LWEvent in enum kxOps */
 
 #define OPENHANDLE_GET_VERSION    100
 #define OPENHANDLE_GET_VERSION2   1002
@@ -96,14 +95,26 @@ struct flock
 #define OPENHANDLE_SETXATTRS      145
 #define OPENHANDLE_REMOVEXATTRS   146
 #define OPENHANDLE_LISTXATTRS     147
-#define OPENHANDLE_FS_LOCATIONS   148
+#define OPENHANDLE_MKNODE_BY_NAME 148
 #define OPENHANDLE_reserved       149
 #define OPENHANDLE_TRACE_ME       150
 #define OPENHANDLE_QUOTA          151
-#define OPENHANDLE_MKNODE_BY_NAME 152
+#define OPENHANDLE_FS_LOCATIONS   152
 
-#define LWE_UPDATE 201
-#define MAX_PATH_LEN 1024
+/* If there is any change in above constants, then update below values.
+ * Currently ignoring opcode 1002 */
+#define GPFS_MIN_OP		 OPENHANDLE_GET_VERSION
+#define GPFS_MAX_OP		 OPENHANDLE_FS_LOCATIONS
+#define GPFS_STAT_NO_OP_1           3
+#define GPFS_STAT_NO_OP_2           4
+#define GPFS_STAT_NO_OP_3           5
+/* max stat ops including placeholder for phantom ops  */
+#define GPFS_STAT_MAX_OPS (GPFS_MAX_OP-GPFS_MIN_OP+2)
+/* placeholder index is the last index in the array */
+#define GPFS_STAT_PH_INDEX (GPFS_STAT_MAX_OPS-1)
+/* total ops excluding the missing ops 103, 104 and 105 and the placeholder
+ * for phantom ops */
+#define GPFS_TOTAL_OPS     (GPFS_STAT_MAX_OPS-4)
 
 struct trace_arg
 {
@@ -747,4 +758,6 @@ struct quotactl_arg
 }
 #endif
 
+extern struct fsal_stats gpfs_stats;
+
 #endif /* H_GPFS_NFS */
