@@ -1,7 +1,7 @@
 /*
  * vim:shiftwidth=8:tabstop=8:
  *
- * Copyright 2017-2018 Red Hat, Inc.
+ * Copyright 2017-2019 Red Hat, Inc.
  * Author: Daniel Gryniewicz  dang@redhat.com
  *
  *
@@ -28,6 +28,8 @@
 
 #include "config.h"
 
+#include <unistd.h>
+#include <stdlib.h>
 #include "fsal.h"
 #include "fsal_convert.h"
 #include "FSAL/fsal_commonlib.h"
@@ -39,6 +41,8 @@
 #include "gsh_lttng/fsal_mem.h"
 #endif
 #include "../Stackable_FSALs/FSAL_MDCACHE/mdcache_ext.h"
+#include "nfs_core.h"
+#include "common_utils.h"
 
 static void mem_release(struct fsal_obj_handle *obj_hdl);
 
@@ -194,16 +198,18 @@ static void _mem_int_put_ref(struct mem_fsal_obj_handle *myself,
 static void package_mem_handle(struct mem_fsal_obj_handle *myself)
 {
 	char buf[MAXPATHLEN];
-	uint16_t len;
+	uint16_t len_fileid, len_name;
 	uint64_t hashkey;
 	int opaque_bytes_used = 0, pathlen = 0;
 
 	memset(buf, 0, sizeof(buf));
 
 	/* Make hashkey */
-	len = sizeof(myself->obj_handle.fileid);
-	memcpy(buf, &myself->obj_handle.fileid, len);
-	strncpy(buf + len, myself->m_name, sizeof(buf) - len);
+	len_fileid = sizeof(myself->obj_handle.fileid);
+	memcpy(buf, &myself->obj_handle.fileid, len_fileid);
+	len_name = strlen(myself->m_name);
+	memcpy(buf + len_fileid, myself->m_name,
+	       MIN(len_name, sizeof(buf) - len_name));
 	hashkey = CityHash64(buf, sizeof(buf));
 
 	memcpy(myself->handle, &hashkey, sizeof(hashkey));
@@ -212,15 +218,14 @@ static void package_mem_handle(struct mem_fsal_obj_handle *myself)
 	/* include length of the name in the handle.
 	 * MAXPATHLEN=4096 ... max path length can be contained in a short int.
 	 */
-	len = strlen(myself->m_name);
-	memcpy(myself->handle + opaque_bytes_used, &len, sizeof(len));
-	opaque_bytes_used += sizeof(len);
+	memcpy(myself->handle + opaque_bytes_used, &len_name, sizeof(len_name));
+	opaque_bytes_used += sizeof(len_name);
 
 	/* Either the nfsv4 fh opaque size or the length of the name.
 	 * Ideally we can include entire mem name for guaranteed
 	 * uniqueness of mem handles.
 	 */
-	pathlen = MIN(V4_FH_OPAQUE_SIZE - opaque_bytes_used, len);
+	pathlen = MIN(V4_FH_OPAQUE_SIZE - opaque_bytes_used, len_name);
 	memcpy(myself->handle + opaque_bytes_used, myself->m_name, pathlen);
 	opaque_bytes_used += pathlen;
 
@@ -252,6 +257,8 @@ static void mem_insert_obj(struct mem_fsal_obj_handle *parent,
 	mem_int_get_ref(child);
 	dirent->dir = parent;
 	dirent->d_name = gsh_strdup(name);
+	/* Index is hash of the name */
+	dirent->d_index = CityHash64(name, strlen(name));
 
 	/* Link into child */
 	PTHREAD_RWLOCK_wrlock(&child->obj_handle.obj_lock);
@@ -262,8 +269,7 @@ static void mem_insert_obj(struct mem_fsal_obj_handle *parent,
 	PTHREAD_RWLOCK_wrlock(&parent->obj_handle.obj_lock);
 	/* Name tree */
 	avltree_insert(&dirent->avl_n, &parent->mh_dir.avl_name);
-	/* Index tree (increment under lock) */
-	dirent->d_index = (parent->mh_dir.next_i)++;
+	/* Index tree */
 	avltree_insert(&dirent->avl_i, &parent->mh_dir.avl_index);
 	/* Update numkids */
 	numkids = atomic_inc_uint32_t(&parent->mh_dir.numkids);
@@ -297,6 +303,88 @@ mem_dirent_lookup(struct mem_fsal_obj_handle *dir, const char *name)
 	return avltree_container_of(node, struct mem_dirent, avl_n);
 }
 
+/**
+ * @brief Get the next dirent in a directory
+ *
+ * @note Caller must hold the obj_lock on the obj
+ *
+ * @param[in] dirent	Current dirent
+ * @return Next dirent, or NULL if at EOD
+ */
+static struct mem_dirent *
+mem_dirent_next(struct mem_dirent *dirent)
+{
+	struct avltree_node *node;
+
+	node = avltree_next(&dirent->avl_i);
+	if (!node) {
+		return NULL;
+	}
+
+	return avltree_container_of(node, struct mem_dirent, avl_i);
+}
+
+/**
+ * @brief Seek to a location in a directory
+ *
+ * Handle normal vs whence-is-name directories.
+ *
+ * @note Caller must hold the obj_lock on the obj
+ *
+ * @param[in] dir	Directory to seek in
+ * @param[in] seekloc	Location to seek to
+ * @return Dirent associated with seekloc
+ */
+static struct mem_dirent *
+mem_readdir_seekloc(struct mem_fsal_obj_handle *dir, fsal_cookie_t seekloc)
+{
+	struct mem_dirent *dirent;
+	struct avltree_node *node;
+	struct mem_dirent key;
+
+	if (!seekloc) {
+		/* Start from the beginning.  We walk the index tree, so always
+		 * grab from the index tree. */
+		node = avltree_first(&dir->mh_dir.avl_index);
+		if (!node) {
+			return NULL;
+		}
+		dirent = avltree_container_of(node, struct mem_dirent,
+					      avl_i);
+		return dirent;
+	}
+
+
+	key.d_index = seekloc;
+	node = avltree_lookup(&key.avl_i, &dir->mh_dir.avl_index);
+	if (!node) {
+		/* Dirent was probably deleted.  Find the next one */
+		node = avltree_sup(&key.avl_i, &dir->mh_dir.avl_index);
+	}
+	if (!node) {
+		/* Done */
+		return NULL;
+	}
+
+	dirent = avltree_container_of(node, struct mem_dirent, avl_i);
+
+	return dirent;
+}
+
+/**
+ * @brief Update the change attribute of the FSAL object
+ *
+ * @note Caller must hold the obj_lock on the obj
+ *
+ * @param[in] obj	FSAL obj which was modified
+ */
+static void mem_update_change_locked(struct mem_fsal_obj_handle *obj)
+{
+	now(&obj->attrs.mtime);
+	obj->attrs.ctime = obj->attrs.mtime;
+	obj->attrs.change = timespec_to_nsecs(&obj->attrs.mtime);
+}
+
 /**
  * @brief Remove an obj from it's parent's tree
  *
@@ -331,6 +419,8 @@ static void mem_remove_dirent_locked(struct mem_fsal_obj_handle *parent,
 	gsh_free(dirent);
 
 	mem_int_put_ref(child);
+
+	mem_update_change_locked(parent);
 }
 
 /**
@@ -462,8 +552,10 @@ static void mem_copy_attrs_mask(struct attrlist *attrs_in,
 
 	/* XXX TODO copy ACL */
 
-	attrs_out->chgtime = attrs_out->ctime;
-	attrs_out->change = timespec_to_nsecs(&attrs_out->chgtime);
+	/** @todo FSF - this calculation may be different than what particular
+	 *              FSALs use. Is that a problem?
+	 */
+	attrs_out->change = timespec_to_nsecs(&attrs_out->ctime);
 }
 
 /**
@@ -475,7 +567,7 @@ static void mem_copy_attrs_mask(struct attrlist *attrs_in,
 static fsal_status_t mem_open_my_fd(struct fsal_fd *fd,
 				    fsal_openflags_t openflags)
 {
-	fd->openflags = openflags;
+	fd->openflags = FSAL_O_NFS_FLAGS(openflags);
 
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
@@ -523,7 +615,15 @@ static fsal_status_t mem_open_func(struct fsal_obj_handle *obj_hdl,
 static fsal_status_t mem_close_func(struct fsal_obj_handle *obj_hdl,
 				    struct fsal_fd *fd)
 {
-	return mem_close_my_fd(fd);
+	fsal_status_t status;
+
+	PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
+
+	status = mem_close_my_fd(fd);
+
+	PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
+
+	return status;
 }
 
 #define mem_alloc_handle(p, n, t, e, a) \
@@ -598,7 +698,6 @@ _mem_alloc_handle(struct mem_fsal_obj_handle *parent,
 
 	/* Use full timer resolution */
 	now(&hdl->attrs.ctime);
-	hdl->attrs.chgtime = hdl->attrs.ctime;
 
 	if ((attrs && attrs->valid_mask & ATTR_ATIME) != 0)
 		hdl->attrs.atime = attrs->atime;
@@ -610,8 +709,12 @@ _mem_alloc_handle(struct mem_fsal_obj_handle *parent,
 	else
 		hdl->attrs.mtime = hdl->attrs.ctime;
 
+
+	/** @todo FSF - this calculation may be different than what particular
+	 *              FSALs use. Is that a problem?
+	 */
 	hdl->attrs.change =
-		timespec_to_nsecs(&hdl->attrs.chgtime);
+		timespec_to_nsecs(&hdl->attrs.ctime);
 
 	switch (type) {
 	case REGULAR_FILE:
@@ -638,9 +741,9 @@ _mem_alloc_handle(struct mem_fsal_obj_handle *parent,
 	case DIRECTORY:
 		avltree_init(&hdl->mh_dir.avl_name, mem_n_cmpf, 0);
 		avltree_init(&hdl->mh_dir.avl_index, mem_i_cmpf, 0);
-		hdl->mh_dir.next_i = 2;
 		hdl->attrs.numlinks = 2;
 		hdl->mh_dir.numkids = 2;
+		hdl->mh_dir.parent = parent;
 		break;
 	default:
 		hdl->attrs.numlinks = 1;
@@ -839,8 +942,8 @@ static fsal_status_t mem_readdir(struct fsal_obj_handle *dir_hdl,
 				 bool *eof)
 {
 	struct mem_fsal_obj_handle *myself;
-	struct avltree_node *node;
-	fsal_cookie_t seekloc = 0;
+	struct mem_dirent *dirent, *dirent_next;
+	fsal_cookie_t cookie = 0;
 	struct attrlist attrs;
 	enum fsal_dir_result cb_rc;
 	int count = 0;
@@ -850,15 +953,13 @@ static fsal_status_t mem_readdir(struct fsal_obj_handle *dir_hdl,
 			      obj_handle);
 
 	if (whence != NULL)
-		seekloc = *whence;
-	else
-		seekloc = 2;
+		cookie = *whence;
 
 	*eof = true;
 
 #ifdef USE_LTTNG
 	tracepoint(fsalmem, mem_readdir, __func__, __LINE__, dir_hdl,
-		   myself->m_name, seekloc);
+		   myself->m_name, cookie);
 #endif
 	LogFullDebug(COMPONENT_FSAL, "hdl=%p, name=%s",
 		     myself, myself->m_name);
@@ -870,19 +971,12 @@ static fsal_status_t mem_readdir(struct fsal_obj_handle *dir_hdl,
 	 */
 	op_ctx->fsal_private = dir_hdl;
 
-	if (seekloc) {
-		struct mem_dirent key;
-
-		key.d_index = seekloc;
-		node = avltree_lookup(&key.avl_i, &myself->mh_dir.avl_index);
-	} else {
-		node = avltree_first(&myself->mh_dir.avl_index);
-	}
+	dirent = mem_readdir_seekloc(myself, cookie);
 
+	/* Always run in index order */
 	for (;
-	     node != NULL;
-	     node = avltree_next(node)) {
-		struct mem_dirent *dirent;
+	     dirent != NULL;
+	     dirent = dirent_next) {
 
 		if (count >= 2 * mdcache_param.dir.avl_chunk) {
 			LogFullDebug(COMPONENT_FSAL, "readahead done %d",
@@ -892,14 +986,19 @@ static fsal_status_t mem_readdir(struct fsal_obj_handle *dir_hdl,
 			break;
 		}
 
-		dirent = avltree_container_of(node, struct mem_dirent, avl_i);
+		dirent_next = mem_dirent_next(dirent);
+		if (dirent_next) {
+			cookie = dirent_next->d_index;
+		} else {
+			cookie = UINT64_MAX;
+		}
 
 		fsal_prepare_attrs(&attrs, attrmask);
 		fsal_copy_attrs(&attrs, &dirent->hdl->attrs, false);
 		mem_int_get_ref(dirent->hdl);
 
 		cb_rc = cb(dirent->d_name, &dirent->hdl->obj_handle, &attrs,
-			   dir_state, dirent->d_index + 1);
+			   dir_state, cookie);
 
 		fsal_release_attrs(&attrs);
 
@@ -1674,6 +1773,49 @@ fsal_status_t mem_reopen2(struct fsal_obj_handle *obj_hdl,
 	return status;
 }
 
+struct mem_async_arg {
+	struct fsal_obj_handle *obj_hdl;
+	struct fsal_io_arg *io_arg;
+	fsal_async_cb done_cb;
+	void *caller_arg;
+	struct gsh_export *ctx_export;
+	struct fsal_export *fsal_export;
+};
+
+static void
+mem_async_complete(struct fridgethr_context *ctx)
+{
+	struct mem_async_arg *async_arg = ctx->arg;
+	struct root_op_context root_op_context;
+	struct mem_fsal_export *mem_export =
+	   container_of(async_arg->fsal_export, struct mem_fsal_export, export);
+	uint32_t async_delay = atomic_fetch_uint32_t(&mem_export->async_delay);
+
+	/* Now check if we need to delay the call back */
+	if (atomic_fetch_uint32_t(&mem_export->async_type) != MEM_FIXED) {
+		/* Randomize delay */
+		async_delay = random() % async_delay;
+	}
+
+	if (async_delay != 0) {
+		/* Now actually delay call back */
+		usleep(async_delay);
+	}
+
+	/* Need an op context for the call back */
+	init_root_op_context(&root_op_context, async_arg->ctx_export,
+			     async_arg->fsal_export, 0, 0,
+			     UNKNOWN_REQUEST);
+
+	async_arg->done_cb(async_arg->obj_hdl, fsalstat(ERR_FSAL_NO_ERROR, 0),
+			   async_arg->io_arg, async_arg->caller_arg);
+
+	release_root_op_context();
+
+	gsh_free(async_arg);
+}
+
+
 /**
  * @brief Read data from a file
  *
@@ -1707,6 +1849,11 @@ void mem_read2(struct fsal_obj_handle *obj_hdl,
 	bool reusing_open_state_fd = false;
 	uint64_t offset = read_arg->offset;
 	int i;
+	struct mem_fsal_export *mem_export =
+	      container_of(op_ctx->fsal_export, struct mem_fsal_export, export);
+	uint32_t async_type = atomic_fetch_uint32_t(&mem_export->async_type);
+	uint32_t async_stall_delay =
+			atomic_fetch_uint32_t(&mem_export->async_stall_delay);
 
 	if (read_arg->info != NULL) {
 		/* Currently we don't support READ_PLUS */
@@ -1769,7 +1916,45 @@ void mem_read2(struct fsal_obj_handle *obj_hdl,
 	if (has_lock)
 		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 
+	if (async_type > MEM_RANDOM_OR_INLINE ||
+	    ((async_type == MEM_RANDOM_OR_INLINE) && ((random() % 1) == 1))) {
+		struct mem_async_arg *async_arg;
+
+		/* Was MEM_FIXED, MEM_RANDOM, or MEM_RANDOM_OR_INLINE and we
+		 * scored a non-inline.
+		 */
+		async_arg = gsh_malloc(sizeof(*async_arg));
+
+		async_arg->obj_hdl = obj_hdl;
+		async_arg->io_arg = read_arg;
+		async_arg->caller_arg = caller_arg;
+		async_arg->done_cb = done_cb;
+		async_arg->ctx_export = op_ctx->ctx_export;
+		async_arg->fsal_export = op_ctx->fsal_export;
+
+		if (fridgethr_submit(mem_async_fridge,
+				     mem_async_complete,
+				     async_arg) == 0) {
+			/* Async fired off... */
+			goto out;
+		}
+
+		/* Could not schedule, fall through and do an immediate
+		 * call back.
+		 */
+		gsh_free(async_arg);
+	}
+
 	done_cb(obj_hdl, fsalstat(ERR_FSAL_NO_ERROR, 0), read_arg, caller_arg);
+
+out:
+
+	if (async_stall_delay > 0) {
+		/* We have been asked to stall the calling thread, whether we
+		 * issued an inline or async callback.
+		 */
+		usleep(async_stall_delay);
+	}
 }
 
 /**
@@ -1805,13 +1990,11 @@ void mem_write2(struct fsal_obj_handle *obj_hdl,
 	bool reusing_open_state_fd = false;
 	uint64_t offset = write_arg->offset;
 	int i;
-
-	if (write_arg->info != NULL) {
-		/* Currently we don't support WRITE_PLUS */
-		done_cb(obj_hdl, fsalstat(ERR_FSAL_NOTSUPP, 0), write_arg,
-			caller_arg);
-		return;
-	}
+	struct mem_fsal_export *mem_export =
+	      container_of(op_ctx->fsal_export, struct mem_fsal_export, export);
+	uint32_t async_type = atomic_fetch_uint32_t(&mem_export->async_type);
+	uint32_t async_stall_delay =
+			atomic_fetch_uint32_t(&mem_export->async_stall_delay);
 
 	if (obj_hdl->type != REGULAR_FILE) {
 		/* Currently can only write to a file */
@@ -1859,14 +2042,56 @@ void mem_write2(struct fsal_obj_handle *obj_hdl,
 
 	/* Update change stats */
 	now(&myself->attrs.mtime);
-	myself->attrs.chgtime = myself->attrs.mtime;
+
+	/** @todo FSF - this calculation may be different than what particular
+	 *              FSALs use. Is that a problem? Also, above, ctime is used
+	 *              not mtime...
+	 */
 	myself->attrs.change =
-		timespec_to_nsecs(&myself->attrs.chgtime);
+		timespec_to_nsecs(&myself->attrs.mtime);
 
 	if (has_lock)
 		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 
+	if (async_type > MEM_RANDOM_OR_INLINE ||
+	    ((async_type == MEM_RANDOM_OR_INLINE) && ((random() % 1) == 1))) {
+		struct mem_async_arg *async_arg;
+
+		/* Was MEM_FIXED, MEM_RANDOM, or MEM_RANDOM_OR_INLINE and we
+		 * scored a non-inline.
+		 */
+		async_arg = gsh_malloc(sizeof(*async_arg));
+
+		async_arg->obj_hdl = obj_hdl;
+		async_arg->io_arg = write_arg;
+		async_arg->caller_arg = caller_arg;
+		async_arg->done_cb = done_cb;
+		async_arg->ctx_export = op_ctx->ctx_export;
+		async_arg->fsal_export = op_ctx->fsal_export;
+
+		if (fridgethr_submit(mem_async_fridge,
+				     mem_async_complete,
+				     async_arg) == 0) {
+			/* Async fired off... */
+			goto out;
+		}
+
+		/* Could not schedule, fall through and do an immediate
+		 * call back.
+		 */
+		gsh_free(async_arg);
+	}
+
 	done_cb(obj_hdl, fsalstat(ERR_FSAL_NO_ERROR, 0), write_arg, caller_arg);
+
+out:
+
+	if (async_stall_delay > 0) {
+		/* We have been asked to stall the calling thread, whether we
+		 * issued an inline or async callback.
+		 */
+		usleep(async_stall_delay);
+	}
 }
 
 /**
@@ -1997,23 +2222,23 @@ fsal_status_t mem_close2(struct fsal_obj_handle *obj_hdl,
 		   myself->m_name, state);
 #endif
 
+	PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
+
 	if (state->state_type == STATE_TYPE_SHARE ||
 	    state->state_type == STATE_TYPE_NLM_SHARE ||
 	    state->state_type == STATE_TYPE_9P_FID) {
 		/* This is a share state, we must update the share counters */
 
 		/* This can block over an I/O operation. */
-		PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
-
 		update_share_counters(&myself->mh_file.share,
 				      my_fd->openflags,
 				      FSAL_O_CLOSED);
-
-		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 	}
 
 	status = mem_close_my_fd(my_fd);
 
+	PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
+
 	return status;
 }
 
