@@ -33,6 +33,8 @@
 #include <arpa/inet.h>
 #include <sys/poll.h>
 #include <netdb.h>
+#include <urcu-bp.h>
+
 #include "gsh_list.h"
 #include "abstract_atomic.h"
 #include "fsal_types.h"
@@ -44,6 +46,7 @@
 #include "nfs_proto_tools.h"
 #include "export_mgr.h"
 #include "common_utils.h"
+#include "fsal_convert.h"
 
 #define FSAL_PROXY_NFS_V4 4
 #define FSAL_PROXY_NFS_V4_MINOR 1
@@ -482,20 +485,6 @@ static int pxy_connect(struct pxy_export *pxy_exp,
 	int sock;
 	int socklen;
 
-	if (pxy_exp->info.use_privileged_client_port) {
-		int priv_port = 0;
-
-		sock = rresvport_af(&priv_port, dest->ss_family);
-		if (sock < 0)
-			LogCrit(COMPONENT_FSAL,
-				"Cannot create TCP socket on privileged port");
-	} else {
-		sock = socket(dest->ss_family, SOCK_STREAM, IPPROTO_TCP);
-		if (sock < 0)
-			LogCrit(COMPONENT_FSAL, "Cannot create TCP socket - %d",
-				errno);
-	}
-
 	switch (dest->ss_family) {
 	case AF_INET:
 		((struct sockaddr_in *)dest)->sin_port = htons(port);
@@ -508,10 +497,23 @@ static int pxy_connect(struct pxy_export *pxy_exp,
 	default:
 		LogCrit(COMPONENT_FSAL, "Unknown address family %d",
 			dest->ss_family);
-		close(sock);
 		return -1;
 	}
 
+	if (pxy_exp->info.use_privileged_client_port) {
+		int priv_port = 0;
+
+		sock = rresvport_af(&priv_port, dest->ss_family);
+		if (sock < 0)
+			LogCrit(COMPONENT_FSAL,
+				"Cannot create TCP socket on privileged port");
+	} else {
+		sock = socket(dest->ss_family, SOCK_STREAM, IPPROTO_TCP);
+		if (sock < 0)
+			LogCrit(COMPONENT_FSAL, "Cannot create TCP socket - %d",
+				errno);
+	}
+
 	if (sock >= 0) {
 		if (connect(sock, (struct sockaddr *)dest, socklen) < 0) {
 			close(sock);
@@ -531,12 +533,12 @@ static int pxy_connect(struct pxy_export *pxy_exp,
 static void *pxy_rpc_recv(void *arg)
 {
 	struct pxy_export *pxy_exp = arg;
-	char addr[INET6_ADDRSTRLEN];
 	struct pollfd pfd;
 	int millisec = pxy_exp->info.srv_timeout * 1000;
 
 	SetNameFunction("pxy_rcv_thread");
 
+	rcu_register_thread();
 	while (!pxy_exp->rpc.close_thread) {
 		int nsleeps = 0;
 
@@ -548,15 +550,21 @@ static void *pxy_rpc_recv(void *arg)
 			/* early stop test */
 			if (pxy_exp->rpc.close_thread) {
 				PTHREAD_MUTEX_unlock(&pxy_exp->rpc.listlock);
-				return NULL;
+				goto out;
 			}
 			if (pxy_exp->rpc.rpc_sock < 0) {
-				if (nsleeps == 0)
-					sprint_sockaddr(&pxy_exp->info.srv_addr,
-							addr, sizeof(addr));
+				if (nsleeps == 0) {
+					char addr[SOCK_NAME_MAX];
+					struct display_buffer dspbuf = {
+						sizeof(addr), addr, addr};
+
+					display_sockaddr(
+					    &dspbuf, &pxy_exp->info.srv_addr);
+
 					LogCrit(COMPONENT_FSAL,
 						"Cannot connect to server %s:%u",
 						addr, pxy_exp->info.srv_port);
+				}
 				PTHREAD_MUTEX_unlock(&pxy_exp->rpc.listlock);
 				sleep(pxy_exp->info.retry_sleeptime);
 				nsleeps++;
@@ -571,7 +579,7 @@ static void *pxy_rpc_recv(void *arg)
 		PTHREAD_MUTEX_unlock(&pxy_exp->rpc.listlock);
 		/* early stop test */
 		if (pxy_exp->rpc.close_thread)
-			return NULL;
+			goto out;
 
 		pfd.fd = pxy_exp->rpc.rpc_sock;
 		pfd.events = POLLIN | POLLRDHUP;
@@ -606,7 +614,8 @@ static void *pxy_rpc_recv(void *arg)
 			PTHREAD_MUTEX_unlock(&pxy_exp->rpc.listlock);
 		}
 	}
-
+out:
+	rcu_unregister_thread();
 	return NULL;
 }
 
@@ -1027,9 +1036,18 @@ static int pxy_setclientid(clientid4 *new_clientid, sequenceid4 *new_seqid,
 	if (getsockname(pxy_exp->rpc.rpc_sock, &sin, &slen))
 		return -errno;
 
-	snprintf(clientid_name, MAXNAMLEN, "%s(%d) - GANESHA NFSv4 Proxy",
-		 inet_ntop(AF_INET, &sin.sin_addr, addrbuf, sizeof(addrbuf)),
-		 getpid());
+	rc = snprintf(clientid_name, sizeof(clientid_name),
+		      "%s(%d, %d) - GANESHA NFSv4 Proxy",
+		      inet_ntop(AF_INET, &sin.sin_addr, addrbuf,
+			       sizeof(addrbuf)),
+		      getpid(), pxy_exp->exp.export_id);
+
+	if (rc < 0) {
+		return -errno;
+	} else if (rc >= sizeof(clientid_name)) {
+		return -EINVAL;
+	}
+
 	clientid.co_ownerid.co_ownerid_len = strlen(clientid_name);
 	clientid.co_ownerid.co_ownerid_val = clientid_name;
 
@@ -1085,6 +1103,7 @@ static void *pxy_clientid_renewer(void *arg)
 
 	SetNameFunction("pxy_clientid_renewer");
 
+	rcu_register_thread();
 	while (!pxy_exp->rpc.close_thread) {
 		clientid4 newcid = 0;
 		sequenceid4 newseqid = 0;
@@ -1132,13 +1151,13 @@ static void *pxy_clientid_renewer(void *arg)
 
 		/* early stop test */
 		if (pxy_exp->rpc.close_thread)
-			return NULL;
+			break;
 
 		/* We've either failed to renew or rpc socket has been
 		 * reconnected and we need new clientid or sessionid. */
 		if (pxy_rpc_need_sock(pxy_exp))
 			/* early stop test */
-			return NULL;
+			break;
 
 		/* We need a new session_id */
 		if (!clientid_needed) {
@@ -1178,7 +1197,7 @@ static void *pxy_clientid_renewer(void *arg)
 			PTHREAD_MUTEX_unlock(&pxy_exp->rpc.pxy_clientid_mutex);
 		}
 	}
-
+	rcu_unregister_thread();
 	return NULL;
 }
 
@@ -1249,10 +1268,16 @@ int pxy_init_rpc(struct pxy_export *pxy_exp)
 	if (pxy_exp->rpc.rpc_xid == 0)
 		pxy_exp->rpc.rpc_xid = getpid() ^ time(NULL);
 	PTHREAD_MUTEX_unlock(&pxy_exp->rpc.listlock);
+
 	if (gethostname(pxy_exp->rpc.pxy_hostname,
-			sizeof(pxy_exp->rpc.pxy_hostname)))
-		strlcpy(pxy_exp->rpc.pxy_hostname, "NFS-GANESHA/Proxy",
-			sizeof(pxy_exp->rpc.pxy_hostname));
+			sizeof(pxy_exp->rpc.pxy_hostname))) {
+		if (strlcpy(pxy_exp->rpc.pxy_hostname, "NFS-GANESHA/Proxy",
+			    sizeof(pxy_exp->rpc.pxy_hostname))
+		    >= sizeof(pxy_exp->rpc.pxy_hostname)) {
+			free_io_contexts(pxy_exp);
+			return -1;
+		}
+	}
 
 	for (i = NB_RPC_SLOT-1; i >= 0; i--) {
 		struct pxy_rpc_io_context *c =
@@ -1800,15 +1825,12 @@ static fsal_status_t pxy_do_readdir(struct pxy_obj_handle *ph,
 
 	for (e4 = rdok->reply.entries; e4; e4 = e4->nextentry) {
 		struct attrlist attrs;
-		char name[MAXNAMLEN + 1];
 		struct fsal_obj_handle *handle;
 		enum fsal_dir_result cb_rc;
 
 		/* UTF8 name does not include trailing 0 */
-		if (e4->name.utf8string_len > sizeof(name) - 1)
+		if (e4->name.utf8string_len > MAXNAMLEN)
 			return fsalstat(ERR_FSAL_SERVERFAULT, E2BIG);
-		memcpy(name, e4->name.utf8string_val, e4->name.utf8string_len);
-		name[e4->name.utf8string_len] = '\0';
 
 		if (nfs4_Fattr_To_FSAL_attr(&attrs, &e4->attrs, NULL))
 			return fsalstat(ERR_FSAL_FAULT, 0);
@@ -1829,11 +1851,13 @@ static fsal_status_t pxy_do_readdir(struct pxy_obj_handle *ph,
 		 *             avoid over large READDIR response.
 		 */
 		st = pxy_lookup_impl(&ph->obj, op_ctx->fsal_export,
-				     op_ctx->creds, name, &handle, NULL);
+				     op_ctx->creds, e4->name.utf8string_val,
+				     &handle, NULL);
 		if (FSAL_IS_ERROR(st))
 			break;
 
-		cb_rc = cb(name, handle, &attrs, cbarg, e4->cookie);
+		cb_rc = cb(e4->name.utf8string_val, handle, &attrs,
+			   cbarg, e4->cookie);
 
 		fsal_release_attrs(&attrs);
 
@@ -2227,7 +2251,7 @@ static fsal_status_t pxy_open2(struct fsal_obj_handle *obj_hdl,
 	fsal_status_t st; /* return code of fsal call */
 	char padfilehandle[NFS4_FHSIZE]; /* gotten FH */
 	char owner_val[128];
-	unsigned int owner_len = 0;
+	int owner_len = 0;
 	uint32_t share_access = 0;
 	uint32_t share_deny = 0;
 	openflag4 openhow;
@@ -2302,10 +2326,22 @@ static fsal_status_t pxy_open2(struct fsal_obj_handle *obj_hdl,
 		}
 
 		/* owner */
-		snprintf(owner_val, sizeof(owner_val),
-			 "GANESHA/PROXY: pid=%u %" PRIu64, getpid(),
-			 atomic_inc_uint64_t(&fcnt));
-		owner_len = strnlen(owner_val, sizeof(owner_val));
+		owner_len = snprintf(owner_val, sizeof(owner_val),
+				     "GANESHA/PROXY: pid=%u %" PRIu64, getpid(),
+				     atomic_inc_uint64_t(&fcnt));
+
+		if (owner_len < 0) {
+			int error = errno;
+
+			LogCrit(COMPONENT_FSAL,
+				"Unexpected return from snprintf %d error %s (%d)",
+				owner_len, strerror(error), error);
+			return posix2fsal_status(error);
+		} else if (owner_len >= sizeof(owner_val)) {
+			LogMajor(COMPONENT_FSAL, "Owner length too long");
+			return posix2fsal_status(EINVAL);
+		}
+
 		/* inattrs and openhow */
 		st = fill_openhow_OPEN4args(&openhow, inattrs, createmode,
 					    verifier, &setattr_needed, name,
@@ -2522,13 +2558,6 @@ static void pxy_write2(struct fsal_obj_handle *obj_hdl,
 	stable_how4 stable_how;
 	size_t buffer_size = write_arg->iov[0].iov_len;
 
-	if (write_arg->info != NULL) {
-		/* Currently we don't support WRITE_PLUS */
-		done_cb(obj_hdl, fsalstat(ERR_FSAL_NOTSUPP, 0), write_arg,
-			caller_arg);
-		return;
-	}
-
 	ph = container_of(obj_hdl, struct pxy_obj_handle, obj);
 
 	/* check max write size */
