@@ -1,7 +1,7 @@
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
- * Copyright 2015-2018 Red Hat, Inc. and/or its affiliates.
+ * Copyright 2015-2019 Red Hat, Inc. and/or its affiliates.
  * Author: Daniel Gryniewicz <dang@redhat.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -191,6 +191,9 @@ static void mdcache_exp_release(struct fsal_export *exp_hdl)
 		fsal_hdl->name, op_ctx->ctx_export->export_id,
 		export_path(op_ctx->ctx_export));
 
+	/* Stop the dirmap thread */
+	dirmap_lru_stop(exp);
+
 	/* Release the sub_export */
 	subcall_shutdown_raw(exp,
 		sub_export->exp_ops.release(sub_export)
@@ -454,6 +457,27 @@ static uint32_t mdcache_fs_umask(struct fsal_export *exp_hdl)
 	return result;
 }
 
+/**
+ * @brief Get the configured expiration time for parent handle
+ *
+ * MDCACHE only caches metadata, so it imposes no restrictions itself.
+ *
+ * @param[in] exp_hdl   Export to query
+ * @return Expiry time for parent handle
+ */
+static int32_t mdcache_fs_expiretimeparent(struct fsal_export *exp_hdl)
+{
+	struct mdcache_fsal_export *exp = mdc_export(exp_hdl);
+	struct fsal_export *sub_export = exp->mfe_exp.sub_export;
+	int32_t result;
+
+	subcall_raw(exp,
+		result = sub_export->exp_ops.fs_expiretimeparent(sub_export)
+	);
+
+	return result;
+}
+
 /**
  * @brief Check quota on a file
  *
@@ -843,6 +867,7 @@ void mdcache_export_ops_init(struct export_ops *ops)
 	ops->alloc_state = mdcache_alloc_state;
 	ops->free_state = mdcache_free_state;
 	ops->is_superuser = mdcache_is_superuser;
+	ops->fs_expiretimeparent = mdcache_fs_expiretimeparent;
 }
 
 #if 0
