@@ -33,6 +33,7 @@
  */
 
 #include "config.h"
+#include "nfs_init.h"
 #include <sys/types.h>
 #include <sys/param.h>
 #include <time.h>
@@ -652,7 +653,8 @@ lru_reap_impl(enum lru_q_id qid)
 		entry = container_of(lru, mdcache_entry_t, lru);
 #ifdef USE_LTTNG
 	tracepoint(mdcache, mdc_lru_ref,
-		   __func__, __LINE__, entry, entry->sub_handle, refcnt);
+		   __func__, __LINE__, &entry->obj_handle, entry->sub_handle,
+		   refcnt);
 #endif
 		QUNLOCK(qlane);
 
@@ -679,7 +681,7 @@ lru_reap_impl(enum lru_q_id qid)
 
 #ifdef USE_LTTNG
 				tracepoint(mdcache, mdc_lru_reap, __func__,
-					   __LINE__, entry,
+					   __LINE__, &entry->obj_handle,
 					   entry->lru.refcnt);
 #endif
 				LRU_DQ_SAFE(lru, q);
@@ -748,7 +750,6 @@ lru_try_reap_entry(void)
  *
  * @param[in] qid        Queue to reap
  * @param[in] parent     The directory we desire a chunk for
- * @param[in] prev_chunk If non-NULL, the previous chunk in this directory
  *
  * @return Available chunk if found, NULL otherwise
  */
@@ -756,8 +757,7 @@ lru_try_reap_entry(void)
 static uint32_t chunk_reap_lane;
 
 static inline mdcache_lru_t *
-lru_reap_chunk_impl(enum lru_q_id qid, mdcache_entry_t *parent,
-		    struct dir_chunk *prev_chunk)
+lru_reap_chunk_impl(enum lru_q_id qid, mdcache_entry_t *parent)
 {
 	uint32_t lane;
 	struct lru_q_lane *qlane;
@@ -766,6 +766,7 @@ lru_reap_chunk_impl(enum lru_q_id qid, mdcache_entry_t *parent,
 	mdcache_entry_t *entry;
 	struct dir_chunk *chunk;
 	int ix;
+	int32_t refcnt;
 
 	lane = LRU_NEXT(chunk_reap_lane);
 
@@ -784,6 +785,8 @@ lru_reap_chunk_impl(enum lru_q_id qid, mdcache_entry_t *parent,
 			continue;
 		}
 
+		refcnt = atomic_fetch_int32_t(&lru->refcnt);
+		assert(refcnt);
 		/* Get the chunk and parent entry that owns the chunk, all of
 		 * this is valid because we hold the QLANE lock, the chunk was
 		 * in the LRU, and thus the chunk is not yet being destroyed,
@@ -792,8 +795,8 @@ lru_reap_chunk_impl(enum lru_q_id qid, mdcache_entry_t *parent,
 		chunk = container_of(lru, struct dir_chunk, chunk_lru);
 		entry = chunk->parent;
 
-		if (chunk == prev_chunk) {
-			/* We can't reap prev_chunk. */
+		if (refcnt != (LRU_SENTINEL_REFCOUNT)) {
+			/* We can't reap a chunk with a ref */
 			QUNLOCK(qlane);
 			continue;
 		}
@@ -817,17 +820,10 @@ lru_reap_chunk_impl(enum lru_q_id qid, mdcache_entry_t *parent,
 			CHUNK_LRU_DQ_SAFE(lru, lq);
 			chunk->chunk_lru.qid = LRU_ENTRY_NONE;
 
-			/* Drop the lane lock, we can now safely clean up the
-			 * chunk. We hold the content_lock on the parent of
-			 * the chunk (even if the chunk belonged to the
-			 * directory a new chunk is requested for).
-			 */
-			QUNLOCK(qlane);
-
 #ifdef USE_LTTNG
 				tracepoint(mdcache, mdc_lru_reap_chunk,
 					   __func__, __LINE__,
-					   entry, chunk);
+					   &entry->obj_handle, chunk);
 #endif
 
 			/* Clean the chunk out and indicate the directory is no
@@ -837,11 +833,6 @@ lru_reap_chunk_impl(enum lru_q_id qid, mdcache_entry_t *parent,
 			atomic_clear_uint32_t_bits(&entry->mde_flags,
 						   MDCACHE_DIR_POPULATED);
 
-			/* Clean out the fields not touched by the cleanup. */
-			chunk->parent = NULL;
-			chunk->next_ck = 0;
-			chunk->num_entries = 0;
-
 			if (entry != parent) {
 				/* And now we're done with the parent of the
 				 * chunk if it wasn't the directory we are
@@ -850,6 +841,7 @@ lru_reap_chunk_impl(enum lru_q_id qid, mdcache_entry_t *parent,
 				PTHREAD_RWLOCK_unlock(&entry->content_lock);
 				mdcache_put(entry);
 			}
+			QUNLOCK(qlane);
 			return lru;
 		}
 
@@ -858,7 +850,7 @@ lru_reap_chunk_impl(enum lru_q_id qid, mdcache_entry_t *parent,
 		 * eligible for reaping. Try the next lane...
 		 */
 		QUNLOCK(qlane);
-	}			/* foreach lane */
+	}	/* foreach lane */
 
 	/* ! reclaimable */
 	return NULL;
@@ -875,20 +867,26 @@ lru_reap_chunk_impl(enum lru_q_id qid, mdcache_entry_t *parent,
  *
  * @param[in] parent     The parent directory we desire a chunk for
  * @param[in] prev_chunk If non-NULL, the previous chunk in this directory
+ * @param[in] whence	 If @a prev_chunk is NULL, the starting whence of chunk
  *
- * @return reused or allocated chunk
+ * @return reused or allocated chunk with a ref taken for the caller
  */
 struct dir_chunk *mdcache_get_chunk(mdcache_entry_t *parent,
-				    struct dir_chunk *prev_chunk)
+				    struct dir_chunk *prev_chunk,
+				    fsal_cookie_t whence)
 {
 	mdcache_lru_t *lru = NULL;
 	struct dir_chunk *chunk = NULL;
 
+	/* Get a ref on prev_chunk, so that it's not reaped */
+	if (prev_chunk)
+		mdcache_lru_ref_chunk(prev_chunk);
+
 	if (lru_state.chunks_used >= lru_state.chunks_hiwat) {
-		lru = lru_reap_chunk_impl(LRU_ENTRY_L2, parent, prev_chunk);
+		lru = lru_reap_chunk_impl(LRU_ENTRY_L2, parent);
 		if (!lru)
 			lru = lru_reap_chunk_impl(
-					LRU_ENTRY_L1, parent, prev_chunk);
+					LRU_ENTRY_L1, parent);
 	}
 
 	if (lru) {
@@ -909,17 +907,20 @@ struct dir_chunk *mdcache_get_chunk(mdcache_entry_t *parent,
 
 	/* Set the chunk's parent and insert */
 	chunk->parent = parent;
+	glist_add_tail(&chunk->parent->fsobj.fsdir.chunks, &chunk->chunks);
 	if (prev_chunk) {
-		glist_add(&prev_chunk->chunks, &chunk->chunks);
+		chunk->reload_ck = glist_last_entry(&prev_chunk->dirents,
+						    mdcache_dir_entry_t,
+						    chunk_list)->ck;
+		/* unref prev_chunk as we had got a ref on prev_chunk
+		 * at the beginning of this function
+		 */
+		mdcache_lru_unref_chunk(prev_chunk);
 	} else {
-		glist_add(&chunk->parent->fsobj.fsdir.chunks, &chunk->chunks);
+		chunk->reload_ck = whence;
 	}
 
-	/* Chunk refcnt is not used (chunks are always protected by content_lock
-	 * and outside of LRU operations are not found other than while holding
-	 * the content lock).
-	 */
-	chunk->chunk_lru.refcnt = 0;
+	chunk->chunk_lru.refcnt = 2;
 	chunk->chunk_lru.cf = 0;
 	chunk->chunk_lru.lane = lru_lane_of(chunk);
 
@@ -1131,7 +1132,8 @@ static inline size_t lru_run_lane(size_t lane, uint64_t *const totalclosed)
 		refcnt = atomic_inc_int32_t(&entry->lru.refcnt);
 #ifdef USE_LTTNG
 	tracepoint(mdcache, mdc_lru_ref,
-		   __func__, __LINE__, entry, entry->sub_handle, refcnt);
+		   __func__, __LINE__, &entry->obj_handle, entry->sub_handle,
+		   refcnt);
 #endif
 
 		init_root_op_context(&ctx, export, export->fsal_export, 0, 0,
@@ -1262,6 +1264,13 @@ lru_run(struct fridgethr_context *ctx)
 	/* The current count (after reaping) of open FDs */
 	size_t currentopen = 0;
 	time_t new_thread_wait;
+	static bool first_time = TRUE;
+
+	if (first_time) {
+		/* Wait for NFS server to properly initialize */
+		nfs_init_wait();
+		first_time = FALSE;
+	}
 
 	SetNameFunction("cache_lru");
 
@@ -1358,17 +1367,16 @@ lru_run(struct fridgethr_context *ctx)
 			 && (totalwork < lru_state.biggest_window));
 
 		currentopen = atomic_fetch_size_t(&open_fd_count);
-		if (extremis
-		    && ((currentopen > formeropen)
-			|| (formeropen - currentopen <
-			    (((formeropen -
-			       lru_state.fds_hiwat) *
-			      mdcache_param.required_progress) /
-			     100)))) {
+		if (extremis &&
+		    ((currentopen > formeropen)
+		     || (formeropen - currentopen <
+			 (((formeropen - lru_state.fds_hiwat) *
+			      mdcache_param.required_progress) / 100)))) {
 			if (++lru_state.futility ==
 			    mdcache_param.futility_count) {
 				LogWarn(COMPONENT_CACHE_INODE_LRU,
-					"Futility count exceeded.  Client load is opening FDs faster than the LRU thread can close them.");
+					"Futility count exceeded.  Client load is opening FDs faster than the LRU thread can close them. current_open = %zu, former_open = %zu",
+					currentopen, formeropen);
 			}
 		}
 	}
@@ -1400,6 +1408,10 @@ lru_run(struct fridgethr_context *ctx)
 	if (new_thread_wait < mdcache_param.lru_run_interval / 10)
 		new_thread_wait = mdcache_param.lru_run_interval / 10;
 
+	/* if new_thread_wait is 0, lru_run will not be scheduled */
+	if (new_thread_wait == 0)
+		new_thread_wait = 1;
+
 	fridgethr_setwait(ctx, new_thread_wait);
 
 	LogDebug(COMPONENT_CACHE_INODE_LRU,
@@ -1435,6 +1447,8 @@ static inline size_t chunk_lru_run_lane(size_t lane)
 	mdcache_lru_t *lru = NULL;
 	/* Current queue lane */
 	struct lru_q_lane *qlane = &CHUNK_LRU[lane];
+	struct dir_chunk *chunk;
+	uint32_t refcnt;
 
 	q = &qlane->L1;
 
@@ -1458,14 +1472,21 @@ static inline size_t chunk_lru_run_lane(size_t lane)
 
 		lru = glist_entry(qlane->iter.glist, mdcache_lru_t, q);
 
+		chunk = container_of(lru, struct dir_chunk, chunk_lru);
+
+		refcnt = atomic_fetch_int32_t(&chunk->chunk_lru.refcnt);
+		assert(refcnt);
+		if (unlikely(refcnt > LRU_SENTINEL_REFCOUNT)) {
+			workdone++;
+			continue;
+		}
+
 		/* Move lru object to MRU of L2 */
 		q = &qlane->L1;
 		CHUNK_LRU_DQ_SAFE(lru, q);
 		lru->qid = LRU_ENTRY_L2;
 		q = &qlane->L2;
 		lru_insert(lru, q, LRU_MRU);
-
-		++workdone;
 	} /* for_each_safe lru */
 
 next_lane:
@@ -1502,6 +1523,13 @@ static void chunk_lru_run(struct fridgethr_context *ctx)
 	time_t new_thread_wait;
 	/* Total work done (number of chunks demoted) across all lanes. */
 	size_t totalwork = 0;
+	static bool first_time = true;
+
+	if (first_time) {
+		/* Wait for NFS server to properly initialize */
+		nfs_init_wait();
+		first_time = false;
+	}
 
 	SetNameFunction("chunk_lru");
 
@@ -1526,6 +1554,10 @@ static void chunk_lru_run(struct fridgethr_context *ctx)
 	if (new_thread_wait < mdcache_param.lru_run_interval / 10)
 		new_thread_wait = mdcache_param.lru_run_interval / 10;
 
+	/* if new_thread_wait is 0, chunk_lru_run will not be scheduled */
+	if (new_thread_wait == 0)
+		new_thread_wait = 1;
+
 	fridgethr_setwait(ctx, new_thread_wait);
 
 	LogDebug(COMPONENT_CACHE_INODE_LRU,
@@ -1533,6 +1565,32 @@ static void chunk_lru_run(struct fridgethr_context *ctx)
 		 ((uint64_t) new_thread_wait), totalwork);
 }
 
+/**
+ * @brief Remove reapable entries until we are below the high-water mark
+ *
+ * If something refs a lot of entries at the same time, this can put the number
+ * of entries above the high water mark.  They will slowly fall, as entries are
+ * actually freed, but this may take a very long time.
+ *
+ * This is a big hammer, that will clean up anything it can until either it
+ * can't anymore, or we're back below the high water mark.
+ *
+ * @param[in] parm     Parameter description
+ * @return Return description
+ */
+void lru_cleanup_entries(void)
+{
+	mdcache_lru_t *lru;
+	mdcache_entry_t *entry = NULL;
+
+	while ((lru = lru_try_reap_entry())) {
+		if (lru) {
+			entry = container_of(lru, mdcache_entry_t, lru);
+			mdcache_lru_unref(entry);
+		}
+	}
+}
+
 void init_fds_limit(void)
 {
 	int code = 0;
@@ -1782,7 +1840,8 @@ mdcache_entry_t *mdcache_lru_get(struct fsal_obj_handle *sub_handle)
 
 #ifdef USE_LTTNG
 	tracepoint(mdcache, mdc_lru_get,
-		  __func__, __LINE__, nentry, sub_handle, nentry->lru.refcnt);
+		  __func__, __LINE__, &nentry->obj_handle, sub_handle,
+		  nentry->lru.refcnt);
 #endif
 	return nentry;
 }
@@ -1843,7 +1902,7 @@ _mdcache_lru_ref(mdcache_entry_t *entry, uint32_t flags, const char *func,
 
 #ifdef USE_LTTNG
 	tracepoint(mdcache, mdc_lru_ref,
-		   func, line, entry, entry->sub_handle, refcnt);
+		   func, line, &entry->obj_handle, entry->sub_handle, refcnt);
 #endif
 
 	/* adjust LRU on initial refs */
@@ -1925,7 +1984,7 @@ _mdcache_lru_unref(mdcache_entry_t *entry, uint32_t flags, const char *func,
 
 #ifdef USE_LTTNG
 	tracepoint(mdcache, mdc_lru_unref,
-		   func, line, entry, entry->sub_handle, refcnt);
+		   func, line, &entry->obj_handle, entry->sub_handle, refcnt);
 #endif
 
 	if (unlikely(refcnt == 0)) {
@@ -1964,18 +2023,16 @@ _mdcache_lru_unref(mdcache_entry_t *entry, uint32_t flags, const char *func,
 /**
  * @brief Remove a chunk from LRU, and clean it
  *
+ * QLane must be locked
+ *
  * @param[in] chunk  The chunk to be removed from LRU
  */
-void lru_clean_chunk(struct dir_chunk *chunk)
+static void lru_clean_chunk(struct dir_chunk *chunk)
 {
-	uint32_t lane = chunk->chunk_lru.lane;
-	struct lru_q_lane *qlane = &CHUNK_LRU[lane];
 	struct lru_q *lq;
 
 	LogFullDebug(COMPONENT_CACHE_INODE, "Removing chunk %p", chunk);
 
-	QLOCK(qlane);
-
 	/* Remove chunk and mark it as dead. */
 	lq = chunk_lru_queue_of(chunk);
 
@@ -1984,41 +2041,45 @@ void lru_clean_chunk(struct dir_chunk *chunk)
 		CHUNK_LRU_DQ_SAFE(&chunk->chunk_lru, lq);
 	}
 
-	QUNLOCK(qlane);
-
 	(void) atomic_dec_int64_t(&lru_state.chunks_used);
 
 	/* Then do the actual cleaning work. */
 	mdcache_clean_dirent_chunk(chunk);
 }
 
-/**
- * @brief Remove a chunk from LRU, clean it, and free it.
- *
- * @param[in] chunk  The chunk to be removed from LRU
- */
-void lru_remove_chunk(struct dir_chunk *chunk)
+void _mdcache_lru_ref_chunk(struct dir_chunk *chunk, const char *func, int line)
 {
-	lru_clean_chunk(chunk);
-
-	/* And now we can free the chunk. */
-	LogFullDebug(COMPONENT_CACHE_INODE, "Freeing chunk %p", chunk);
-	gsh_free(chunk);
+	atomic_inc_int32_t(&chunk->chunk_lru.refcnt);
 }
 
 /**
- * @brief Remove a chunk from LRU, and clean it
- *
- * @param[in] chunk  The chunk to be removed from LRU
+ * @brief Unref a dirent chunk
+ * Should be called with content_lock held in write mode.
+ * @param [in] chunk	The chunk to unref
  */
-void lru_reuse_chunk(mdcache_entry_t *parent, struct dir_chunk *chunk)
+void _mdcache_lru_unref_chunk(struct dir_chunk *chunk, const char *func,
+			      int line)
 {
-	lru_clean_chunk(chunk);
-	chunk->parent = parent;
-	chunk->chunk_lru.refcnt = 0;
-	chunk->chunk_lru.cf = 0;
-	chunk->chunk_lru.lane = lru_lane_of(chunk);
-	lru_insert_chunk(chunk, &CHUNK_LRU[chunk->chunk_lru.lane].L2, LRU_MRU);
+	int refcnt;
+	uint32_t lane;
+	struct lru_q_lane *qlane;
+
+	if (!chunk)
+		return;
+
+	lane = chunk->chunk_lru.lane;
+	qlane = &CHUNK_LRU[lane];
+	QLOCK(qlane);
+
+	refcnt = atomic_dec_int32_t(&chunk->chunk_lru.refcnt);
+	if (refcnt == 0) {
+		lru_clean_chunk(chunk);
+
+		/* And now we can free the chunk. */
+		LogFullDebug(COMPONENT_CACHE_INODE, "Freeing chunk %p", chunk);
+		gsh_free(chunk);
+	}
+	QUNLOCK(qlane);
 }
 
 /**
@@ -2067,24 +2128,30 @@ void lru_bump_chunk(struct dir_chunk *chunk)
  */
 bool mdcache_lru_fds_available(void)
 {
-	if (atomic_fetch_size_t(&open_fd_count) >= lru_state.fds_hard_limit) {
+	size_t open_fds = atomic_fetch_size_t(&open_fd_count);
+
+	if (open_fds >= lru_state.fds_hard_limit) {
 		LogAtLevel(COMPONENT_CACHE_INODE_LRU,
 			   atomic_fetch_uint32_t(&lru_state.fd_state)
 								!= FD_LIMIT
 				? NIV_CRIT
 				: NIV_DEBUG,
-			   "FD Hard Limit Exceeded, waking LRU thread.");
+			   "FD Hard Limit (%"PRIu32
+			   ") Exceeded (open_fd_count = %zu), waking LRU thread.",
+			   lru_state.fds_hard_limit, open_fds);
 		atomic_store_uint32_t(&lru_state.fd_state, FD_LIMIT);
 		fridgethr_wake(lru_fridge);
 		return false;
 	}
 
-	if (atomic_fetch_size_t(&open_fd_count) >= lru_state.fds_hiwat) {
+	if (open_fds >= lru_state.fds_hiwat) {
 		LogAtLevel(COMPONENT_CACHE_INODE_LRU,
 			   atomic_fetch_uint32_t(&lru_state.fd_state) == FD_LOW
 				? NIV_INFO
 				: NIV_DEBUG,
-			   "FDs above high water mark, waking LRU thread.");
+			   "FDs above high water mark (%"PRIu32
+			   ", open_fd_count = %zu), waking LRU thread.",
+			   lru_state.fds_hiwat, open_fds);
 		atomic_store_uint32_t(&lru_state.fd_state, FD_HIGH);
 		fridgethr_wake(lru_fridge);
 	}
@@ -2092,4 +2159,237 @@ bool mdcache_lru_fds_available(void)
 	return true;
 }
 
+static inline void mdc_lru_dirmap_add(struct mdcache_fsal_export *exp,
+				      mdcache_dmap_entry_t *dmap)
+{
+	avltree_insert(&dmap->node, &exp->dirent_map.map);
+	/* MRU is the tail; Mdd to MRU of list */
+	glist_add_tail(&exp->dirent_map.lru, &dmap->lru_entry);
+	exp->dirent_map.count++;
+}
+
+static inline void mdc_lru_dirmap_del(struct mdcache_fsal_export *exp,
+				      mdcache_dmap_entry_t *dmap)
+{
+	glist_del(&dmap->lru_entry);
+	avltree_remove(&dmap->node, &exp->dirent_map.map);
+	exp->dirent_map.count--;
+}
+
+/**
+ * @brief Add a dirent to the dirmap
+ *
+ * Add this dirent to the dirmap.  The dirmap is a mapping of cookies to names
+ * that allows whence-is-name to restart where it left off if the chunk was
+ * reaped, instead of reloading the whole directory to find the cookie.
+ *
+ * @param[in] dirent	Dirent to add
+ */
+void mdc_lru_map_dirent(mdcache_dir_entry_t *dirent)
+{
+	struct mdcache_fsal_export *exp = mdc_cur_export();
+	mdcache_dmap_entry_t key, *dmap;
+	struct avltree_node *node;
+
+	PTHREAD_MUTEX_lock(&exp->dirent_map.mtx);
+
+	key.ck = dirent->ck;
+	node = avltree_lookup(&key.node, &exp->dirent_map.map);
+	if (node) {
+		LogFullDebug(COMPONENT_NFS_READDIR, "Already map for %s -> %lx",
+			     dirent->name, dirent->ck);
+		/* Move to MRU */
+		dmap = avltree_container_of(node, mdcache_dmap_entry_t, node);
+		now(&dmap->timestamp);
+		glist_del(&dmap->lru_entry);
+		glist_add_tail(&exp->dirent_map.lru, &dmap->lru_entry);
+		PTHREAD_MUTEX_unlock(&exp->dirent_map.mtx);
+		return;
+	}
+
+	if (exp->dirent_map.count > mdcache_param.dirmap_hwmark) {
+		/* LRU end is the head; grab the LRU entry */
+		dmap = glist_first_entry(&exp->dirent_map.lru,
+					 mdcache_dmap_entry_t, lru_entry);
+		mdc_lru_dirmap_del(exp, dmap);
+		/* Free name */
+		gsh_free(dmap->name);
+	} else {
+		dmap = gsh_malloc(sizeof(*dmap));
+	}
+
+	dmap->ck = dirent->ck;
+	dmap->name = gsh_strdup(dirent->name);
+	now(&dmap->timestamp);
+	LogFullDebug(COMPONENT_NFS_READDIR, "Mapping %s -> %lx %p:%d",
+		     dmap->name, dmap->ck, exp, exp->dirent_map.count);
+
+	mdc_lru_dirmap_add(exp, dmap);
+
+	PTHREAD_MUTEX_unlock(&exp->dirent_map.mtx);
+}
+
+/**
+ * @brief Look up and remove an entry from the dirmap
+ *
+ * This looks up the cookie in the dirmap, and returns the associated name, if
+ * it's in the cache.  The entry is removed from the cache and freed, and the
+ * name is returned.
+ *
+ * @note the returned name must be freed by the caller
+ *
+ * @param[in] ck	Cookie to look up
+ * @return Name, if found, or NULL otherwise
+ */
+fsal_cookie_t *mdc_lru_unmap_dirent(uint64_t ck)
+{
+	struct mdcache_fsal_export *exp = mdc_cur_export();
+	struct avltree_node *node;
+	mdcache_dmap_entry_t key, *dmap;
+	char *name;
+
+	PTHREAD_MUTEX_lock(&exp->dirent_map.mtx);
+
+	key.ck = ck;
+	node = avltree_lookup(&key.node, &exp->dirent_map.map);
+	if (!node) {
+		LogFullDebug(COMPONENT_NFS_READDIR, "No map for %lx", ck);
+		PTHREAD_MUTEX_unlock(&exp->dirent_map.mtx);
+		return NULL;
+	}
+
+	dmap = avltree_container_of(node, mdcache_dmap_entry_t, node);
+	mdc_lru_dirmap_del(exp, dmap);
+
+	PTHREAD_MUTEX_unlock(&exp->dirent_map.mtx);
+
+	name = dmap->name;
+
+	LogFullDebug(COMPONENT_NFS_READDIR, "Unmapping %s -> %lx", dmap->name,
+		     dmap->ck);
+
+	/* Don't free name, we're passing it back to the caller */
+	gsh_free(dmap);
+
+	return (fsal_cookie_t *)name;
+}
+
+#define DIRMAP_MAX_PER_SCAN 1000
+#define DIRMAP_KEEP_NS (60 * NS_PER_SEC)
+
+static void dirmap_lru_run(struct fridgethr_context *ctx)
+{
+	struct mdcache_fsal_export *exp = ctx->arg;
+	mdcache_dmap_entry_t *cur, *next;
+	int i;
+	struct timespec curtime;
+	nsecs_elapsed_t age;
+	static bool first_time = true;
+
+	/* XXX dang this needs to be here or this will hijack another thread,
+	 * causing that one to never run again. */
+	if (first_time) {
+		/* Wait for NFS server to properly initialize */
+		nfs_init_wait();
+		first_time = false;
+	}
+
+	PTHREAD_MUTEX_lock(&exp->dirent_map.mtx);
+
+	now(&curtime);
+
+	cur = glist_last_entry(&exp->dirent_map.lru, mdcache_dmap_entry_t,
+			       lru_entry);
+	for (i = 0; i < DIRMAP_MAX_PER_SCAN && cur != NULL; ++i) {
+		next = glist_prev_entry(&exp->dirent_map.lru,
+					mdcache_dmap_entry_t,
+					lru_entry, &cur->lru_entry);
+		age = timespec_diff(&cur->timestamp, &curtime);
+		if (age < DIRMAP_KEEP_NS) {
+			/* LRU is in timestamp order; done */
+			goto out;
+		}
+		mdc_lru_dirmap_del(exp, cur);
+		gsh_free(cur->name);
+		gsh_free(cur);
+		cur = next;
+	}
+
+out:
+	PTHREAD_MUTEX_unlock(&exp->dirent_map.mtx);
+	fridgethr_setwait(ctx, mdcache_param.lru_run_interval);
+}
+
+
+fsal_status_t dirmap_lru_init(struct mdcache_fsal_export *exp)
+{
+	struct fridgethr_params frp;
+	int rc;
+
+	if (!exp->mfe_exp.exp_ops.fs_supports(&exp->mfe_exp,
+					      fso_whence_is_name)) {
+		LogDebug(COMPONENT_NFS_READDIR, "Skipping dirmap %s",
+			 exp->name);
+		return fsalstat(0, 0);
+	}
+
+	avltree_init(&exp->dirent_map.map, avl_dmap_ck_cmpf, 0 /* flags */);
+	glist_init(&exp->dirent_map.lru);
+	rc = pthread_mutex_init(&exp->dirent_map.mtx, NULL);
+	if (rc != 0) {
+		return posix2fsal_status(rc);
+	}
+
+	memset(&frp, 0, sizeof(struct fridgethr_params));
+	frp.thr_max = 1;
+	frp.thr_min = 1;
+	frp.thread_delay = mdcache_param.lru_run_interval;
+	frp.flavor = fridgethr_flavor_looper;
+
+	rc = fridgethr_init(&exp->dirmap_fridge, exp->name, &frp);
+	if (rc != 0) {
+		LogMajor(COMPONENT_NFS_READDIR,
+			 "Unable to initialize %s dirmap fridge, error code %d.",
+			 exp->name, rc);
+		return posix2fsal_status(rc);
+	}
+
+	rc = fridgethr_submit(exp->dirmap_fridge, dirmap_lru_run, exp);
+	if (rc != 0) {
+		LogMajor(COMPONENT_NFS_READDIR,
+			 "Unable to start %s dirmap thread, error code %d.",
+			 exp->name, rc);
+		return posix2fsal_status(rc);
+	}
+
+	LogDebug(COMPONENT_NFS_READDIR, "started dirmap %s", exp->name);
+
+	return fsalstat(0, 0);
+}
+
+void dirmap_lru_stop(struct mdcache_fsal_export *exp)
+{
+	if (!exp->dirmap_fridge) {
+		/* Wasn't running */
+		return;
+	}
+
+	int rc = fridgethr_sync_command(exp->dirmap_fridge,
+					fridgethr_comm_stop,
+					10);
+
+	if (rc == ETIMEDOUT) {
+		LogDebug(COMPONENT_NFS_READDIR,
+			 "Shutdown timed out, cancelling threads.");
+		fridgethr_cancel(exp->dirmap_fridge);
+	} else if (rc != 0) {
+		LogMajor(COMPONENT_NFS_READDIR,
+			 "Failed shutting down LRU thread: %d", rc);
+	}
+
+	fridgethr_destroy(exp->dirmap_fridge);
+
+	LogDebug(COMPONENT_NFS_READDIR, "stopped dirmap %s", exp->name);
+}
+
 /** @} */
