@@ -196,12 +196,22 @@ static inline void mdcache_put(mdcache_entry_t *entry)
 	mdcache_lru_unref(entry);
 }
 
-void lru_reuse_chunk(mdcache_entry_t *parent, struct dir_chunk *chunk);
-void lru_remove_chunk(struct dir_chunk *chunk);
+#define mdcache_lru_ref_chunk(chunk) \
+	_mdcache_lru_ref_chunk(chunk, __func__, __LINE__)
+void _mdcache_lru_ref_chunk(struct dir_chunk *chunk, const char *func,
+			    int line);
+#define mdcache_lru_unref_chunk(chunk) \
+	_mdcache_lru_unref_chunk(chunk, __func__, __LINE__)
+void _mdcache_lru_unref_chunk(struct dir_chunk *chunk, const char *func,
+			      int line);
 struct dir_chunk *mdcache_get_chunk(mdcache_entry_t *parent,
 				    struct dir_chunk *prev_chunk,
 				    fsal_cookie_t whence);
 void lru_bump_chunk(struct dir_chunk *chunk);
 
+void mdc_lru_map_dirent(mdcache_dir_entry_t *dirent);
+fsal_cookie_t *mdc_lru_unmap_dirent(uint64_t ck);
+fsal_status_t dirmap_lru_init(struct mdcache_fsal_export *exp);
+void dirmap_lru_stop(struct mdcache_fsal_export *exp);
 #endif				/* MDCACHE_LRU_H */
 /** @} */
