@@ -1,7 +1,7 @@
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
- * Copyright 2015-2017 Red Hat, Inc. and/or its affiliates.
+ * Copyright 2015-2019 Red Hat, Inc. and/or its affiliates.
  * Author: Daniel Gryniewicz <dang@redhat.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -178,13 +178,10 @@ mdcache_fsal_create_export(struct fsal_module *sub_fsal, void *parse_node,
 {
 	fsal_status_t status = {0, 0};
 	struct mdcache_fsal_export *myself;
-	int namelen;
 	pthread_rwlockattr_t attrs;
 
 	myself = gsh_calloc(1, sizeof(struct mdcache_fsal_export));
-	namelen = strlen(sub_fsal->name) + 5;
-	myself->name = gsh_calloc(1, namelen);
-	snprintf(myself->name, namelen, "%s/MDC", sub_fsal->name);
+	myself->name = gsh_concat(sub_fsal->name, "/MDC");
 
 	fsal_export_init(&myself->mfe_exp);
 	mdcache_export_ops_init(&myself->mfe_exp.exp_ops);
@@ -228,6 +225,13 @@ mdcache_fsal_create_export(struct fsal_module *sub_fsal, void *parse_node,
 
 	fsal_export_stack(op_ctx->fsal_export, &myself->mfe_exp);
 
+	status = dirmap_lru_init(myself);
+	if (FSAL_IS_ERROR(status)) {
+		LogMajor(COMPONENT_FSAL, "Failed to call dirmap_lru_init");
+		gsh_free(myself->name);
+		gsh_free(myself);
+		return status;
+	}
 
 	/* Set up op_ctx */
 	op_ctx->fsal_export = &myself->mfe_exp;
@@ -386,13 +390,9 @@ fsal_status_t mdcache_pkginit(void)
 #ifdef USE_DBUS
 void mdcache_dbus_show(DBusMessageIter *iter)
 {
-	struct timespec timestamp;
 	DBusMessageIter struct_iter;
 	char *type;
 
-	now(&timestamp);
-	dbus_append_timestamp(iter, &timestamp);
-
 	dbus_message_iter_open_container(iter, DBUS_TYPE_STRUCT, NULL,
 					 &struct_iter);
 	type = "cache_req";
