@@ -55,8 +55,8 @@ mdc_up_invalidate(const struct fsal_up_vector *vec, struct gsh_buffdesc *handle,
 	op_ctx = &req_ctx;
 
 	key.fsal = vec->up_fsal_export->sub_export->fsal;
-	(void) cih_hash_key(&key, vec->up_fsal_export->sub_export->fsal, handle,
-			    CIH_HASH_KEY_PROTOTYPE);
+	cih_hash_key(&key, vec->up_fsal_export->sub_export->fsal, handle,
+		     CIH_HASH_KEY_PROTOTYPE);
 
 	status = mdcache_find_keyed(&key, &entry);
 	if (status.major == ERR_FSAL_NOENT) {
@@ -110,6 +110,7 @@ mdc_up_update(const struct fsal_up_vector *vec, struct gsh_buffdesc *handle,
 	bool mutatis_mutandis = false;
 	struct req_op_context *save_ctx, req_ctx = {0};
 	mdcache_key_t key;
+	attrmask_t mask_set = 0;
 
 	/* These cannot be updated, changing any of them is
 	   tantamount to destroying and recreating the file. */
@@ -125,7 +126,7 @@ mdc_up_update(const struct fsal_up_vector *vec, struct gsh_buffdesc *handle,
 	if (flags &
 	    ~(fsal_up_update_filesize_inc | fsal_up_update_atime_inc |
 	      fsal_up_update_creation_inc | fsal_up_update_ctime_inc |
-	      fsal_up_update_mtime_inc | fsal_up_update_chgtime_inc |
+	      fsal_up_update_mtime_inc |
 	      fsal_up_update_spaceused_inc | fsal_up_nlink)) {
 		return fsalstat(ERR_FSAL_INVAL, 0);
 	}
@@ -136,8 +137,8 @@ mdc_up_update(const struct fsal_up_vector *vec, struct gsh_buffdesc *handle,
 	op_ctx = &req_ctx;
 
 	key.fsal = vec->up_fsal_export->sub_export->fsal;
-	(void) cih_hash_key(&key, vec->up_fsal_export->sub_export->fsal, handle,
-			    CIH_HASH_KEY_PROTOTYPE);
+	cih_hash_key(&key, vec->up_fsal_export->sub_export->fsal, handle,
+		     CIH_HASH_KEY_PROTOTYPE);
 
 	status = mdcache_find_keyed(&key, &entry);
 	if (status.major == ERR_FSAL_NOENT) {
@@ -187,10 +188,12 @@ mdc_up_update(const struct fsal_up_vector *vec, struct gsh_buffdesc *handle,
 			if (attr->filesize > entry->attrs.filesize) {
 				entry->attrs.filesize = attr->filesize;
 				mutatis_mutandis = true;
+				mask_set |= ATTR_SIZE;
 			}
 		} else {
 			entry->attrs.filesize = attr->filesize;
 			mutatis_mutandis = true;
+			mask_set |= ATTR_SIZE;
 		}
 	}
 
@@ -199,10 +202,12 @@ mdc_up_update(const struct fsal_up_vector *vec, struct gsh_buffdesc *handle,
 			if (attr->spaceused > entry->attrs.spaceused) {
 				entry->attrs.spaceused = attr->spaceused;
 				mutatis_mutandis = true;
+				mask_set |= ATTR_SPACEUSED;
 			}
 		} else {
 			entry->attrs.spaceused = attr->spaceused;
 			mutatis_mutandis = true;
+			mask_set |= ATTR_SPACEUSED;
 		}
 	}
 
@@ -219,26 +224,31 @@ mdc_up_update(const struct fsal_up_vector *vec, struct gsh_buffdesc *handle,
 
 		entry->attrs.acl = attr->acl;
 		mutatis_mutandis = true;
+		mask_set |= ATTR_ACL;
 	}
 
 	if (FSAL_TEST_MASK(attr->valid_mask, ATTR_MODE)) {
 		entry->attrs.mode = attr->mode;
 		mutatis_mutandis = true;
+		mask_set |= ATTR_MODE;
 	}
 
 	if (FSAL_TEST_MASK(attr->valid_mask, ATTR_NUMLINKS)) {
 		entry->attrs.numlinks = attr->numlinks;
 		mutatis_mutandis = true;
+		mask_set |= ATTR_NUMLINKS;
 	}
 
 	if (FSAL_TEST_MASK(attr->valid_mask, ATTR_OWNER)) {
 		entry->attrs.owner = attr->owner;
 		mutatis_mutandis = true;
+		mask_set |= ATTR_OWNER;
 	}
 
 	if (FSAL_TEST_MASK(attr->valid_mask, ATTR_GROUP)) {
 		entry->attrs.group = attr->group;
 		mutatis_mutandis = true;
+		mask_set |= ATTR_GROUP;
 	}
 
 	if (FSAL_TEST_MASK(attr->valid_mask, ATTR_ATIME)
@@ -247,6 +257,7 @@ mdc_up_update(const struct fsal_up_vector *vec, struct gsh_buffdesc *handle,
 		(gsh_time_cmp(&attr->atime, &entry->attrs.atime) == 1))) {
 		entry->attrs.atime = attr->atime;
 		mutatis_mutandis = true;
+		mask_set |= ATTR_ATIME;
 	}
 
 	if (FSAL_TEST_MASK(attr->valid_mask, ATTR_CREATION)
@@ -255,6 +266,7 @@ mdc_up_update(const struct fsal_up_vector *vec, struct gsh_buffdesc *handle,
 		(gsh_time_cmp(&attr->creation, &entry->attrs.creation) == 1))) {
 		entry->attrs.creation = attr->creation;
 		mutatis_mutandis = true;
+		mask_set |= ATTR_CREATION;
 	}
 
 	if (FSAL_TEST_MASK(attr->valid_mask, ATTR_CTIME)
@@ -263,6 +275,7 @@ mdc_up_update(const struct fsal_up_vector *vec, struct gsh_buffdesc *handle,
 		(gsh_time_cmp(&attr->ctime, &entry->attrs.ctime) == 1))) {
 		entry->attrs.ctime = attr->ctime;
 		mutatis_mutandis = true;
+		mask_set |= ATTR_CTIME;
 	}
 
 	if (FSAL_TEST_MASK(attr->valid_mask, ATTR_MTIME)
@@ -271,19 +284,13 @@ mdc_up_update(const struct fsal_up_vector *vec, struct gsh_buffdesc *handle,
 		(gsh_time_cmp(&attr->mtime, &entry->attrs.mtime) == 1))) {
 		entry->attrs.mtime = attr->mtime;
 		mutatis_mutandis = true;
-	}
-
-	if (FSAL_TEST_MASK(attr->valid_mask, ATTR_CHGTIME)
-	    && ((flags & ~fsal_up_update_chgtime_inc)
-		||
-		(gsh_time_cmp(&attr->chgtime, &entry->attrs.chgtime) == 1))) {
-		entry->attrs.chgtime = attr->chgtime;
-		mutatis_mutandis = true;
+		mask_set |= ATTR_MTIME;
 	}
 
 	if (FSAL_TEST_MASK(attr->valid_mask, ATTR_CHANGE)) {
 		entry->attrs.change = attr->change;
 		mutatis_mutandis = true;
+		mask_set |= ATTR_CHANGE;
 	}
 
 	if (FSAL_TEST_MASK(attr->valid_mask, ATTR4_FS_LOCATIONS)) {
@@ -291,6 +298,7 @@ mdc_up_update(const struct fsal_up_vector *vec, struct gsh_buffdesc *handle,
 
 		entry->attrs.fs_locations = attr->fs_locations;
 		mutatis_mutandis = true;
+		mask_set |= ATTR4_FS_LOCATIONS;
 	}
 
 	if (FSAL_TEST_MASK(attr->valid_mask, ATTR4_SEC_LABEL)) {
@@ -299,10 +307,12 @@ mdc_up_update(const struct fsal_up_vector *vec, struct gsh_buffdesc *handle,
 		attr->sec_label.slai_data.slai_data_len = 0;
 		attr->sec_label.slai_data.slai_data_val = NULL;
 		mutatis_mutandis = true;
+		mask_set |= ATTR4_SEC_LABEL;
 	}
 
 	if (mutatis_mutandis) {
 		mdc_fixup_md(entry, attr);
+		entry->attrs.valid_mask |= mask_set;
 		/* If directory can not trust content anymore. */
 		if (entry->obj_handle.type == DIRECTORY) {
 			LogFullDebug(COMPONENT_CACHE_INODE,
@@ -388,7 +398,7 @@ state_status_t mdc_up_lock_grant(const struct fsal_up_vector *vec,
  * Pass up to upper layer
  *
  * @param[in] vec	   Up ops vector
- * @param[in] file         The file in question
+ * @param[in] file	 The file in question
  * @param[in] owner        The lock owner
  * @param[in] lock_param   A description of the lock
  *
