@@ -39,6 +39,40 @@
 #include "FSAL/fsal_commonlib.h"
 #include "nullfs_methods.h"
 
+/**
+ * @brief Callback arg for NULL async callbacks
+ *
+ * NULL needs to know what its object is related to the sub-FSAL's object.
+ * This wraps the given callback arg with NULL specific info
+ */
+struct null_async_arg {
+	struct fsal_obj_handle *obj_hdl;	/**< NULL's handle */
+	fsal_async_cb cb;			/**< Wrapped callback */
+	void *cb_arg;				/**< Wrapped callback data */
+};
+
+/**
+ * @brief Callback for NULL async calls
+ *
+ * Unstack, and call up.
+ *
+ * @param[in] obj		Object being acted on
+ * @param[in] ret		Return status of call
+ * @param[in] obj_data		Data for call
+ * @param[in] caller_data	Data for caller
+ */
+void null_async_cb(struct fsal_obj_handle *obj, fsal_status_t ret,
+		   void *obj_data, void *caller_data)
+{
+	struct fsal_export *save_exp = op_ctx->fsal_export;
+	struct null_async_arg *arg = caller_data;
+
+	op_ctx->fsal_export = save_exp->super_export;
+	arg->cb(arg->obj_hdl, ret, obj_data, arg->cb_arg);
+	op_ctx->fsal_export = save_exp;
+
+	gsh_free(arg);
+}
 
 /* nullfs_close
  * Close the file if it is still open.
@@ -59,7 +93,7 @@ fsal_status_t nullfs_close(struct fsal_obj_handle *obj_hdl)
 	/* calling subfsal method */
 	op_ctx->fsal_export = export->export.sub_export;
 	fsal_status_t status =
-		handle->sub_handle->obj_ops.close(handle->sub_handle);
+		handle->sub_handle->obj_ops->close(handle->sub_handle);
 	op_ctx->fsal_export = &export->export;
 
 	return status;
@@ -87,7 +121,7 @@ fsal_status_t nullfs_open2(struct fsal_obj_handle *obj_hdl,
 	/* calling subfsal method */
 	op_ctx->fsal_export = export->export.sub_export;
 	fsal_status_t status =
-		handle->sub_handle->obj_ops.open2(handle->sub_handle, state,
+		handle->sub_handle->obj_ops->open2(handle->sub_handle, state,
 						  openflags, createmode, name,
 						  attrs_in, verifier,
 						  &sub_handle, attrs_out,
@@ -118,7 +152,7 @@ bool nullfs_check_verifier(struct fsal_obj_handle *obj_hdl,
 	/* calling subfsal method */
 	op_ctx->fsal_export = export->export.sub_export;
 	bool result =
-		handle->sub_handle->obj_ops.check_verifier(handle->sub_handle,
+		handle->sub_handle->obj_ops->check_verifier(handle->sub_handle,
 							   verifier);
 	op_ctx->fsal_export = &export->export;
 
@@ -139,7 +173,7 @@ fsal_openflags_t nullfs_status2(struct fsal_obj_handle *obj_hdl,
 	/* calling subfsal method */
 	op_ctx->fsal_export = export->export.sub_export;
 	fsal_openflags_t result =
-		handle->sub_handle->obj_ops.status2(handle->sub_handle,
+		handle->sub_handle->obj_ops->status2(handle->sub_handle,
 						    state);
 	op_ctx->fsal_export = &export->export;
 
@@ -161,52 +195,45 @@ fsal_status_t nullfs_reopen2(struct fsal_obj_handle *obj_hdl,
 	/* calling subfsal method */
 	op_ctx->fsal_export = export->export.sub_export;
 	fsal_status_t status =
-		handle->sub_handle->obj_ops.reopen2(handle->sub_handle,
+		handle->sub_handle->obj_ops->reopen2(handle->sub_handle,
 						    state, openflags);
 	op_ctx->fsal_export = &export->export;
 
 	return status;
 }
 
-fsal_status_t nullfs_read2(struct fsal_obj_handle *obj_hdl,
-			   bool bypass,
-			   struct state_t *state,
-			   uint64_t offset,
-			   size_t buf_size,
-			   void *buffer,
-			   size_t *read_amount,
-			   bool *eof,
-			   struct io_info *info)
+void nullfs_read2(struct fsal_obj_handle *obj_hdl,
+		  bool bypass,
+		  fsal_async_cb done_cb,
+		  struct fsal_io_arg *read_arg,
+		  void *caller_arg)
 {
 	struct nullfs_fsal_obj_handle *handle =
 		container_of(obj_hdl, struct nullfs_fsal_obj_handle,
 			     obj_handle);
-
 	struct nullfs_fsal_export *export =
 		container_of(op_ctx->fsal_export, struct nullfs_fsal_export,
 			     export);
+	struct null_async_arg *arg;
+
+	/* Set up async callback */
+	arg = gsh_calloc(1, sizeof(*arg));
+	arg->obj_hdl = obj_hdl;
+	arg->cb = done_cb;
+	arg->cb_arg = caller_arg;
 
 	/* calling subfsal method */
 	op_ctx->fsal_export = export->export.sub_export;
-	fsal_status_t status =
-		handle->sub_handle->obj_ops.read2(handle->sub_handle, bypass,
-						  state, offset, buf_size,
-						  buffer, read_amount, eof,
-						  info);
+	handle->sub_handle->obj_ops->read2(handle->sub_handle, bypass,
+					  null_async_cb, read_arg, arg);
 	op_ctx->fsal_export = &export->export;
-
-	return status;
 }
 
-fsal_status_t nullfs_write2(struct fsal_obj_handle *obj_hdl,
-			    bool bypass,
-			    struct state_t *state,
-			    uint64_t offset,
-			    size_t buf_size,
-			    void *buffer,
-			    size_t *write_amount,
-			    bool *fsal_stable,
-			    struct io_info *info)
+void nullfs_write2(struct fsal_obj_handle *obj_hdl,
+		   bool bypass,
+		   fsal_async_cb done_cb,
+		   struct fsal_io_arg *write_arg,
+		   void *caller_arg)
 {
 	struct nullfs_fsal_obj_handle *handle =
 		container_of(obj_hdl, struct nullfs_fsal_obj_handle,
@@ -215,17 +242,19 @@ fsal_status_t nullfs_write2(struct fsal_obj_handle *obj_hdl,
 	struct nullfs_fsal_export *export =
 		container_of(op_ctx->fsal_export, struct nullfs_fsal_export,
 			     export);
+	struct null_async_arg *arg;
+
+	/* Set up async callback */
+	arg = gsh_calloc(1, sizeof(*arg));
+	arg->obj_hdl = obj_hdl;
+	arg->cb = done_cb;
+	arg->cb_arg = caller_arg;
 
 	/* calling subfsal method */
 	op_ctx->fsal_export = export->export.sub_export;
-	fsal_status_t status =
-		handle->sub_handle->obj_ops.write2(handle->sub_handle, bypass,
-						  state, offset, buf_size,
-						  buffer, write_amount,
-						  fsal_stable, info);
+	handle->sub_handle->obj_ops->write2(handle->sub_handle, bypass,
+					   null_async_cb, write_arg, arg);
 	op_ctx->fsal_export = &export->export;
-
-	return status;
 }
 
 fsal_status_t nullfs_seek2(struct fsal_obj_handle *obj_hdl,
@@ -243,7 +272,7 @@ fsal_status_t nullfs_seek2(struct fsal_obj_handle *obj_hdl,
 	/* calling subfsal method */
 	op_ctx->fsal_export = export->export.sub_export;
 	fsal_status_t status =
-		handle->sub_handle->obj_ops.seek2(handle->sub_handle, state,
+		handle->sub_handle->obj_ops->seek2(handle->sub_handle, state,
 						  info);
 	op_ctx->fsal_export = &export->export;
 
@@ -265,7 +294,7 @@ fsal_status_t nullfs_io_advise2(struct fsal_obj_handle *obj_hdl,
 	/* calling subfsal method */
 	op_ctx->fsal_export = export->export.sub_export;
 	fsal_status_t status =
-		handle->sub_handle->obj_ops.io_advise2(handle->sub_handle,
+		handle->sub_handle->obj_ops->io_advise2(handle->sub_handle,
 						       state, hints);
 	op_ctx->fsal_export = &export->export;
 
@@ -286,7 +315,7 @@ fsal_status_t nullfs_commit2(struct fsal_obj_handle *obj_hdl, off_t offset,
 	/* calling subfsal method */
 	op_ctx->fsal_export = export->export.sub_export;
 	fsal_status_t status =
-		handle->sub_handle->obj_ops.commit2(handle->sub_handle, offset,
+		handle->sub_handle->obj_ops->commit2(handle->sub_handle, offset,
 						    len);
 	op_ctx->fsal_export = &export->export;
 
@@ -311,7 +340,7 @@ fsal_status_t nullfs_lock_op2(struct fsal_obj_handle *obj_hdl,
 	/* calling subfsal method */
 	op_ctx->fsal_export = export->export.sub_export;
 	fsal_status_t status =
-		handle->sub_handle->obj_ops.lock_op2(handle->sub_handle, state,
+		handle->sub_handle->obj_ops->lock_op2(handle->sub_handle, state,
 						     p_owner, lock_op, req_lock,
 						     conflicting_lock);
 	op_ctx->fsal_export = &export->export;
@@ -333,8 +362,30 @@ fsal_status_t nullfs_close2(struct fsal_obj_handle *obj_hdl,
 	/* calling subfsal method */
 	op_ctx->fsal_export = export->export.sub_export;
 	fsal_status_t status =
-		handle->sub_handle->obj_ops.close2(handle->sub_handle, state);
+		handle->sub_handle->obj_ops->close2(handle->sub_handle, state);
 	op_ctx->fsal_export = &export->export;
 
 	return status;
 }
+
+fsal_status_t nullfs_fallocate(struct fsal_obj_handle *obj_hdl,
+			       struct state_t *state, uint64_t offset,
+			       uint64_t length, bool allocate)
+{
+	struct nullfs_fsal_obj_handle *handle =
+		container_of(obj_hdl, struct nullfs_fsal_obj_handle,
+			     obj_handle);
+
+	struct nullfs_fsal_export *export =
+		container_of(op_ctx->fsal_export, struct nullfs_fsal_export,
+			     export);
+	fsal_status_t status;
+
+	/* calling subfsal method */
+	op_ctx->fsal_export = export->export.sub_export;
+	status = handle->sub_handle->obj_ops->fallocate(handle->sub_handle,
+							state, offset, length,
+							allocate);
+	op_ctx->fsal_export = &export->export;
+	return status;
+}
