@@ -1014,10 +1014,10 @@ static bool posix_get_fsid(struct fsal_filesystem *fs)
 	dev_name = blkid_devno_to_devname(mnt_stat.st_dev);
 
 	if (dev_name == NULL) {
-		LogInfo(COMPONENT_FSAL,
-			"blkid_devno_to_devname of %s failed for dev %d.%d",
-			fs->path, major(mnt_stat.st_dev),
-			minor(mnt_stat.st_dev));
+		LogDebug(COMPONENT_FSAL,
+			 "blkid_devno_to_devname of %s failed for dev %d.%d",
+			 fs->path, major(mnt_stat.st_dev),
+			 minor(mnt_stat.st_dev));
 		goto out;
 	}
 
@@ -1073,14 +1073,6 @@ static void posix_create_file_system(struct mntent *mnt)
 	struct fsal_filesystem *fs;
 	struct avltree_node *node;
 
-	if (strncasecmp(mnt->mnt_type, "nfs", 3) == 0) {
-		LogDebug(COMPONENT_FSAL,
-			 "Ignoring %s because type %s",
-			 mnt->mnt_dir,
-			 mnt->mnt_type);
-		return;
-	}
-
 	fs = gsh_calloc(1, sizeof(*fs));
 
 	fs->path = gsh_strdup(mnt->mnt_dir);
@@ -1293,6 +1285,20 @@ int populate_posix_file_systems(bool force)
 		if (mnt->mnt_dir == NULL)
 			continue;
 
+		/* stat() on NFS mount points is prone to get stuck in
+		 * kernel due to unavailable NFS servers. Since we don't
+		 * support them anyway, check this early and avoid
+		 * hangs!
+		 */
+		if (strcasecmp(mnt->mnt_type, "nfs") == 0 ||
+		    strcasecmp(mnt->mnt_type, "autofs") == 0) {
+			LogDebug(COMPONENT_FSAL,
+				 "Ignoring %s because type %s",
+				 mnt->mnt_dir,
+				 mnt->mnt_type);
+			continue;
+		}
+
 		if (stat(mnt->mnt_dir, &st) < 0 || !S_ISDIR(st.st_mode)) {
 			continue;
 		}
@@ -1326,6 +1332,38 @@ int populate_posix_file_systems(bool force)
 	return retval;
 }
 
+int reload_posix_filesystems(const char *path,
+			     struct fsal_module *fsal,
+			     struct fsal_export *exp,
+			     claim_filesystem_cb claim,
+			     unclaim_filesystem_cb unclaim,
+			     struct fsal_filesystem **root_fs)
+{
+	int retval = 0;
+
+	retval = populate_posix_file_systems(true);
+
+	if (retval != 0) {
+		LogCrit(COMPONENT_FSAL,
+			"populate_posix_file_systems returned %s (%d)",
+			strerror(retval), retval);
+		return retval;
+	}
+
+	retval = claim_posix_filesystems(path, fsal, exp,
+					 claim, unclaim, root_fs);
+
+	if (retval != 0) {
+		if (retval == EAGAIN)
+			retval = ENOENT;
+		LogCrit(COMPONENT_FSAL,
+			"claim_posix_filesystems(%s) returned %s (%d)",
+			path, strerror(retval), retval);
+	}
+
+	return retval;
+}
+
 int resolve_posix_filesystem(const char *path,
 			     struct fsal_module *fsal,
 			     struct fsal_export *exp,
@@ -1349,30 +1387,22 @@ int resolve_posix_filesystem(const char *path,
 	/* second attempt to resolve file system with force option in case of
 	 * ganesha isn't during startup.
 	 */
-	if (!nfs_init.init_complete || retval != EAGAIN)
+	if (!nfs_init.init_complete || retval != EAGAIN) {
+		LogDebug(COMPONENT_FSAL,
+			 "Not trying to claim filesystems again because %s %s(%d)",
+			 nfs_init.init_complete
+				? "retval != EAGAIN"
+				: "init is not complete",
+			 strerror(retval), retval);
 		return retval;
+	}
 
 	LogDebug(COMPONENT_FSAL,
-		 "Call populate_posix_file_systems one more time");
+		 "Attempting to find a filesystem for %s, reload filesystems",
+		 path);
 
-	retval = populate_posix_file_systems(true);
-	if (retval != 0) {
-		LogCrit(COMPONENT_FSAL,
-			"populate_posix_file_systems returned %s (%d)",
-			strerror(retval), retval);
-		return retval;
-	}
-
-	retval = claim_posix_filesystems(path, fsal, exp,
-					 claim, unclaim, root_fs);
-
-	if (retval != 0) {
-		if (retval == EAGAIN)
-			retval = ENOENT;
-		LogCrit(COMPONENT_FSAL,
-			"claim_posix_filesystems(%s) returned %s (%d)",
-			path, strerror(retval), retval);
-	}
+	retval =
+	    reload_posix_filesystems(path, fsal, exp, claim, unclaim, root_fs);
 
 	return retval;
 }
@@ -1485,6 +1515,7 @@ int process_claim(const char *path,
 	struct glist_head *glist;
 	struct fsal_filesystem *fs;
 	int retval = 0;
+	bool already_claimed = this->fsal == fsal;
 
 	/* Check if the filesystem is already directly exported by some other
 	 * FSAL - note we can only get here is this is the root filesystem for
@@ -1528,9 +1559,15 @@ int process_claim(const char *path,
 		return retval;
 	}
 
-	LogDebug(COMPONENT_FSAL,
-		 "FSAL %s Claiming %s",
-		 fsal->name, this->path);
+	if (already_claimed) {
+		LogDebug(COMPONENT_FSAL,
+			 "FSAL %s Repeat Claiming %s",
+			 fsal->name, this->path);
+	} else {
+		LogInfo(COMPONENT_FSAL,
+			"FSAL %s Claiming %s",
+			fsal->name, this->path);
+	}
 
 	/* Complete the claim */
 	this->fsal = fsal;
@@ -1651,7 +1688,7 @@ static bool posix_showfs(DBusMessageIter *args,
 
 	dbus_message_iter_init_append(reply, &iter);
 	now(&timestamp);
-	dbus_append_timestamp(&iter, &timestamp);
+	gsh_dbus_append_timestamp(&iter, &timestamp);
 
 	dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY,
 					 "(stt)",
@@ -3084,9 +3121,11 @@ bool fsal_common_is_referral(struct fsal_obj_handle *obj_hdl,
 				 "Failed to get attrs for referral, "
 				 "handle: %p, valid_mask: %" PRIx64
 				 ", request_mask: %" PRIx64
-				 ", supported: %" PRIx64,
+				 ", supported: %" PRIx64
+				 ", error: %s",
 				 obj_hdl, attrs->valid_mask,
-				 attrs->request_mask, attrs->supported);
+				 attrs->request_mask, attrs->supported,
+				 fsal_err_txt(status));
 			return false;
 		}
 	}
