@@ -104,6 +104,8 @@ bool fsal_supports(struct fsal_staticfsinfo_t *info,
 		return !!info->whence_is_name;
 	case fso_readdir_plus:
 		return !!info->readdir_plus;
+	case fso_compliant_eof_behavior:
+		return !!info->compliant_eof_behavior;
 	default:
 		return false;	/* whatever I don't know about,
 				 * you can't do
