@@ -45,6 +45,7 @@
 #include "delayed_exec.h"
 #include "export_mgr.h"
 #include "server_stats.h"
+#include "sal_data.h"
 
 struct delegrecall_context {
 	/* Reserve lease during delegation recall */
@@ -1118,6 +1119,8 @@ static void delegrecall_completion_func(rpc_call_t *call)
 		goto out_free_drc;
 	}
 
+	op_ctx = &req_ctx;
+
 	ret = get_state_obj_export_owner_refs(state, &obj,
 					     &export,
 					     NULL);
@@ -1127,7 +1130,6 @@ static void delegrecall_completion_func(rpc_call_t *call)
 		goto out_free_drc;
 	}
 
-	op_ctx = &req_ctx;
 	op_ctx->ctx_export = export;
 	op_ctx->fsal_export = export->fsal_export;
 
@@ -1188,7 +1190,10 @@ out_revoke:
 	deleg_ctx->drc_clid->num_revokes++;
 	inc_revokes(deleg_ctx->drc_clid->gsh_client);
 
+	/* state_del_locked is called from deleg_revoke, take the lock. */
+	PTHREAD_RWLOCK_wrlock(&obj->state_hdl->state_lock);
 	rc = deleg_revoke(obj, state);
+	PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
 
 	if (rc != NFS4_OK) {
 		LogCrit(COMPONENT_NFS_V4,
@@ -1347,6 +1352,9 @@ static void delegrevoke_check(void *ctx)
 		str_valid = true;
 	}
 
+	/* op_ctx may be used by state_del_locked and others */
+	op_ctx = &req_ctx;
+
 	ret = get_state_obj_export_owner_refs(state, &obj,
 					     &export,
 					     NULL);
@@ -1356,8 +1364,6 @@ static void delegrevoke_check(void *ctx)
 		goto out;
 	}
 
-	/* op_ctx may be used by state_del_locked and others */
-	op_ctx = &req_ctx;
 	op_ctx->ctx_export = export;
 	op_ctx->fsal_export = export->fsal_export;
 
@@ -1366,7 +1372,12 @@ static void delegrevoke_check(void *ctx)
 			LogDebug(COMPONENT_STATE,
 				"Revoking delegation for %s", str);
 
+		/* state_del_locked is called from deleg_revoke,
+		 * take the lock.
+		 */
+		PTHREAD_RWLOCK_wrlock(&obj->state_hdl->state_lock);
 		rc = deleg_revoke(obj, state);
+		PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
 
 		if (rc != NFS4_OK) {
 			if (!str_valid)
@@ -1442,7 +1453,10 @@ static void delegrecall_task(void *ctx)
 	op_ctx->ctx_export = export;
 	op_ctx->fsal_export = export->fsal_export;
 
+	/* state_del_locked is called from deleg_revoke, take the lock. */
+	PTHREAD_RWLOCK_wrlock(&obj->state_hdl->state_lock);
 	delegrecall_one(obj, state, deleg_ctx);
+	PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
 
 	/* Release the obj ref and export ref. */
 	obj->obj_ops->put_ref(obj);
@@ -1609,6 +1623,340 @@ state_status_t delegrecall(const struct fsal_up_vector *vec,
 	return rc;
 }
 
+struct cbgetattr_context {
+	struct fsal_obj_handle *obj;
+	nfs_client_id_t *clid; /* client holding deleg */
+	struct gsh_export *ctx_export;	/*< current export */
+};
+
+/*
+ * release cbgetattr_context args reference and
+ * free the memory allocated.
+ */
+static inline void
+free_cbgetattr_context(struct cbgetattr_context *cbgetattr_ctx)
+{
+	PTHREAD_MUTEX_lock(&cbgetattr_ctx->clid->cid_mutex);
+	update_lease(cbgetattr_ctx->clid);
+	PTHREAD_MUTEX_unlock(&cbgetattr_ctx->clid->cid_mutex);
+
+	put_gsh_export(cbgetattr_ctx->ctx_export);
+
+	dec_client_id_ref(cbgetattr_ctx->clid);
+	cbgetattr_ctx->obj->obj_ops->put_ref(cbgetattr_ctx->obj);
+
+	gsh_free(cbgetattr_ctx);
+}
+
+/**
+ * @brief act on response received to CB_GETATTR request
+ *
+ * On successful rpc callback request, read and store
+ * the attributes sent by client and mark cbgetattr_state
+ * as CB_GETATTR_RESP_OK
+ *
+ * @param[in] cbgetattr_context
+ * @param[in] rpc_call
+ *
+ * @return CB_GETATTR_RESP_OK on success
+ *	 CB_GETATTR_FAILED on failure
+ */
+enum
+cbgetattr_state handle_getattr_response(struct cbgetattr_context *cbg_ctx,
+					rpc_call_t *call)
+{
+	fattr4 attr;
+	struct attrlist rsp_attr;
+	int rc = 0;
+	enum cbgetattr_state cb_state;
+	struct fsal_obj_handle *obj = cbg_ctx->obj;
+	nfs_client_id_t *clid = cbg_ctx->clid;
+	nfs_cb_resop4 *cbr = NULL;
+	CB_GETATTR4res *res = NULL;
+	const struct fsal_up_vector *event_func;
+	struct attrlist up_attr = {0, };
+	uint32_t upflags = 0;
+	struct gsh_buffdesc key;
+	fsal_status_t fsal_status = {0,};
+	struct root_op_context root_ctx;
+	struct timespec	timeo = { .tv_sec = time(NULL) + 5,
+				  .tv_nsec = 0 };
+
+	if (clid->cid_minorversion == 0)
+		cbr = &call->cbt.v_u.v4.res.resarray.resarray_val[0];
+	else
+		cbr = &call->cbt.v_u.v4.res.resarray.resarray_val[1];
+
+	res = &cbr->nfs_cb_resop4_u.opcbgetattr;
+	attr = res->CB_GETATTR4res_u.resok4.obj_attributes;
+
+	rc = nfs4_Fattr_To_FSAL_attr(&rsp_attr, &attr, NULL);
+
+	if (rc) {
+		cb_state = CB_GETATTR_FAILED;
+		goto out;
+	}
+
+	cb_state = CB_GETATTR_RSP_OK;
+
+	/* Determine if the file is modified the very first time */
+	if (!obj->state_hdl->file.cbgetattr.modified &&
+	   ((obj->state_hdl->file.cbgetattr.change ==
+		rsp_attr.change) &&
+	    (obj->state_hdl->file.cbgetattr.filesize ==
+		rsp_attr.filesize))) {
+		goto out;
+	} else {
+		/* One/more attributes changed */
+		obj->state_hdl->file.cbgetattr.modified = true;
+	}
+
+	/* everytime we need to update change number to let
+	 * other clients know that file contents could have
+	 * been modified even if the filesize remains same
+	 */
+	obj->state_hdl->file.cbgetattr.change++;
+	obj->state_hdl->file.cbgetattr.filesize = rsp_attr.filesize;
+
+	event_func = (cbg_ctx->ctx_export->fsal_export->up_ops);
+
+	init_root_op_context(&root_ctx, cbg_ctx->ctx_export,
+			 cbg_ctx->ctx_export->fsal_export,
+			 0, 0, UNKNOWN_REQUEST);
+
+	/* @todo : log message if event_func is NULL */
+	obj->obj_ops->handle_to_key(obj, &key);
+
+	up_attr.valid_mask |= (ATTR_CHANGE | ATTR_SIZE | ATTR_MTIME
+			      | ATTR_CTIME);
+	up_attr.request_mask = up_attr.valid_mask;
+
+	up_attr.filesize = obj->state_hdl->file.cbgetattr.filesize;
+	up_attr.change = obj->state_hdl->file.cbgetattr.change;
+	up_attr.mtime = timeo;
+	up_attr.ctime = timeo;
+
+	fsal_status = event_func->update(event_func, &key,
+					 &up_attr, upflags);
+
+	release_root_op_context();
+
+	if (FSAL_IS_ERROR(fsal_status)) {
+		cb_state = CB_GETATTR_FAILED;
+		/* log message */
+		return CB_GETATTR_FAILED;
+	}
+out:
+	return cb_state;
+}
+
+/**
+ * @brief Handle the reply to a CB_GETATTR
+ *
+ * @param[in] call  The RPC call being completed
+ *
+ */
+
+static void cbgetattr_completion_func(rpc_call_t *call)
+{
+	struct cbgetattr_context *cbg_ctx = call->call_arg;
+	enum cbgetattr_state *cbgetattr_state;
+	CB_GETATTR4args *opcbgetattr;
+
+	LogDebug(COMPONENT_NFS_CB, "%p %s", call,
+		 !(call->states & NFS_CB_CALL_ABORTED) ? "Success" : "Failed");
+
+	PTHREAD_RWLOCK_wrlock(&cbg_ctx->obj->state_hdl->state_lock);
+
+	cbgetattr_state = &cbg_ctx->obj->state_hdl->file.cbgetattr.state;
+
+	if (!(call->states & NFS_CB_CALL_ABORTED)) {
+		LogMidDebug(COMPONENT_NFS_CB, "call result: %d",
+			    call->call_req.cc_error.re_status);
+
+		if (call->call_req.cc_error.re_status != RPC_SUCCESS) {
+			LogEvent(COMPONENT_NFS_CB,
+				 "CB_GETATTR call result: %d, marking CB channel down",
+				 call->call_req.cc_error.re_status);
+			set_cb_chan_down(cbg_ctx->clid, true);
+			*cbgetattr_state = CB_GETATTR_FAILED;
+			goto out;
+		}
+
+	} else {
+		LogEvent(COMPONENT_NFS_CB,
+			 "CB_GETATTR Aborted: %d, marking CB channel down",
+			 call->call_req.cc_error.re_status);
+		set_cb_chan_down(cbg_ctx->clid, true);
+		/* Mark the getattr as failed */
+		*cbgetattr_state = CB_GETATTR_FAILED;
+		goto out;
+	}
+	switch (call->cbt.v_u.v4.res.status) {
+	case NFS4_OK:
+		LogDebug(COMPONENT_NFS_CB,
+			 "CB_GEATTR succeded for client(%s)",
+			 cbg_ctx->clid->gsh_client->hostaddr_str);
+		*cbgetattr_state = handle_getattr_response(cbg_ctx,
+							   call);
+		break;
+	default:
+		*cbgetattr_state = CB_GETATTR_FAILED;
+	}
+
+out:
+	PTHREAD_RWLOCK_unlock(&cbg_ctx->obj->state_hdl->state_lock);
+	if (cbg_ctx->clid->cid_minorversion == 0) {
+		opcbgetattr = &call->cbt.v_u.v4.args.argarray.argarray_val[0]
+				.nfs_cb_argop4_u.opcbgetattr;
+		nfs4_freeFH(&opcbgetattr->fh);
+	} else {
+		opcbgetattr = &call->cbt.v_u.v4.args.argarray.argarray_val[1]
+				.nfs_cb_argop4_u.opcbgetattr;
+		nfs4_freeFH(&opcbgetattr->fh);
+		nfs41_release_single(call);
+	}
+	free_cbgetattr_context(cbg_ctx);
+}
+
+/**
+ * @brief Send CB_GETATTR to write_delegated client.
+ *
+ * This function sends a CB_GETATTR, the caller has to lock
+ * cache_entry->state_lock before calling this function.
+ *
+ * @param[in] obj The file holding delegation
+ * @param[in] cbgetattr_context
+ *
+ * @return 0, success
+ *	   otherwise, failure
+ */
+int send_cbgetattr(struct fsal_obj_handle *obj,
+		   struct cbgetattr_context *p_cargs)
+{
+	int ret = 0;
+	nfs_cb_argop4 argop;
+	bitmap4	*cb_attr_req;
+	struct root_op_context root_ctx;
+
+	init_root_op_context(&root_ctx, p_cargs->ctx_export,
+			     p_cargs->ctx_export->fsal_export,
+			     0, 0, UNKNOWN_REQUEST);
+
+	LogDebug(COMPONENT_FSAL_UP, "Sending CB_GETATTR to client %s",
+		 p_cargs->clid->gsh_client->hostaddr_str);
+
+	argop.argop = NFS4_OP_CB_GETATTR;
+
+	/* Convert it to a file handle */
+	if (!nfs4_FSALToFhandle(true, &argop.nfs_cb_argop4_u.opcbgetattr.fh,
+				obj, p_cargs->ctx_export)) {
+		LogCrit(COMPONENT_FSAL_UP,
+			"nfs4_FSALToFhandle failed, can not process recall");
+		goto out;
+	}
+
+	cb_attr_req = &argop.nfs_cb_argop4_u.opcbgetattr.attr_request;
+
+	memset(cb_attr_req, 0, sizeof(*cb_attr_req));
+
+	/*
+	 * If we already know that client has modified data, no need to
+	 * query for change attribute
+	 */
+	if (!obj->state_hdl->file.cbgetattr.modified)
+		set_attribute_in_bitmap(cb_attr_req, FATTR4_CHANGE);
+
+	set_attribute_in_bitmap(cb_attr_req, FATTR4_SIZE);
+
+	ret = nfs_rpc_cb_single(p_cargs->clid, &argop, NULL,
+				cbgetattr_completion_func, p_cargs);
+	if (ret == 0)
+		return 0;
+	LogDebug(COMPONENT_FSAL_UP, "CB_GETATTR nfs_rpc_cb_single returned %d",
+		 ret);
+
+out:
+	nfs4_freeFH(&argop.nfs_cb_argop4_u.opcbgetattr.fh);
+
+	LogCrit(COMPONENT_STATE, "CB_GETATTR failed for %s",
+		p_cargs->clid->gsh_client->hostaddr_str);
+
+	free_cbgetattr_context(p_cargs);
+	release_root_op_context();
+	return ret;
+}
+
+/**
+ * @brief Implement CB_GETATTR op
+ *
+ * By making use of cbgetattr_context, prepare and
+ * send CB_GETATTR request to the client. On success,
+ * set cbgetattr_state to CB_GETATTR_WIP state,
+ * otherwise mark it as CB_GETATTR_FAILED.
+ *
+ * @param[in] obj file being operated on
+ * @param[in] client client holding delegation
+ * @param[in] ctx_exp current export reference
+ *
+ * @return: 0, success
+ *	    otherwise, failure
+ */
+int cbgetattr_impl(struct fsal_obj_handle *obj,
+		   nfs_client_id_t *client,
+		   struct gsh_export *ctx_exp)
+{
+	int rc = 0;
+	enum  cbgetattr_state *cb_state = NULL;
+	struct cbgetattr_context *cbg_ctx;
+
+	LogDebug(COMPONENT_FSAL_UP, "CB_GETATTR: obj %p type %u",
+		 obj, obj->type);
+
+	PTHREAD_RWLOCK_wrlock(&obj->state_hdl->state_lock);
+
+	cb_state = &obj->state_hdl->file.cbgetattr.state;
+
+	if (*cb_state != CB_GETATTR_NONE) {
+		/* Either its already WIP or got error */
+		goto out;
+	}
+	*cb_state = CB_GETATTR_WIP;
+
+	cbg_ctx = gsh_malloc(sizeof(struct cbgetattr_context));
+
+	obj->obj_ops->get_ref(obj);
+	cbg_ctx->obj = obj;
+
+	inc_client_id_ref(client);
+	cbg_ctx->clid = client;
+	/* Prevent client's lease expiring until we complete
+	 * this cb_getattr operation. If the client's lease
+	 * has already expired, let the reaper thread handling
+	 * expired clients revoke this delegation, and we just
+	 * skip it here.
+	 */
+	PTHREAD_MUTEX_lock(&cbg_ctx->clid->cid_mutex);
+	if (!reserve_lease(cbg_ctx->clid)) {
+		PTHREAD_MUTEX_unlock(&cbg_ctx->clid->cid_mutex);
+		gsh_free(cbg_ctx);
+		*cb_state = CB_GETATTR_FAILED;
+		goto out;
+	}
+	PTHREAD_MUTEX_unlock(&cbg_ctx->clid->cid_mutex);
+	get_gsh_export_ref(ctx_exp);
+	cbg_ctx->ctx_export = ctx_exp;
+
+	rc = send_cbgetattr(obj, cbg_ctx);
+out:
+	if (rc)
+		*cb_state = CB_GETATTR_FAILED;
+
+	PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
+
+	return rc;
+}
+
 void up_ready_init(struct fsal_up_vector *up_ops)
 {
 	up_ops->up_ready = false;
