@@ -53,7 +53,6 @@
 #include "mdcache.h"
 #endif
 #include "conf_url.h"
-#include "conf_url_rados.h"
 
 /**
  * @brief Mutex protecting shutdown flag.
@@ -103,7 +102,7 @@ static bool admin_dbus_get_grace(DBusMessageIter *args,
 	dbus_message_iter_append_basic(&iter, DBUS_TYPE_BOOLEAN, &ingrace);
 
  out:
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	return success;
 }
 
@@ -159,10 +158,11 @@ static bool admin_dbus_grace(DBusMessageIter *args,
 	if (ip == NULL)
 		gsp.ipaddr = input;	/* no event specified */
 	else {
-		char *buf = alloca(strlen(input) + 1);
+		int size = strlen(input) + 1;
+		char *buf = alloca(size);
 
 		gsp.ipaddr = ip + 1;	/* point at the ip passed the : */
-		strcpy(buf, input);
+		memcpy(buf, input, size);
 		ip = strstr(buf, ":");
 		if (ip != NULL) {
 			*ip = '\0';	/* replace ":" with null */
@@ -173,7 +173,7 @@ static bool admin_dbus_grace(DBusMessageIter *args,
 	}
 	nfs_start_grace(&gsp);
  out:
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	return success;
 }
 
@@ -211,7 +211,7 @@ static bool admin_dbus_shutdown(DBusMessageIter *args,
 	admin_halt();
 
  out:
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	return success;
 }
 
@@ -247,7 +247,7 @@ static bool admin_dbus_purge_gids(DBusMessageIter *args,
 	uid2grp_clear_cache();
 
  out:
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	return success;
 }
 
@@ -283,7 +283,7 @@ static bool admin_dbus_purge_netgroups(DBusMessageIter *args,
 	ng_clear_cache();
 
  out:
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	return success;
 }
 
@@ -317,7 +317,7 @@ static bool admin_dbus_purge_idmapper_cache(DBusMessageIter *args,
 	}
 	idmapper_clear_cache();
  out:
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	return success;
 }
 
@@ -353,7 +353,7 @@ static bool admin_dbus_init_fds_limit(DBusMessageIter *args,
 	init_fds_limit();
 
  out:
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	return success;
 }
 
@@ -405,7 +405,7 @@ admin_dbus_malloc_trace(DBusMessageIter *args,
 #endif
 
  out:
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	return success;
 }
 
@@ -440,7 +440,7 @@ admin_dbus_malloc_untrace(DBusMessageIter *args,
 #endif
 
  out:
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	return success;
 }
 
@@ -474,6 +474,48 @@ static struct gsh_dbus_method *admin_methods[] = {
 	NULL
 };
 
+#define HANDLE_VERSION_PROP(prop_name, prop_string) \
+static bool dbus_prop_get_VERSION_##prop_name(DBusMessageIter *reply) \
+{ \
+	const char *version_string = prop_string; \
+	if (!dbus_message_iter_append_basic \
+	    (reply, DBUS_TYPE_STRING, &version_string)) \
+		return false; \
+	return true; \
+} \
+\
+static struct gsh_dbus_prop VERSION_##prop_name##_prop = { \
+	.name = "VERSION_" #prop_name, \
+	.access = DBUS_PROP_READ, \
+	.type = "s", \
+	.get = dbus_prop_get_VERSION_##prop_name, \
+	.set = NULL \
+}
+
+#define VERSION_PROPERTY_ITEM(name) (&VERSION_##name##_prop)
+
+HANDLE_VERSION_PROP(RELEASE, GANESHA_VERSION);
+
+#if !GANESHA_BUILD_RELEASE
+HANDLE_VERSION_PROP(COMPILE_DATE, __DATE__);
+HANDLE_VERSION_PROP(COMPILE_TIME, __TIME__);
+HANDLE_VERSION_PROP(COMMENT, VERSION_COMMENT);
+HANDLE_VERSION_PROP(GIT_HEAD, _GIT_HEAD_COMMIT);
+HANDLE_VERSION_PROP(GIT_DESCRIBE, _GIT_DESCRIBE);
+#endif
+
+static struct gsh_dbus_prop *admin_props[] = {
+	VERSION_PROPERTY_ITEM(RELEASE),
+#if !GANESHA_BUILD_RELEASE
+	VERSION_PROPERTY_ITEM(COMPILE_DATE),
+	VERSION_PROPERTY_ITEM(COMPILE_TIME),
+	VERSION_PROPERTY_ITEM(COMMENT),
+	VERSION_PROPERTY_ITEM(GIT_HEAD),
+	VERSION_PROPERTY_ITEM(GIT_DESCRIBE),
+#endif
+	NULL
+};
+
 static struct gsh_dbus_signal heartbeat_signal = {
 	.name = HEARTBEAT_NAME,
 	.signal = NULL,
@@ -488,7 +530,7 @@ static struct gsh_dbus_signal *admin_signals[] = {
 
 static struct gsh_dbus_interface admin_interface = {
 	.name = DBUS_ADMIN_IFACE,
-	.props = NULL,
+	.props = admin_props,
 	.methods = admin_methods,
 	.signals = admin_signals
 };
@@ -536,7 +578,9 @@ static void do_shutdown(void)
 
 	LogEvent(COMPONENT_MAIN, "NFS EXIT: stopping NFS service");
 
+#ifdef USE_RADOS_URLS
 	rados_url_shutdown_watch();
+#endif
 	config_url_shutdown();
 
 #ifdef USE_DBUS
