@@ -45,6 +45,7 @@
 #ifdef USE_DBUS
 #include "gsh_dbus.h"
 #endif
+#include "FSAL/fsal_commonlib.h"
 #ifdef _USE_CB_SIMULATOR
 #include "nfs_rpc_callback_simulator.h"
 #endif
@@ -73,7 +74,7 @@
 #include "mdcache.h"
 #include "common_utils.h"
 #include "nfs_init.h"
-#include "conf_url_rados.h"
+#include <urcu-bp.h>
 
 /**
  * @brief init_complete used to indicate if ganesha is during
@@ -82,7 +83,6 @@
 struct nfs_init nfs_init;
 
 /* global information exported to all layers (as extern vars) */
-pool_t *nfs_request_pool;
 nfs_parameter_t nfs_param;
 struct _nfs_health nfs_health_;
 
@@ -203,9 +203,11 @@ void reread_config(void)
  */
 static void *sigmgr_thread(void *UnusedArg)
 {
-	SetNameFunction("sigmgr");
 	int signal_caught = 0;
 
+	SetNameFunction("sigmgr");
+	rcu_register_thread();
+
 	/* Loop until we catch SIGTERM */
 	while (signal_caught != SIGTERM) {
 		sigset_t signals_to_catch;
@@ -232,6 +234,7 @@ static void *sigmgr_thread(void *UnusedArg)
 	admin_halt();
 
 	/* Might as well exit - no need for this thread any more */
+	rcu_unregister_thread();
 	return NULL;
 }
 
@@ -315,7 +318,6 @@ void nfs_print_param_config(void)
 	printf("\tMNT_Port = %u ;\n", nfs_param.core_param.port[P_MNT]);
 	printf("\tNFS_Program = %u ;\n", nfs_param.core_param.program[P_NFS]);
 	printf("\tMNT_Program = %u ;\n", nfs_param.core_param.program[P_NFS]);
-	printf("\tNb_Worker = %u ;\n", nfs_param.core_param.nb_worker);
 	printf("\tDRC_TCP_Npart = %u ;\n", nfs_param.core_param.drc.tcp.npart);
 	printf("\tDRC_TCP_Size = %u ;\n", nfs_param.core_param.drc.tcp.size);
 	printf("\tDRC_TCP_Cachesz = %u ;\n",
@@ -340,20 +342,17 @@ void nfs_print_param_config(void)
 	printf("\tManage_Gids_Expiration = %" PRIu64 " ;\n",
 	       (uint64_t) nfs_param.core_param.manage_gids_expiration);
 
-	if (nfs_param.core_param.drop_io_errors)
-		printf("\tDrop_IO_Errors = true ;\n");
-	else
-		printf("\tDrop_IO_Errors = false ;\n");
+	printf("\tDrop_IO_Errors = %s ;\n",
+	       nfs_param.core_param.drop_io_errors ?  "true" : "false");
 
-	if (nfs_param.core_param.drop_inval_errors)
-		printf("\tDrop_Inval_Errors = true ;\n");
-	else
-		printf("\tDrop_Inval_Errors = false ;\n");
+	printf("\tDrop_Inval_Errors = %s ;\n",
+	       nfs_param.core_param.drop_inval_errors ?  "true" : "false");
 
-	if (nfs_param.core_param.drop_delay_errors)
-		printf("\tDrop_Delay_Errors = true ;\n");
-	else
-		printf("\tDrop_Delay_Errors = false ;\n");
+	printf("\tDrop_Delay_Errors = %s ;\n",
+	       nfs_param.core_param.drop_delay_errors ? "true" : "false");
+
+	printf("\tEnable UDP = %s ;\n", nfs_param.core_param.enable_UDP ?
+	       "true" : "false");
 
 	printf("}\n\n");
 }
@@ -445,13 +444,13 @@ int nfs_set_param_from_conf(config_file_t parse_tree,
 	if (mdcache_set_param_from_conf(parse_tree, err_type) < 0)
 		return -1;
 
-#ifdef USE_RADOS_RECOV
-	if (rados_kv_set_param_from_conf(parse_tree, err_type) < 0)
+	if (load_recovery_param_from_conf(parse_tree, err_type) < 0)
 		return -1;
-#endif
 
+#ifdef USE_RADOS_URLS
 	if (rados_url_setup_watch() != 0)
 		return -1;
+#endif
 
 	LogEvent(COMPONENT_INIT, "Configuration file successfully parsed");
 
@@ -631,6 +630,7 @@ static void nfs_Init(const nfs_start_info_t *p_start_info)
 	gsh_dbus_pkginit();
 	dbus_export_init();
 	dbus_client_init();
+	dbus_cache_init();
 #endif
 
 	/* acls cache may be needed by exports_pkginit */
@@ -646,16 +646,13 @@ static void nfs_Init(const nfs_start_info_t *p_start_info)
 	nfs41_session_pool =
 	    pool_basic_init("NFSv4.1 session pool", sizeof(nfs41_session_t));
 
-	nfs_request_pool =
-	    pool_basic_init("Request pool", sizeof(request_data_t));
-
 	/* If rpcsec_gss is used, set the path to the keytab */
 #ifdef _HAVE_GSSAPI
 #ifdef HAVE_KRB5
 	if (nfs_param.krb5_param.active_krb5) {
 		OM_uint32 gss_status = GSS_S_COMPLETE;
 
-		if (*nfs_param.krb5_param.keytab != '\0')
+		if (strcmp(nfs_param.krb5_param.keytab, DEFAULT_NFS_KEYTAB))
 			gss_status = krb5_gss_register_acceptor_identity(
 						nfs_param.krb5_param.keytab);
 
