@@ -188,12 +188,37 @@ nc_type nfs_netid_to_nc(const char *netid)
 static inline void setup_client_saddr(nfs_client_id_t *clientid,
 				      const char *uaddr)
 {
-	char addr_buf[SOCK_NAME_MAX + 1];
-	uint32_t bytes[11];
 	int code;
+	char *uaddr2 = gsh_strdup(uaddr);
+	char *dot, *p1, *p2;
+	uint16_t port;
 
 	assert(clientid->cid_minorversion == 0);
 
+	/* find the two port bytes and terminate the string */
+	dot = strrchr(uaddr2, '.');
+
+	if (dot == NULL)
+		goto out;
+
+	p2 = dot + 1;
+	*dot = '\0';
+
+	dot = strrchr(uaddr2, '.');
+
+	if (dot == NULL)
+		goto out;
+
+	p1 = dot + 1;
+	*dot = '\0';
+
+	port = htons((atoi(p1) << 8) | atoi(p2));
+
+	/* At this point, the port has been extracted and uaddr2 is now
+	 * terminated without the port portion and is thus suitable for
+	 * passing to inet_pton as is and will support a variety of formats.
+	 */
+
 	memset(&clientid->cid_cb.v40.cb_addr.ss, 0,
 	       sizeof(struct sockaddr_storage));
 
@@ -202,68 +227,58 @@ static inline void setup_client_saddr(nfs_client_id_t *clientid,
 	case _NC_RDMA:
 	case _NC_SCTP:
 	case _NC_UDP:
+	{
 		/* IPv4 (ws inspired) */
-		if (sscanf(uaddr, "%u.%u.%u.%u.%u.%u", &bytes[1], &bytes[2],
-			   &bytes[3], &bytes[4], &bytes[5], &bytes[6]) != 6)
-			return;
-
 		struct sockaddr_in *sin = ((struct sockaddr_in *)
 					   &clientid->cid_cb.v40.cb_addr.ss);
 
-		snprintf(addr_buf, sizeof(addr_buf), "%u.%u.%u.%u",
-			 bytes[1], bytes[2], bytes[3], bytes[4]);
-
 		sin->sin_family = AF_INET;
-		sin->sin_port = htons((bytes[5] << 8) | bytes[6]);
-		code = inet_pton(AF_INET, addr_buf, &sin->sin_addr);
+		sin->sin_port = port;
+		code = inet_pton(AF_INET, uaddr2, &sin->sin_addr);
 
 		if (code != 1)
 			LogWarn(COMPONENT_NFS_CB, "inet_pton failed (%d %s)",
-				code, addr_buf);
+				code, uaddr);
 		else
 			LogDebug(COMPONENT_NFS_CB,
 				 "client callback addr:port %s:%d",
-				 addr_buf, ntohs(sin->sin_port));
+				 uaddr2, ntohs(port));
 
 		break;
 
+	}
 	case _NC_TCP6:
 	case _NC_RDMA6:
 	case _NC_SCTP6:
 	case _NC_UDP6:
+	{
 		/* IPv6 (ws inspired) */
-		if (sscanf(uaddr, "%2x:%2x:%2x:%2x:%2x:%2x:%2x:%2x.%u.%u",
-			   &bytes[1], &bytes[2], &bytes[3], &bytes[4],
-			   &bytes[5], &bytes[6], &bytes[7], &bytes[8],
-			   &bytes[9], &bytes[10]) != 10)
-			return;
-
 		struct sockaddr_in6 *sin6 = ((struct sockaddr_in6 *)
 					     &clientid->cid_cb.v40.cb_addr.ss);
 
-		snprintf(addr_buf, sizeof(addr_buf),
-			 "%2x:%2x:%2x:%2x:%2x:%2x:%2x:%2x",
-			 bytes[1], bytes[2], bytes[3], bytes[4], bytes[5],
-			 bytes[6], bytes[7], bytes[8]);
-
-		code = inet_pton(AF_INET6, addr_buf, &sin6->sin6_addr);
-		sin6->sin6_port = htons((bytes[9] << 8) | bytes[10]);
 		sin6->sin6_family = AF_INET6;
+		sin6->sin6_port = port;
+		code = inet_pton(AF_INET6, uaddr2, &sin6->sin6_addr);
 
 		if (code != 1)
 			LogWarn(COMPONENT_NFS_CB,
-				"inet_pton failed (%d %s)", code, addr_buf);
+				"inet_pton failed (%d %s)", code, uaddr);
 		else
 			LogDebug(COMPONENT_NFS_CB,
 				 "client callback addr:port %s:%d",
-				 addr_buf, ntohs(sin6->sin6_port));
+				 uaddr2, ntohs(port));
 
 		break;
 
+	}
 	default:
 		/* unknown netid */
 		break;
 	};
+
+out:
+
+	gsh_free(uaddr2);
 }
 
 /**
@@ -278,8 +293,12 @@ void nfs_set_client_location(nfs_client_id_t *clientid,
 {
 	assert(clientid->cid_minorversion == 0);
 	clientid->cid_cb.v40.cb_addr.nc = nfs_netid_to_nc(addr4->r_netid);
-	strlcpy(clientid->cid_cb.v40.cb_client_r_addr, addr4->r_addr,
-		SOCK_NAME_MAX);
+	if (strlcpy(clientid->cid_cb.v40.cb_client_r_addr, addr4->r_addr,
+		    sizeof(clientid->cid_cb.v40.cb_client_r_addr))
+	    >= sizeof(clientid->cid_cb.v40.cb_client_r_addr)) {
+		LogCrit(COMPONENT_CLIENTID, "Callback r_addr %s too long",
+			addr4->r_addr);
+	}
 	setup_client_saddr(clientid, clientid->cid_cb.v40.cb_client_r_addr);
 }
 
@@ -404,10 +423,13 @@ gss_OID_desc krb5oid = { 9, "\052\206\110\206\367\022\001\002\002" };
 static inline char *format_host_principal(rpc_call_channel_t *chan, char *buf,
 					  size_t len)
 {
-	char addr_buf[SOCK_NAME_MAX + 1];
-	const char *host = NULL;
+	const char *qualifier = "nfs@";
+	int qualifier_len = strlen(qualifier);
 	void *sin;
 
+	if (len < SOCK_NAME_MAX)
+		return NULL;
+
 	switch (chan->type) {
 	case RPC_CHAN_V40:
 		sin = &chan->source.clientid->cid_cb.v40.cb_addr.ss;
@@ -416,27 +438,10 @@ static inline char *format_host_principal(rpc_call_channel_t *chan, char *buf,
 		return NULL;
 	}
 
-	switch (((struct sockaddr_in *)sin)->sin_family) {
-	case AF_INET:
-		host = inet_ntop(AF_INET,
-				 &((struct sockaddr_in *)sin)->sin_addr,
-				 addr_buf, INET_ADDRSTRLEN);
-		break;
+	memcpy(buf, qualifier, qualifier_len + 1);
 
-	case AF_INET6:
-		host = inet_ntop(AF_INET6,
-				 &((struct sockaddr_in6 *)sin)->sin6_addr,
-				 addr_buf, INET6_ADDRSTRLEN);
-		break;
-
-	default:
-		break;
-	}
-
-	if (host) {
-		snprintf(buf, len, "nfs@%s", host);
+	if (sprint_sockip(sin, buf + qualifier_len, len - qualifier_len))
 		return buf;
-	}
 
 	return NULL;
 }
@@ -887,14 +892,13 @@ static inline void free_resop(nfs_cb_resop4 *op)
  * @return The newly allocated call or NULL.
  */
 
-rpc_call_t *alloc_rpc_call(void)
+struct _rpc_call *alloc_rpc_call(void)
 {
-	request_data_t *reqdata = pool_alloc(nfs_request_pool);
+	struct _rpc_call *call = gsh_calloc(1, sizeof(struct _rpc_call));
 
 	(void) atomic_inc_uint64_t(&nfs_health_.enqueued_reqs);
 
-	reqdata->rtype = NFS_CALL;
-	return &reqdata->r_u.call;
+	return call;
 }
 
 /**
@@ -917,10 +921,9 @@ void free_rpc_call(rpc_call_t *call)
  */
 static void nfs_rpc_call_free(struct clnt_req *cc, size_t unused)
 {
-	rpc_call_t *call = container_of(cc, rpc_call_t, call_req);
-	request_data_t *reqdata = container_of(call, request_data_t, r_u.call);
+	rpc_call_t *call = container_of(cc, struct _rpc_call, call_req);
 
-	pool_free(nfs_request_pool, reqdata);
+	gsh_free(call);
 	(void) atomic_inc_uint64_t(&nfs_health_.dequeued_reqs);
 }
 
@@ -973,7 +976,7 @@ enum clnt_stat nfs_rpc_call(rpc_call_t *call, uint32_t flags)
 	clnt_req_fill(cc, call->chan->clnt, call->chan->auth, CB_COMPOUND,
 		      (xdrproc_t) xdr_CB_COMPOUND4args, &call->cbt.v_u.v4.args,
 		      (xdrproc_t) xdr_CB_COMPOUND4res, &call->cbt.v_u.v4.res);
-	cc->cc_size = sizeof(request_data_t);
+	cc->cc_size = sizeof(nfs_request_t);
 	cc->cc_free_cb = nfs_rpc_call_free;
 
 	if (!call->chan->clnt) {
