@@ -112,7 +112,8 @@ int _9p_link(struct _9p_request_data *req9p, u32 *plenout, char *preply)
 		return _9p_rerror(req9p, msgtag, ENAMETOOLONG, plenout,
 				  preply);
 	}
-	snprintf(link_name, sizeof(link_name), "%.*s", *name_len, name_str);
+
+	_9p_get_fname(link_name, *name_len, name_str);
 
 	fsal_status = fsal_link(ptargetfid->pentry, pdfid->pentry, link_name);
 
