@@ -79,13 +79,10 @@ int _9p_lock(struct _9p_request_data *req9p, u32 *plenout, char *preply)
 	u8 status = _9P_LOCK_SUCCESS;
 
 	state_status_t state_status = STATE_SUCCESS;
-	state_owner_t *holder;
 	state_owner_t *powner;
 	fsal_lock_param_t lock;
-	fsal_lock_param_t conflict;
 
 	memset(&lock, 0, sizeof(lock));
-	memset(&conflict, 0, sizeof(conflict));
 
 	char name[MAXNAMLEN+1];
 
@@ -132,7 +129,8 @@ int _9p_lock(struct _9p_request_data *req9p, u32 *plenout, char *preply)
 		return _9p_rerror(req9p, msgtag, ENAMETOOLONG, plenout,
 				  preply);
 	}
-	snprintf(name, sizeof(name), "%.*s", *client_id_len, client_id_str);
+
+	_9p_get_fname(name, *client_id_len, client_id_str);
 
 	memset((char *)&hints, 0, sizeof(struct addrinfo));
 	hints.ai_family = AF_INET;
@@ -160,15 +158,16 @@ int _9p_lock(struct _9p_request_data *req9p, u32 *plenout, char *preply)
 		lock.lock_start = *start;
 		lock.lock_length = *length;
 
-		if (nfs_in_grace()) {
+		if (!nfs_get_grace_status(false)) {
 			status = _9P_LOCK_GRACE;
 			break;
 		}
 		PTHREAD_RWLOCK_wrlock(&pfid->pentry->state_hdl->state_lock);
 		state_status = state_lock(pfid->pentry, powner, pfid->state,
 					  STATE_NON_BLOCKING, NULL, &lock,
-					  &holder, &conflict);
+					  NULL, NULL);
 		PTHREAD_RWLOCK_unlock(&pfid->pentry->state_hdl->state_lock);
+		nfs_put_grace_status();
 
 		if (state_status == STATE_SUCCESS)
 			status = _9P_LOCK_SUCCESS;
