@@ -96,7 +96,8 @@ int _9p_mkdir(struct _9p_request_data *req9p, u32 *plenout, char *preply)
 		return _9p_rerror(req9p, msgtag, ENAMETOOLONG, plenout,
 				  preply);
 	}
-	snprintf(dir_name, sizeof(dir_name), "%.*s", *name_len, name_str);
+
+	_9p_get_fname(dir_name, *name_len, name_str);
 
 	fsal_prepare_attrs(&sattr, ATTR_MODE);
 
@@ -116,7 +117,7 @@ int _9p_mkdir(struct _9p_request_data *req9p, u32 *plenout, char *preply)
 				  _9p_tools_errno(fsal_status), plenout,
 				  preply);
 
-	pentry_newdir->obj_ops.put_ref(pentry_newdir);
+	pentry_newdir->obj_ops->put_ref(pentry_newdir);
 
 	/* Build the qid */
 	qid_newdir.type = _9P_QTDIR;
