@@ -53,8 +53,6 @@ fsal_errors_t nfs3_readdirplus_callback(void *opaque,
 					       uint64_t cookie,
 					       enum cb_state cb_state);
 
-static void free_entryplus3s(entryplus3 *entryplus3s);
-
 /**
  * @brief Opaque bookkeeping structure for NFSPROC3_READDIRPLUS
  *
@@ -63,13 +61,10 @@ static void free_entryplus3s(entryplus3 *entryplus3s);
  */
 
 struct nfs3_readdirplus_cb_data {
-	entryplus3 *entries;	/*< The array holding individual entries */
-	size_t mem_left;	/*< The amount of memory remaining before we
+	XDR xdr;	/*< The xdrmem to serialize the entries */
+	char *entries;	/*< The array holding serialized entries */
+	size_t mem_avail;	/*< The amount of memory available before we
 				   hit maxcount */
-	size_t count;		/*< The count of complete entries stored in the
-				   buffer */
-	size_t total_entries;	/*< The number of entires we allocated for
-				   the array. */
 	nfsstat3 error;		/*< Set to a value other than NFS_OK if the
 				   callback function finds a fatal error. */
 };
@@ -116,18 +111,12 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 	uint64_t fsal_cookie = 0;
 	cookieverf3 cookie_verifier;
 	unsigned int num_entries = 0;
-	unsigned long estimated_num_entries = 0;
 	object_file_type_t dir_filetype = 0;
 	bool eod_met = false;
 	fsal_status_t fsal_status = {0, 0};
 	fsal_status_t fsal_status_gethandle = {0, 0};
 	int rc = NFS_REQ_OK;
-	struct nfs3_readdirplus_cb_data tracker = {
-		.entries = NULL,
-		.mem_left = 0,
-		.count = 0,
-		.error = NFS3_OK,
-	};
+	struct nfs3_readdirplus_cb_data tracker;
 	struct attrlist attrs_dir, attrs_parent;
 	bool use_cookie_verifier = op_ctx_export_has_option(
 					EXPORT_OPTION_USE_COOKIE_VERIFIER);
@@ -141,46 +130,42 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 	fsal_prepare_attrs(&attrs_dir, ATTRS_NFS3 | ATTR_RDATTR_ERR);
 	fsal_prepare_attrs(&attrs_parent, ATTRS_NFS3 | ATTR_RDATTR_ERR);
 
-	if (isDebug(COMPONENT_NFSPROTO) || isDebug(COMPONENT_NFS_READDIR)) {
-		char str[LEN_FH_STR];
-
-		sprint_fhandle3(str, &(arg->arg_readdirplus3.dir));
-
-		LogDebugAlt(COMPONENT_NFSPROTO, COMPONENT_NFS_READDIR,
-			    "REQUEST PROCESSING: Calling NFS3_READDIRPLUS handle: %s",
-			    str);
-	}
+	LogNFS3_Operation(COMPONENT_NFSPROTO, req, &arg->arg_readdirplus3.dir,
+			  "");
 
 	/* to avoid setting it on each error case */
 	res->res_readdir3.READDIR3res_u.resfail.dir_attributes.attributes_follow
 		= FALSE;
 
+	memset(&tracker, 0, sizeof(tracker));
+
 	if (op_ctx_export_has_option(EXPORT_OPTION_NO_READDIR_PLUS)) {
 		res->res_readdirplus3.status = NFS3ERR_NOTSUPP;
 		LogDebug(COMPONENT_NFS_READDIR, "Request not supported");
 		goto out;
 	}
 
-	tracker.mem_left = (arg->arg_readdirplus3.maxcount * 9) / 10;
-	begin_cookie = arg->arg_readdirplus3.cookie;
-
-	tracker.mem_left -= sizeof(READDIRPLUS3resok);
-
-	/* Estimate assuming that we're going to send no names and no handles.
-	 * Don't count space for pointers for nextentry or
-	 * name_handle.data.data_val in entryplus3
+	/* Figure out how much memory we have available for the dirlist3.
+	 * Subtract:
+	 *	nfsstat3 status (BYTES_PER_XDR_UNIT)
+	 *	post_op_attr dir_attributes
+	 *		bool_t attributes_follow
+	 *		struct fattr3_wire attributes
+	 *	cookieverf3 cookieverf (8 or sizeof(cookieverf3))
 	 */
-	estimated_num_entries = MIN((tracker.mem_left + sizeof(entryplus3 *))
-				    / (sizeof(entryplus3) - sizeof(char *) * 2),
-				    50);
+	tracker.mem_avail = arg->arg_readdirplus3.maxcount
+				- BYTES_PER_XDR_UNIT
+				- BYTES_PER_XDR_UNIT
+				- sizeof(struct fattr3_wire)
+				- sizeof(cookieverf3);
 
-	tracker.total_entries = estimated_num_entries;
+	begin_cookie = arg->arg_readdirplus3.cookie;
 
 	LogDebug(COMPONENT_NFS_READDIR,
 		 "NFS3_READDIRPLUS: dircount=%u begin_cookie=%" PRIu64
-		 " estimated_num_entries=%lu, mem_left=%zd",
+		 " mem_avail=%zd",
 		 arg->arg_readdirplus3.dircount, begin_cookie,
-		 estimated_num_entries, tracker.mem_left);
+		 tracker.mem_avail);
 
 	/* Convert file handle into a vnode */
 	dir_obj = nfs3_FhandleToCache(&(arg->arg_readdirplus3.dir),
@@ -214,10 +199,11 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 		goto out;
 	}
 
-	memset(cookie_verifier, 0, sizeof(cookieverf3));
+	/* To make or check the cookie verifier */
+	memset(cookie_verifier, 0, sizeof(cookie_verifier));
 
 	/* If cookie verifier is used, then an non-trivial value is
-	 * returned to the client This value is the ctime of the
+	 * returned to the client This value is the change attribute of the
 	 * directory. If verifier is unused (as in many NFS Servers) then
 	 * only a set of zeros is returned (trivial value)
 	 */
@@ -229,9 +215,8 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 			goto out_fail;
 		}
 
-		memcpy(cookie_verifier,
-		       &attrs_dir.ctime.tv_sec,
-		       sizeof(attrs_dir.ctime.tv_sec));
+		memcpy(cookie_verifier, &attrs_dir.change,
+		       MIN(sizeof(cookie_verifier), sizeof(attrs_dir.change)));
 	}
 
 	if (use_cookie_verifier && begin_cookie != 0) {
@@ -256,7 +241,9 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 		fsal_cookie = 0;
 
 	/* Allocate space for entries */
-	tracker.entries = gsh_calloc(estimated_num_entries, sizeof(entryplus3));
+	tracker.entries = gsh_malloc(tracker.mem_avail);
+	xdrmem_create(&tracker.xdr, tracker.entries, tracker.mem_avail,
+		      XDR_ENCODE);
 
 	if (begin_cookie == 0) {
 		/* Fill in "." */
@@ -267,7 +254,7 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 
 		if (res->res_readdir3.status != NFS3_OK) {
 			rc = NFS_REQ_OK;
-			goto out_fail;
+			goto out_destroy;
 		}
 	}
 
@@ -283,7 +270,7 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 			res->res_readdirplus3.status =
 			    nfs3_Errno_status(fsal_status_gethandle);
 			rc = NFS_REQ_OK;
-			goto out_fail;
+			goto out_destroy;
 		}
 
 		res->res_readdir3.status =
@@ -298,7 +285,7 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 
 		if (res->res_readdir3.status != NFS3_OK) {
 			rc = NFS_REQ_OK;
-			goto out_fail;
+			goto out_destroy;
 		}
 	}
 
@@ -315,16 +302,16 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 		/* Is this a retryable error */
 		if (nfs_RetryableError(fsal_status.major)) {
 			rc = NFS_REQ_DROP;
-			goto out_fail;
+			goto out_destroy;
 		}
 
 		res->res_readdirplus3.status = nfs3_Errno_status(fsal_status);
-		goto out_fail;
+		goto out_destroy;
 	}
 
 	if (tracker.error != NFS3_OK) {
 		res->res_readdir3.status = tracker.error;
-		goto out_fail;
+		goto out_destroy;
 	}
 
 	if ((num_entries == 0) && (begin_cookie > 1)) {
@@ -332,8 +319,54 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 		resok->reply.entries = NULL;
 		resok->reply.eof = TRUE;
 	} else {
-		resok->reply.entries = tracker.entries;
-		resok->reply.eof = eod_met;
+		struct xdr_uio *uio;
+		u_int pos_end;
+		bool_t tmp;
+
+		if (eod_met) {
+			/* If we hit end of directory, then the dirlist3 we have
+			 * encoded so far has a complete entry (we MUST have
+			 * consumed the last entry and encoded it). Now we need
+			 * to encode a FALSE to indicate no next entry.
+			 */
+			tmp = FALSE;
+			if (!xdr_bool(&tracker.xdr, &tmp)) {
+				/* Oops... */
+				LogCrit(COMPONENT_NFS_READDIR,
+					"Encode of no next entry failed.");
+				res->res_readdir3.status = NFS3ERR_SERVERFAULT;
+				goto out_destroy;
+			}
+		}
+
+		/* Serialize eod_met into the entries buffer */
+		tmp = eod_met;
+		if (!xdr_bool(&tracker.xdr, &tmp)) {
+			/* Oops... */
+			LogCrit(COMPONENT_NFS_READDIR,
+				"Encode of EOD failed.");
+			res->res_readdir3.status = NFS3ERR_SERVERFAULT;
+			goto out_destroy;
+		}
+
+		pos_end = xdr_getpos(&tracker.xdr);
+
+		/* Get an xdr_uio and fill it in */
+		uio = gsh_calloc(
+			1, sizeof(struct xdr_uio) + sizeof(struct xdr_uio));
+		uio->uio_release = xdr_dirlistplus3_uio_release;
+		uio->uio_count = 1;
+		uio->uio_vio[0].vio_base = tracker.entries;
+		uio->uio_vio[0].vio_head = tracker.entries;
+		uio->uio_vio[0].vio_tail = tracker.entries + pos_end;
+		uio->uio_vio[0].vio_wrap = tracker.entries + pos_end;
+		uio->uio_vio[0].vio_length = pos_end;
+		uio->uio_vio[0].vio_type = VIO_DATA;
+
+		/* Take over entries buffer */
+		tracker.entries = NULL;
+		resok->reply.uio = uio;
+		resok->reply.entries = NULL;
 	}
 
 	nfs_SetPostOpAttr(dir_obj, &resok->dir_attributes, &attrs_dir);
@@ -344,12 +377,17 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 
 	rc = NFS_REQ_OK;
 
-	goto out;
+ out_destroy:
+
+	xdr_destroy(&tracker.xdr);
 
  out_fail:
 
-	/* Set the post op attributes as we have found them. */
-	nfs_SetPostOpAttr(dir_obj, &resfail->dir_attributes, &attrs_dir);
+	if (res->res_readdirplus3.status != NFS3_OK) {
+		/* Set the post op attributes as we have found them. */
+		nfs_SetPostOpAttr(dir_obj, &resfail->dir_attributes,
+				  &attrs_dir);
+	}
 
  out:
 
@@ -360,9 +398,8 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 	if (dir_obj)
 		dir_obj->obj_ops->put_ref(dir_obj);
 
-	if (((res->res_readdir3.status != NFS3_OK) || (rc != NFS_REQ_OK))
-	    && (tracker.entries != NULL))
-		free_entryplus3s(tracker.entries);
+	/* If we allocated but didn't consume entries, free it now. */
+	gsh_free(tracker.entries);
 
 	return rc;
 }				/* nfs3_readdirplus */
@@ -377,10 +414,7 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
  */
 void nfs3_readdirplus_free(nfs_res_t *resp)
 {
-#define RESREADDIRPLUSREPLY resp->res_readdirplus3.READDIRPLUS3res_u.resok.reply
-	if ((resp->res_readdirplus3.status == NFS3_OK)
-	    && (RESREADDIRPLUSREPLY.entries != NULL))
-		free_entryplus3s(RESREADDIRPLUSREPLY.entries);
+	/* Nothing to do, entries has been handed off or already freed. */
 }
 
 /**
@@ -409,101 +443,71 @@ fsal_errors_t nfs3_readdirplus_callback(void *opaque,
 	/* Not-so-opaque pointer to callback data` */
 	struct fsal_readdir_cb_parms *cb_parms = opaque;
 	struct nfs3_readdirplus_cb_data *tracker = cb_parms->opaque;
-	/* Length of the current filename */
-	size_t namelen = strlen(cb_parms->name);
-	entryplus3 *ep3 = tracker->entries + tracker->count;
-
-	if (tracker->count == tracker->total_entries) {
-		cb_parms->in_result = false;
-		return ERR_FSAL_NO_ERROR;
-	}
-
-	/* This is a pessimistic check, which assumes that we're going
-	 * to send attributes and full size handle - if it fails then
-	 * we're close enough to the buffer size limit and t's time to
-	 * stop anyway */
-	if (tracker->mem_left
-	    < (sizeof(entryplus3) + namelen + NFS3_FHSIZE)) {
-		if (tracker->count == 0)
-			tracker->error = NFS3ERR_TOOSMALL;
-
-		cb_parms->in_result = false;
-		return ERR_FSAL_NO_ERROR;
-	}
+	entryplus3 ep3;
+	u_int pos_start = xdr_getpos(&tracker->xdr);
 
 	LogDebug(COMPONENT_NFS_READDIR,
 		"Callback for %s cookie %"PRIu64,
 		cb_parms->name, cookie);
 
-	ep3->fileid = obj->fileid;
-	ep3->name = gsh_strdup(cb_parms->name);
-	ep3->cookie = cookie;
-
-	/* Account for file name + length + cookie */
-	tracker->mem_left -= sizeof(ep3->cookie) + RNDUP(namelen) +
-							BYTES_PER_XDR_UNIT;
+	memset(&ep3, 0, sizeof(ep3));
+	ep3.fileid = obj->fileid;
+	ep3.name = (char *) cb_parms->name;
+	ep3.cookie = cookie;
 
 	if (cb_parms->attr_allowed) {
-		ep3->name_handle.handle_follows = TRUE;
+		ep3.name_handle.handle_follows = TRUE;
 
 		if (!nfs3_FSALToFhandle(true,
-					&ep3->name_handle.post_op_fh3_u.handle,
+					&ep3.name_handle.post_op_fh3_u.handle,
 					obj,
 					op_ctx->ctx_export)) {
 			tracker->error = NFS3ERR_SERVERFAULT;
-			gsh_free(ep3->name);
 			cb_parms->in_result = false;
 			return ERR_FSAL_NO_ERROR;
 		}
 
-		/* Account for filehande + length + follows + nextentry */
-		tracker->mem_left -=
-		    ep3->name_handle.post_op_fh3_u.handle.data.data_len + 12;
-
-		/* Check if attributes follow and then place the attributes
-		 * that follow
-		 */
-		ep3->name_attributes.attributes_follow = nfs3_FSALattr_To_Fattr(
-			obj, attr,
-			&ep3->name_attributes.post_op_attr_u.attributes);
+		/* Check if attributes follow */
+		ep3.name_attributes.attributes_follow =
+						nfs3_Fixup_FSALattr(obj, attr);
 	} else {
-		ep3->name_handle.handle_follows = false;
-		ep3->name_attributes.attributes_follow = false;
-		tracker->mem_left -= sizeof(ep3->name_handle.handle_follows);
+		ep3.name_handle.handle_follows = false;
+		ep3.name_attributes.attributes_follow = false;
 	}
 
-	if (ep3->name_attributes.attributes_follow)
-		tracker->mem_left -= sizeof(ep3->name_attributes);
-	else
-		tracker->mem_left -=
-		    sizeof(ep3->name_attributes.attributes_follow);
+	/* Encode the entry into the xdrmem buffer and then assure there is
+	 * space for at least two booleans (one to be false to terminate the
+	 * entry ist, the other to encode EOD or not). Note we use the special
+	 * xdr_encode_entryplus3 that uses a passed in struct attrlist rather
+	 * than name_attributes from entryplus3, though we will use the boolean
+	 * attributes_follow from the entryplus3.
+	 */
+	if (!xdr_encode_entryplus3(&tracker->xdr, &ep3, attr) ||
+	    (xdr_getpos(&tracker->xdr) + BYTES_PER_XDR_UNIT)
+	    >= tracker->mem_avail) {
+		bool_t res_false = false;
 
-	if (tracker->count > 0)
-		tracker->entries[tracker->count - 1].nextentry = ep3;
+		/* XDR serialization of the entry failed or we ran out of room
+		 * to serialize at least a boolean after the result. */
+		cb_parms->in_result = false;
 
-	++(tracker->count);
-	cb_parms->in_result = true;
+		/* Reset to where we started this entry and encode a boolean
+		 * false instead (entry_follows is false).
+		 */
+		if (!xdr_setpos(&tracker->xdr, pos_start) ||
+		    !xdr_bool(&tracker->xdr, &res_false)) {
+			/* Oops, what broke... */
+			LogCrit(COMPONENT_NFS_READDIR,
+				"Unexpected XDR failure processing readdir result");
+			tracker->error = NFS3ERR_SERVERFAULT;
+		}
+	} else {
+		/* The entry fit, let the caller know it fit */
+		cb_parms->in_result = true;
+	}
+
+	/* Now we are done with anything allocated */
+	gsh_free(ep3.name_handle.post_op_fh3_u.handle.data.data_val);
 
 	return ERR_FSAL_NO_ERROR;
 }				/* nfs3_readdirplus_callback */
-
-/**
- * @brief Clean up memory allocated to serve NFSv3 READDIRPLUS
- *
- * This function traverses the list of entries, freeing all names
- * allocated in the callback function, then frees the list itself.
- *
- * @param entryplus3s [in] Pointer to first obj
- */
-
-static void free_entryplus3s(entryplus3 *entryplus3s)
-{
-	entryplus3 *entry = NULL;
-
-	for (entry = entryplus3s; entry != NULL; entry = entry->nextentry) {
-		gsh_free(entry->name);
-		gsh_free(entry->name_handle.post_op_fh3_u.handle.data.data_val);
-	}
-
-	gsh_free(entryplus3s);
-}
