@@ -75,8 +75,9 @@
  *
  */
 
-int nfs4_op_create_session(struct nfs_argop4 *op, compound_data_t *data,
-			   struct nfs_resop4 *resp)
+enum nfs_req_result nfs4_op_create_session(struct nfs_argop4 *op,
+					   compound_data_t *data,
+					   struct nfs_resop4 *resp)
 {
 	/* Result of looking up the clientid in the confirmed ID
 	   table */
@@ -136,8 +137,10 @@ int nfs4_op_create_session(struct nfs_argop4 *op, compound_data_t *data,
 
 	display_clientid(&dspbuf_clientid4, clientid);
 
-	if (data->minorversion == 0)
-		return res_CREATE_SESSION4->csr_status = NFS4ERR_INVAL;
+	if (data->minorversion == 0) {
+		res_CREATE_SESSION4->csr_status = NFS4ERR_INVAL;
+		return NFS_REQ_ERROR;
+	}
 
 	LogDebug(component,
 		 "CREATE_SESSION client addr=%s clientid=%s -------------------",
@@ -163,7 +166,7 @@ int nfs4_op_create_session(struct nfs_argop4 *op, compound_data_t *data,
 			res_CREATE_SESSION4->csr_status =
 			    clientid_error_to_nfsstat_no_expire(rc);
 
-			return res_CREATE_SESSION4->csr_status;
+			return NFS_REQ_ERROR;
 		}
 
 		client_record = conf->cid_client_record;
@@ -195,10 +198,9 @@ int nfs4_op_create_session(struct nfs_argop4 *op, compound_data_t *data,
 	LogDebug(component,
 		 "CREATE_SESSION clientid=%s csa_sequence=%" PRIu32
 		 " clientid_cs_seq=%" PRIu32
-		 " data_oppos=%d data_use_slot_cached_result=%d",
+		 " data_oppos=%d",
 		 str_clientid4, arg_CREATE_SESSION4->csa_sequence,
-		 found->cid_create_session_sequence, data->oppos,
-		 data->use_slot_cached_result);
+		 found->cid_create_session_sequence, data->oppos);
 
 	if (isFullDebug(component)) {
 		char str[LOG_BUFF_LEN] = "\0";
@@ -208,15 +210,12 @@ int nfs4_op_create_session(struct nfs_argop4 *op, compound_data_t *data,
 		LogFullDebug(component, "Found %s", str);
 	}
 
-	data->use_slot_cached_result = false;
-
 	if ((arg_CREATE_SESSION4->csa_sequence + 1) ==
 	     found->cid_create_session_sequence) {
 		*res_CREATE_SESSION4 = found->cid_create_session_slot;
 
 		LogDebug(component,
-			 "CREATE_SESSION replay=%p special case",
-			 data->cached_result);
+			 "CREATE_SESSION special replay case, used response in cid_create_session_slot");
 		goto out;
 	} else if (arg_CREATE_SESSION4->csa_sequence !=
 		   found->cid_create_session_sequence) {
@@ -323,7 +322,8 @@ int nfs4_op_create_session(struct nfs_argop4 *op, compound_data_t *data,
 						NFS41_MIN_OPERATIONS ||
 	    arg_CREATE_SESSION4->csa_back_chan_attrs.ca_maxrequests == 0) {
 		LogWarn(component,
-			"Invalid channel attributes");
+			"Invalid channel attributes for %s",
+			data->tagname);
 		res_CREATE_SESSION4->csr_status = NFS4ERR_TOOSMALL;
 		goto out;
 	}
@@ -389,7 +389,6 @@ int nfs4_op_create_session(struct nfs_argop4 *op, compound_data_t *data,
 	       nfs41_session->session_id,
 	       NFS4_SESSIONID_SIZE);
 
-	LogDebug(component, "CREATE_SESSION replay=%p", data->cached_result);
 #ifdef USE_LTTNG
 	tracepoint(nfs4, session_ref, __func__, __LINE__, nfs41_session, 2);
 #endif
@@ -582,7 +581,7 @@ int nfs4_op_create_session(struct nfs_argop4 *op, compound_data_t *data,
 	PTHREAD_MUTEX_unlock(&client_record->cr_mutex);
 	/* Release our reference to the client record and return */
 	dec_client_record_ref(client_record);
-	return res_CREATE_SESSION4->csr_status;
+	return nfsstat4_to_nfs_req_result(res_CREATE_SESSION4->csr_status);
 }
 
 /**
