@@ -57,8 +57,9 @@ static const char *lock_tag = "LOCK";
 
 #define SUCCESS_RESP_SIZE (sizeof(nfsstat4) + sizeof(stateid4))
 
-int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
-		 struct nfs_resop4 *resp)
+enum nfs_req_result nfs4_op_lock(struct nfs_argop4 *op,
+				 compound_data_t *data,
+				 struct nfs_resop4 *resp)
 {
 	/* Shorter alias for arguments */
 	LOCK4args * const arg_LOCK4 = &op->nfs_argop4_u.oplock;
@@ -100,6 +101,7 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 	struct state_refer refer;
 	/* Indicate if we let FSAL to handle requests during grace. */
 	bool_t fsal_grace = false;
+	bool_t have_grace_ref = false;
 	int rc;
 	struct fsal_obj_handle *obj = data->current_obj;
 	bool state_lock_held = false;
@@ -119,7 +121,7 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 	res_LOCK4->status = check_resp_room(data, SUCCESS_RESP_SIZE);
 
 	if (res_LOCK4->status != NFS4_OK)
-		return res_LOCK4->status;
+		return NFS_REQ_ERROR;
 
 	/* Record the sequence info */
 	if (data->minorversion > 0) {
@@ -127,13 +129,13 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 		       data->session->session_id,
 		       sizeof(sessionid4));
 		refer.sequence = data->sequence;
-		refer.slot = data->slot;
+		refer.slot = data->slotid;
 	}
 
 	res_LOCK4->status = nfs4_sanity_check_FH(data, REGULAR_FILE, false);
 
 	if (res_LOCK4->status != NFS4_OK)
-		return res_LOCK4->status;
+		return NFS_REQ_ERROR;
 
 	/* Convert lock parameters to internal types */
 	switch (arg_LOCK4->locktype) {
@@ -157,7 +159,7 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 		LogDebug(COMPONENT_NFS_V4_LOCK,
 			 "Invalid lock type");
 		res_LOCK4->status = NFS4ERR_INVAL;
-		return res_LOCK4->status;
+		return NFS_REQ_ERROR;
 	}
 
 	lock_desc.lock_start = arg_LOCK4->offset;
@@ -198,7 +200,7 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 			res_LOCK4->status = nfs_status;
 			LogDebug(COMPONENT_NFS_V4_LOCK,
 				 "LOCK failed nfs4_Check_Stateid for open owner");
-			return res_LOCK4->status;
+			return NFS_REQ_ERROR;
 		}
 
 		open_owner = get_state_owner_ref(state_open);
@@ -311,7 +313,7 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 			res_LOCK4->status = nfs_status;
 			LogDebug(COMPONENT_NFS_V4_LOCK,
 				 "LOCK failed nfs4_Check_Stateid for existing lock owner");
-			return res_LOCK4->status;
+			return NFS_REQ_ERROR;
 		}
 
 		/* Check if lock state belongs to same export */
@@ -449,39 +451,41 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 		goto out;
 	}
 
+
+	fsal_grace = op_ctx->fsal_export->exp_ops.fs_supports(
+					op_ctx->fsal_export, fso_grace_method);
+
 	/* Do grace period checking (use resp_owner below since a new
 	 * lock request with a new lock owner doesn't have a lock owner
 	 * yet, but does have an open owner - resp_owner is always one or
 	 * the other and non-NULL at this point - so makes for a better log).
 	 */
-	if (nfs_in_grace()) {
-		if (op_ctx->fsal_export->exp_ops.fs_supports(
-					op_ctx->fsal_export, fso_grace_method))
-			fsal_grace = true;
-
-		if (!fsal_grace && !arg_LOCK4->reclaim) {
-			LogLock(COMPONENT_NFS_V4_LOCK, NIV_DEBUG,
-			"LOCK failed, non-reclaim while in grace",
-				obj, resp_owner, &lock_desc);
-			res_LOCK4->status = NFS4ERR_GRACE;
-			goto out;
-		}
-		if (!fsal_grace && arg_LOCK4->reclaim
-		    && !clientid->cid_allow_reclaim) {
-			LogLock(COMPONENT_NFS_V4_LOCK, NIV_DEBUG,
-				"LOCK failed, invalid reclaim while in grace",
-				obj, resp_owner, &lock_desc);
-			res_LOCK4->status = NFS4ERR_NO_GRACE;
-			goto out;
-		}
-	} else {
+	if (!fsal_grace) {
 		if (arg_LOCK4->reclaim) {
-			LogLock(COMPONENT_NFS_V4_LOCK, NIV_DEBUG,
-				"LOCK failed, reclaim while not in grace",
-				obj, resp_owner, &lock_desc);
-			res_LOCK4->status = NFS4ERR_NO_GRACE;
-			goto out;
+			if (!clientid->cid_allow_reclaim) {
+				LogLock(COMPONENT_NFS_V4_LOCK, NIV_DEBUG,
+					"LOCK failed, invalid reclaim while in grace",
+					obj, resp_owner, &lock_desc);
+				res_LOCK4->status = NFS4ERR_NO_GRACE;
+				goto out;
+			}
+			if (!nfs_get_grace_status(true)) {
+				LogLock(COMPONENT_NFS_V4_LOCK, NIV_DEBUG,
+					"LOCK failed, reclaim while not in grace",
+					obj, resp_owner, &lock_desc);
+				res_LOCK4->status = NFS4ERR_NO_GRACE;
+				goto out;
+			}
+		} else {
+			if (!nfs_get_grace_status(false)) {
+				LogLock(COMPONENT_NFS_V4_LOCK, NIV_DEBUG,
+					"LOCK failed, non-reclaim while in grace",
+					obj, resp_owner, &lock_desc);
+				res_LOCK4->status = NFS4ERR_GRACE;
+				goto out;
+			}
 		}
+		have_grace_ref = true;
 	}
 
 	/* Test if this request is attempting to create a new lock owner */
@@ -700,6 +704,8 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 	}
 
  out2:
+	if (have_grace_ref)
+		nfs_put_grace_status();
 
 	if (state_lock_held) {
 		/* Now release the state_lock */
@@ -724,7 +730,7 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 	if (clientid != NULL)
 		dec_client_id_ref(clientid);
 
-	return res_LOCK4->status;
+	return nfsstat4_to_nfs_req_result(res_LOCK4->status);
 }				/* nfs4_op_lock */
 
 /**
