@@ -89,18 +89,19 @@ static nfsstat4 open4_create_fh(compound_data_t *data,
 		return NFS4ERR_SERVERFAULT;
 	}
 
-	/* Update the current entry */
-	set_current_entry(data, obj);
-
 	if (state_lock_held) {
 		/* Make sure we don't do cleanup holding the state_lock.
 		 * there will be an additional put_ref without the state_lock
-		 * being held.
+		 * being held. Prevents cleanup in set_current_entry (when
+		 * data->current_obj == obj) and put_ref.
 		 */
 		obj->state_hdl->no_cleanup = true;
 		set_no_cleanup = true;
 	}
 
+	/* Update the current entry */
+	set_current_entry(data, obj);
+
 	/* Put our ref */
 	obj->obj_ops->put_ref(obj);
 
@@ -286,7 +287,6 @@ bool open4_open_owner(struct nfs_argop4 *op, compound_data_t *data,
 	if (res_OPEN4->status == NFS4_OK) {
 		utf8string *utfile;
 		open_claim4 *oc = &arg_OPEN4->claim;
-		char *filename;
 
 		/* Load up CLAIM_DELEGATE_CUR file */
 
@@ -302,19 +302,17 @@ bool open4_open_owner(struct nfs_argop4 *op, compound_data_t *data,
 			return false;
 		}
 		/* Check if filename is correct */
-		res_OPEN4->status = nfs4_utf8string2dynamic(
-					utfile, UTF8_SCAN_ALL, &filename);
+		res_OPEN4->status =
+			nfs4_utf8string_scan(utfile, UTF8_SCAN_PATH_COMP);
 
 		if (res_OPEN4->status != NFS4_OK)
 			return false;
 
 		status = fsal_lookup(data->current_obj,
-				     filename,
+				     utfile->utf8string_val,
 				     &obj_lookup,
 				     NULL);
 
-		gsh_free(filename);
-
 		if (obj_lookup == NULL) {
 			res_OPEN4->status = nfs4_Errno_status(status);
 			return false;
@@ -339,7 +337,6 @@ static nfsstat4 open4_claim_deleg(OPEN4args *arg, compound_data_t *data)
 	stateid4 *rcurr_state;
 	struct fsal_obj_handle *obj_lookup;
 	const utf8string *utfname;
-	char *filename;
 	fsal_status_t fsal_status;
 	nfsstat4 status;
 	state_t *found_state = NULL;
@@ -367,26 +364,25 @@ static nfsstat4 open4_claim_deleg(OPEN4args *arg, compound_data_t *data)
 	rcurr_state =
 		&arg->claim.open_claim4_u.delegate_cur_info.delegate_stateid;
 
-	LogDebug(COMPONENT_NFS_V4, "file name: %.*s",
-		 utfname->utf8string_len, utfname->utf8string_val);
+	LogDebug(COMPONENT_NFS_V4, "file name: %s",
+		 utfname->utf8string_val);
 
 	/* Check if filename is correct */
-	status = nfs4_utf8string2dynamic(utfname, UTF8_SCAN_ALL, &filename);
+	status = nfs4_utf8string_scan(utfname, UTF8_SCAN_PATH_COMP);
 	if (status != NFS4_OK) {
 		LogDebug(COMPONENT_NFS_V4, "Invalid filename");
 		return status;
 	}
 
 	/* Does a file with this name already exist ? */
-	fsal_status = fsal_lookup(data->current_obj, filename,
+	fsal_status = fsal_lookup(data->current_obj, utfname->utf8string_val,
 				  &obj_lookup, NULL);
 
 	if (FSAL_IS_ERROR(fsal_status)) {
-		LogDebug(COMPONENT_NFS_V4, "%s lookup failed.", filename);
-		gsh_free(filename);
+		LogDebug(COMPONENT_NFS_V4, "%s lookup failed.",
+			 utfname->utf8string_val);
 		return nfs4_Errno_status(fsal_status);
 	}
-	gsh_free(filename);
 
 	status = open4_create_fh(data, obj_lookup, false);
 	if (status != NFS4_OK) {
@@ -575,10 +571,11 @@ static void do_delegation(OPEN4args *arg_OPEN4, OPEN4res *res_OPEN4,
 	}
 
 	/* This will be updated later if we actually delegate */
-	if (clientid->cid_minorversion == 0)
-		resok->delegation.delegation_type = OPEN_DELEGATE_NONE;
-	else
+	if (clientid->cid_minorversion != 0 &&
+	    (arg_OPEN4->share_access & OPEN4_SHARE_ACCESS_WANT_DELEG_MASK) != 0)
 		resok->delegation.delegation_type = OPEN_DELEGATE_NONE_EXT;
+	else
+		resok->delegation.delegation_type = OPEN_DELEGATE_NONE;
 
 	/* Client doesn't want a delegation. */
 	if (arg_OPEN4->share_access & OPEN4_SHARE_ACCESS_WANT_NO_DELEG) {
@@ -773,14 +770,16 @@ static void open4_ex(OPEN4args *arg,
 			}
 		}
 
-		/* Validate and convert the utf8 filename */
-		res_OPEN4->status =
-		    nfs4_utf8string2dynamic(&arg->claim.open_claim4_u.file,
-					    UTF8_SCAN_ALL, &filename);
+		/* Validate the utf8 filename */
+		res_OPEN4->status = nfs4_utf8string_scan(
+						&arg->claim.open_claim4_u.file,
+						UTF8_SCAN_PATH_COMP);
 
 		if (res_OPEN4->status != NFS4_OK)
 			goto out;
 
+		filename = arg->claim.open_claim4_u.file.utf8string_val;
+
 		/* Set the createmode if appropriate) */
 		if (arg->openhow.opentype == OPEN4_CREATE) {
 			open4_ex_create_args(arg, data, res_OPEN4, verifier,
@@ -790,7 +789,11 @@ static void open4_ex(OPEN4args *arg,
 				goto out;
 		}
 
-		status = fsal_lookup(parent, filename, &file_obj, NULL);
+		status =
+		    fsal_lookup(parent,
+				arg->claim.open_claim4_u.file.utf8string_val,
+				&file_obj,
+				NULL);
 
 		if (!FSAL_IS_ERROR(status)) {
 			/* Check create situations. */
@@ -823,11 +826,10 @@ static void open4_ex(OPEN4args *arg,
 				}
 			}
 
-			/* We found the file by lookup, discard the filename
+			/* We found the file by lookup, set filename to NULL
 			 * and remember that we found the entry by lookup.
 			 */
 			looked_up_file_obj = true;
-			gsh_free(filename);
 			filename = NULL;
 		} else if (status.major != ERR_FSAL_NOENT ||
 			   arg->openhow.opentype != OPEN4_CREATE) {
@@ -847,6 +849,7 @@ static void open4_ex(OPEN4args *arg,
 			res_OPEN4->status = NFS4ERR_RECLAIM_BAD;
 			goto out;
 		}
+		openflags |= FSAL_O_RECLAIM;
 		file_obj = data->current_obj;
 		break;
 
@@ -866,6 +869,7 @@ static void open4_ex(OPEN4args *arg,
 		res_OPEN4->status = open4_claim_deleg(arg, data);
 		if (res_OPEN4->status != NFS4_OK)
 			goto out;
+		openflags |= FSAL_O_RECLAIM;
 		file_obj = data->current_obj;
 		break;
 
@@ -1179,9 +1183,6 @@ static void open4_ex(OPEN4args *arg,
 	if (state_lock_held)
 		PTHREAD_RWLOCK_unlock(&file_obj->state_hdl->state_lock);
 
-	if (filename)
-		gsh_free(filename);
-
 	if (res_OPEN4->status != NFS4_OK) {
 		/* Cleanup state on error */
 		if (*new_state)
