@@ -54,8 +54,9 @@
  *
  */
 
-int nfs4_op_setclientid(struct nfs_argop4 *op, compound_data_t *data,
-			struct nfs_resop4 *resp)
+enum nfs_req_result nfs4_op_setclientid(struct nfs_argop4 *op,
+					compound_data_t *data,
+					struct nfs_resop4 *resp)
 {
 	SETCLIENTID4args * const arg_SETCLIENTID4 =
 	    &op->nfs_argop4_u.opsetclientid;
@@ -63,7 +64,7 @@ int nfs4_op_setclientid(struct nfs_argop4 *op, compound_data_t *data,
 	    &resp->nfs_resop4_u.opsetclientid;
 	clientaddr4 * const res_SETCLIENTID4_INUSE =
 	    &resp->nfs_resop4_u.opsetclientid.SETCLIENTID4res_u.client_using;
-	char str_verifier[NFS4_VERIFIER_SIZE * 2 + 1];
+	char str_verifier[OPAQUE_BYTES_SIZE(NFS4_VERIFIER_SIZE)];
 	struct display_buffer dspbuf_verifier = {
 			sizeof(str_verifier), str_verifier, str_verifier};
 	char str_client[NFS4_OPAQUE_LIMIT * 2 + 1];
@@ -86,7 +87,7 @@ int nfs4_op_setclientid(struct nfs_argop4 *op, compound_data_t *data,
 
 	if (data->minorversion > 0) {
 		res_SETCLIENTID4->status = NFS4ERR_NOTSUPP;
-		return res_SETCLIENTID4->status;
+		return NFS_REQ_ERROR;
 	}
 
 	if (op_ctx->client != NULL)
@@ -124,7 +125,7 @@ int nfs4_op_setclientid(struct nfs_argop4 *op, compound_data_t *data,
 		LogCrit(COMPONENT_CLIENTID, "SETCLIENTID failed");
 
 		res_SETCLIENTID4->status = NFS4ERR_SERVERFAULT;
-		return res_SETCLIENTID4->status;
+		return NFS_REQ_ERROR;
 	}
 
 	/* The following checks are based on RFC3530bis draft 16
@@ -309,12 +310,13 @@ int nfs4_op_setclientid(struct nfs_argop4 *op, compound_data_t *data,
 		goto out;
 	}
 
-	if (strmaxcpy(unconf->cid_cb.v40.cb_client_r_addr,
-		      arg_SETCLIENTID4->callback.cb_location.r_addr,
-		      sizeof(unconf->cid_cb.v40.cb_client_r_addr)) == -1) {
+	if (strlcpy(unconf->cid_cb.v40.cb_client_r_addr,
+		    arg_SETCLIENTID4->callback.cb_location.r_addr,
+		    sizeof(unconf->cid_cb.v40.cb_client_r_addr))
+	    >= sizeof(unconf->cid_cb.v40.cb_client_r_addr)) {
 		LogCrit(COMPONENT_CLIENTID, "Callback r_addr %s too long",
 			arg_SETCLIENTID4->callback.cb_location.r_addr);
-		free_client_id(unconf);
+
 		res_SETCLIENTID4->status = NFS4ERR_INVAL;
 
 		free_client_id(unconf);
@@ -347,11 +349,12 @@ int nfs4_op_setclientid(struct nfs_argop4 *op, compound_data_t *data,
 		char str[LOG_BUFF_LEN] = "\0";
 		struct display_buffer dspbuf = {sizeof(str), str, str};
 
-		sprint_mem(str_verifier, verifier, NFS4_VERIFIER_SIZE);
-
+		display_cat(&dspbuf, "Verifier=");
+		display_opaque_bytes(&dspbuf, verifier, NFS4_VERIFIER_SIZE);
+		display_cat(&dspbuf, " ");
 		display_client_id_rec(&dspbuf, unconf);
-		LogDebug(COMPONENT_CLIENTID, "SETCLIENTID reply Verifier=%s %s",
-			 str_verifier, str);
+
+		LogDebug(COMPONENT_CLIENTID, "SETCLIENTID reply %s", str);
 	}
 
 	res_SETCLIENTID4->status = NFS4_OK;
@@ -366,7 +369,7 @@ int nfs4_op_setclientid(struct nfs_argop4 *op, compound_data_t *data,
 	/* Release our reference to the client record */
 	dec_client_record_ref(client_record);
 
-	return res_SETCLIENTID4->status;
+	return nfsstat4_to_nfs_req_result(res_SETCLIENTID4->status);
 }
 
 /**
