@@ -53,8 +53,9 @@
  * @return per RFC5661, p. 375
  */
 
-int nfs4_op_verify(struct nfs_argop4 *op, compound_data_t *data,
-		   struct nfs_resop4 *resp)
+enum nfs_req_result nfs4_op_verify(struct nfs_argop4 *op,
+				   compound_data_t *data,
+				   struct nfs_resop4 *resp)
 {
 	VERIFY4args * const arg_VERIFY4 = &op->nfs_argop4_u.opverify;
 	VERIFY4res * const res_VERIFY4 = &resp->nfs_resop4_u.opverify;
@@ -69,19 +70,19 @@ int nfs4_op_verify(struct nfs_argop4 *op, compound_data_t *data,
 	res_VERIFY4->status = nfs4_sanity_check_FH(data, NO_FILE_TYPE, false);
 
 	if (res_VERIFY4->status != NFS4_OK)
-		return res_VERIFY4->status;
+		return NFS_REQ_ERROR;
 
 	/* Get only attributes that are allowed to be read */
 	if (!nfs4_Fattr_Check_Access
 	    (&arg_VERIFY4->obj_attributes, FATTR4_ATTR_READ)) {
 		res_VERIFY4->status = NFS4ERR_INVAL;
-		return res_VERIFY4->status;
+		return NFS_REQ_ERROR;
 	}
 
 	/* Ask only for supported attributes */
 	if (!nfs4_Fattr_Supported(&arg_VERIFY4->obj_attributes)) {
 		res_VERIFY4->status = NFS4ERR_ATTRNOTSUPP;
-		return res_VERIFY4->status;
+		return NFS_REQ_ERROR;
 	}
 
 	fsal_prepare_attrs(&attrs, 0);
@@ -91,14 +92,14 @@ int nfs4_op_verify(struct nfs_argop4 *op, compound_data_t *data,
 				      &attrs.request_mask);
 
 	if (res_VERIFY4->status != NFS4_OK)
-		return res_VERIFY4->status;
+		return NFS_REQ_ERROR;
 
 	res_VERIFY4->status =
 		file_To_Fattr(data, attrs.request_mask, &attrs, &file_attr4,
 			      &arg_VERIFY4->obj_attributes.attrmask);
 
 	if (res_VERIFY4->status != NFS4_OK)
-		return res_VERIFY4->status;
+		return NFS_REQ_ERROR;
 
 	/* Done with the attrs */
 	fsal_release_attrs(&attrs);
@@ -113,7 +114,7 @@ int nfs4_op_verify(struct nfs_argop4 *op, compound_data_t *data,
 		res_VERIFY4->status = NFS4ERR_NOT_SAME;
 
 	nfs4_Fattr_Free(&file_attr4);
-	return res_VERIFY4->status;
+	return nfsstat4_to_nfs_req_result(res_VERIFY4->status);
 }				/* nfs4_op_verify */
 
 /**
