@@ -49,16 +49,15 @@ int nlm4_Lock(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
 	char buffer[MAXNETOBJ_SZ * 2] = "\0";
 	state_nsm_client_t *nsm_client;
 	state_nlm_client_t *nlm_client;
-	state_owner_t *nlm_owner, *holder;
+	state_owner_t *nlm_owner;
 	state_t *nlm_state;
-	fsal_lock_param_t lock, conflict;
+	fsal_lock_param_t lock;
 	int rc;
-	bool grace = nfs_in_grace();
-	state_block_data_t *pblock_data;
+	state_block_data_t *pblock_data = NULL;
 	const char *proc_name = "nlm4_Lock";
 	care_t care = CARE_MONITOR;
 	/* Indicate if we let FSAL to handle requests during grace. */
-	bool_t fsal_grace = false;
+	bool grace_ref;
 
 	if (req->rq_msg.cb_proc == NLMPROC4_NM_LOCK) {
 		/* If call is a NM lock, indicate that we care about NLM
@@ -86,27 +85,21 @@ int nlm4_Lock(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
 		 (unsigned long long)arg->alock.l_len, buffer,
 		 arg->reclaim ? "yes" : "no");
 
-	copy_netobj(&res->res_nlm4test.cookie, &arg->cookie);
+	copy_netobj(&res->res_nlm4.cookie, &arg->cookie);
 
-	if (grace) {
-		/* allow only reclaim lock request during recovery */
-		if (op_ctx->fsal_export->exp_ops.fs_supports(
-					op_ctx->fsal_export, fso_grace_method))
-			fsal_grace = true;
-		if (!fsal_grace && !arg->reclaim) {
+	grace_ref = !op_ctx->fsal_export->exp_ops.fs_supports(
+					op_ctx->fsal_export, fso_grace_method);
+	if (grace_ref) {
+		if (!nfs_get_grace_status(arg->reclaim)) {
+			grace_ref = false;
 			res->res_nlm4.stat.stat = NLM4_DENIED_GRACE_PERIOD;
 			LogDebug(COMPONENT_NLM,
-				 "REQUEST RESULT: in grace %s %s",
+				 "REQUEST RESULT:%s in grace %s %s",
+				 arg->reclaim ? " NOT" : "",
 				 proc_name,
 				 lock_result_str(res->res_nlm4.stat.stat));
 			return NFS_REQ_OK;
 		}
-	} else if (arg->reclaim) { /* don't allow reclaim if not in recovery */
-		res->res_nlm4.stat.stat = NLM4_DENIED_GRACE_PERIOD;
-		LogDebug(COMPONENT_NLM,
-			 "REQUEST RESULT: not in grace %s %s",
-			 proc_name, lock_result_str(res->res_nlm4.stat.stat));
-		return NFS_REQ_OK;
 	}
 
 	rc = nlm_process_parameters(req,
@@ -118,7 +111,7 @@ int nlm4_Lock(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
 				    &nsm_client,
 				    &nlm_client,
 				    &nlm_owner,
-				    &pblock_data,
+				    arg->block ? &pblock_data : NULL,
 				    arg->state,
 				    &nlm_state);
 
@@ -131,20 +124,18 @@ int nlm4_Lock(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
 			 "REQUEST RESULT: %s %s",
 			 proc_name,
 			 lock_result_str(res->res_nlm4.stat.stat));
-		return NFS_REQ_OK;
+		rc = NFS_REQ_OK;
+		goto out;
 	}
 
 	/* Check if v4 delegations conflict with v3 op */
-	PTHREAD_RWLOCK_rdlock(&obj->state_hdl->state_lock);
 	if (state_deleg_conflict(obj, lock.lock_type == FSAL_LOCK_W)) {
-		PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
 		LogDebug(COMPONENT_NLM,
 			 "NLM lock request DROPPED due to delegation conflict");
 		rc = NFS_REQ_DROP;
-		goto out;
+		goto out_dec;
 	} else {
 		(void) atomic_inc_uint32_t(&obj->state_hdl->file.anon_ops);
-		PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
 	}
 
 	/* Cast the state number into a state pointer to protect
@@ -157,10 +148,10 @@ int nlm4_Lock(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
 				  nlm_state,
 				  arg->block ? STATE_NLM_BLOCKING :
 					       STATE_NON_BLOCKING,
-				  pblock_data,
+				  arg->block ? &pblock_data : NULL,
 				  &lock,
-				  &holder,
-				  &conflict);
+				  NULL, /* We don't need conflict info */
+				  NULL);
 	PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
 
 	/* We prevented delegations from being granted while trying to acquire
@@ -170,34 +161,22 @@ int nlm4_Lock(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
 	(void) atomic_dec_uint32_t(&obj->state_hdl->file.anon_ops);
 
 	if (state_status != STATE_SUCCESS) {
-		res->res_nlm4test.test_stat.stat =
+		res->res_nlm4.stat.stat =
 				nlm_convert_state_error(state_status);
 
-		if (state_status == STATE_LOCK_CONFLICT) {
-			nlm_process_conflict(
-			    &res->res_nlm4test.test_stat.nlm4_testrply_u.holder,
-			    holder,
-			    &conflict);
-		}
-
-		/* If we didn't block, release the block data */
-		if (state_status != STATE_LOCK_BLOCKED && pblock_data != NULL)
-			gsh_free(pblock_data);
-
-		if (state_status == STATE_IN_GRACE) {
+		if (state_status == STATE_IN_GRACE)
 			res->res_nlm4.stat.stat = NLM4_DENIED_GRACE_PERIOD;
-			goto out_ok;
-		}
-
 	} else {
 		res->res_nlm4.stat.stat = NLM4_GRANTED;
 	}
-
- out_ok:
-
 	rc = NFS_REQ_OK;
 
- out:
+ out_dec:
+	/* If we didn't block, release the block data. Note that
+	 * state_lock() would set pblock_data to NULL if the lock was
+	 * blocked!
+	 */
+	gsh_free(pblock_data);
 
 	/* Release the NLM Client and NLM Owner references we have */
 	dec_nsm_client_ref(nsm_client);
@@ -210,7 +189,9 @@ int nlm4_Lock(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
 		 "REQUEST RESULT: %s %s",
 		 proc_name,
 		 lock_result_str(res->res_nlm4.stat.stat));
-
+out:
+	if (grace_ref)
+		nfs_put_grace_status();
 	return rc;
 }
 
@@ -299,5 +280,5 @@ int nlm4_Lock_Message(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
  */
 void nlm4_Lock_Free(nfs_res_t *res)
 {
-	netobj_free(&res->res_nlm4test.cookie);
+	netobj_free(&res->res_nlm4.cookie);
 }
