@@ -38,29 +38,89 @@
 #include <sys/types.h>
 #include <fcntl.h>
 #include <ctype.h>
+#include <dlfcn.h>
 #include "bsd-base64.h"
 #include "client_mgr.h"
 #include "fsal.h"
 
 /* The grace_mutex protects current_grace, clid_list, and clid_count */
 static pthread_mutex_t grace_mutex = PTHREAD_MUTEX_INITIALIZER;
-static time_t current_grace; /* current grace period timeout */
+static struct timespec current_grace; /* current grace period timeout */
 static int clid_count; /* number of active clients */
 static struct glist_head clid_list = GLIST_HEAD_INIT(clid_list);  /* clients */
 
-static struct nfs4_recovery_backend *recovery_backend;
+/*
+ * Low two bits of grace_status word are flags. One for whether we're currently
+ * in a grace period and one if a change was requested.
+ */
+#define GRACE_STATUS_ACTIVE_SHIFT	0
+#define GRACE_STATUS_CHANGE_REQ_SHIFT	1
+
+/* The remaining bits are for the refcount */
+#define GRACE_STATUS_COUNTER_SHIFT	2
+
+#define GRACE_STATUS_ACTIVE		(1U << GRACE_STATUS_ACTIVE_SHIFT)
+#define GRACE_STATUS_CHANGE_REQ		(1U << GRACE_STATUS_CHANGE_REQ_SHIFT)
+#define GRACE_STATUS_REF_INCREMENT	(1U << GRACE_STATUS_COUNTER_SHIFT)
+#define GRACE_STATUS_COUNT_MASK		((~0U) << GRACE_STATUS_COUNTER_SHIFT)
+
+static uint32_t	grace_status;
+
+static int default_recovery_init(void)
+{
+	return 0;
+}
+
+static void default_end_grace(void)
+{
+}
+
+static void default_recovery_read_clids(nfs_grace_start_t *gsp,
+					add_clid_entry_hook add_clid_entry,
+					add_rfh_entry_hook add_rfs_entry)
+{
+}
+
+static void default_add_clid(nfs_client_id_t *clientid)
+{
+}
+
+static void default_rm_clid(nfs_client_id_t *clientid)
+{
+}
+
+static void default_add_revoke_fh(nfs_client_id_t *dlr_clid,
+				  nfs_fh4 *dlr_handle)
+{
+}
+
+static struct nfs4_recovery_backend default_recovery_backend = {
+	.recovery_init = default_recovery_init,
+	.end_grace = default_end_grace,
+	.recovery_read_clids = default_recovery_read_clids,
+	.add_clid = default_add_clid,
+	.rm_clid = default_rm_clid,
+	.add_revoke_fh = default_add_revoke_fh,
+};
+
+
+static struct nfs4_recovery_backend *recovery_backend =
+					&default_recovery_backend;
 int32_t reclaim_completes; /* atomic */
 
 static void nfs4_recovery_load_clids(nfs_grace_start_t *gsp);
 static void nfs_release_nlm_state(char *release_ip);
-static void nfs_release_v4_client(char *ip);
+static void nfs_release_v4_clients(char *ip);
+
+
+
 
 clid_entry_t *nfs4_add_clid_entry(char *cl_name)
 {
 	clid_entry_t *new_ent = gsh_malloc(sizeof(clid_entry_t));
 
 	glist_init(&new_ent->cl_rfh_list);
-	strcpy(new_ent->cl_name, cl_name);
+	(void) strlcpy(new_ent->cl_name, cl_name, sizeof(new_ent->cl_name));
 	glist_add(&clid_list, &new_ent->cl_list);
 	++clid_count;
 	return new_ent;
@@ -90,23 +150,67 @@ void nfs4_cleanup_clid_entries(void)
 	atomic_store_int32_t(&reclaim_completes, 0);
 }
 
+/*
+ * Check the current status of the grace period against what the caller needs.
+ * If it's different then return false without taking a reference. If a change
+ * has been requested, then we also don't want to give out a reference.
+ */
+bool nfs_get_grace_status(bool want_grace)
+{
+	uint32_t cur, pro, old;
+
+	old = atomic_fetch_uint32_t(&grace_status);
+	do {
+		cur = old;
+
+		/* If it's not the state we want, then no reference */
+		if (want_grace != (bool)(cur & GRACE_STATUS_ACTIVE))
+			return false;
+
+		/* If a change was requested, no reference */
+		if (cur & GRACE_STATUS_CHANGE_REQ)
+			return false;
+
+		/* Bump the counter */
+		pro = cur + GRACE_STATUS_REF_INCREMENT;
+		old = __sync_val_compare_and_swap(&grace_status, cur, pro);
+	} while (old != cur);
+	return true;
+}
+
+/*
+ * Put grace status. If the refcount goes to zero, and a change was requested,
+ * then wake the reaper thread to do its thing.
+ */
+void nfs_put_grace_status(void)
+{
+	uint32_t cur;
+
+	cur = __sync_fetch_and_sub(&grace_status, GRACE_STATUS_REF_INCREMENT);
+	if (cur & GRACE_STATUS_CHANGE_REQ &&
+	    !(cur >> GRACE_STATUS_COUNTER_SHIFT))
+		reaper_wake();
+}
+
 /**
- * Lift the grace period, if current_grace has not changed since we last
- * checked it. If something has changed in the interim, then don't do
- * anything. Either someone has set a new grace period, or someone else
- * beat us to lifting this one.
+ * Lift the grace period if it's still active.
  */
 static void
-nfs_lift_grace_locked(time_t current)
+nfs_lift_grace_locked(void)
 {
+	uint32_t cur;
+
 	/*
 	 * Caller must hold grace_mutex. Only the thread that actually sets
 	 * the value to 0 gets to clean up the recovery db.
 	 */
-	if (atomic_fetch_time_t(&current_grace) == current) {
+	if (nfs_in_grace()) {
 		nfs_end_grace();
 		__sync_synchronize();
-		atomic_store_time_t(&current_grace, (time_t)0);
+		/* Now change the actual status */
+		cur = __sync_and_and_fetch(&grace_status,
+			~(GRACE_STATUS_ACTIVE|GRACE_STATUS_CHANGE_REQ));
+		assert(!(cur & GRACE_STATUS_COUNT_MASK));
 		LogEvent(COMPONENT_STATE, "NFS Server Now NOT IN GRACE");
 	}
 }
@@ -131,12 +235,14 @@ static void nfs4_set_enforcing(void)
  */
 void nfs_start_grace(nfs_grace_start_t *gsp)
 {
+	int ret;
 	bool was_grace;
+	uint32_t cur, old, pro;
 
 	PTHREAD_MUTEX_lock(&grace_mutex);
 
 	if (nfs_param.nfsv4_param.graceless) {
-		nfs_lift_grace_locked(atomic_fetch_time_t(&current_grace));
+		nfs_lift_grace_locked();
 		LogEvent(COMPONENT_STATE,
 			 "NFS Server skipping GRACE (Graceless is true)");
 		goto out;
@@ -151,8 +257,53 @@ void nfs_start_grace(nfs_grace_start_t *gsp)
 	 * that the callers see the
 	 * Full barrier to ensure enforcement begins ASAP.
 	 */
-	was_grace = (bool)atomic_fetch_time_t(&current_grace);
-	atomic_store_time_t(&current_grace, time(NULL));
+
+	/*
+	 * Ensure there are no outstanding references to the current state of
+	 * grace. If there are, set flag indicating that a change has been
+	 * requested and that no more references will be handed out until it
+	 * takes effect.
+	 */
+	ret = clock_gettime(CLOCK_MONOTONIC, &current_grace);
+	if (ret != 0) {
+		LogCrit(COMPONENT_MAIN, "Failed to get timestamp");
+		assert(0);	/* if this is broken, we are toast so die */
+	}
+
+	cur = atomic_fetch_uint32_t(&grace_status);
+	do {
+		old = cur;
+		was_grace = cur & GRACE_STATUS_ACTIVE;
+
+		/* If we're already in a grace period then we're done */
+		if (was_grace)
+			break;
+
+		/*
+		 * Are there outstanding refs? If so, then set the change req
+		 * flag and nothing else. If not, then clear the change req
+		 * flag and flip the active bit.
+		 */
+		if (old & GRACE_STATUS_COUNT_MASK) {
+			pro = old | GRACE_STATUS_CHANGE_REQ;
+		} else {
+			pro = old | GRACE_STATUS_ACTIVE;
+			pro &= ~GRACE_STATUS_CHANGE_REQ;
+		}
+
+		/* If there are no changes, then we don't need to update */
+		if (pro == old)
+			break;
+		cur = __sync_val_compare_and_swap(&grace_status, old, pro);
+	} while (cur != old);
+
+	/*
+	 * If we were not in a grace period before and there were still
+	 * references outstanding, then we can't do anything else.
+	 */
+	if (!was_grace && (old & GRACE_STATUS_COUNT_MASK))
+		goto out;
+
 	__sync_synchronize();
 
 	if ((int)nfs_param.nfsv4_param.grace_period <
@@ -193,7 +344,7 @@ void nfs_start_grace(nfs_grace_start_t *gsp)
 			nfs_release_nlm_state(gsp->ipaddr);
 			if (gsp->event == EVENT_RELEASE_IP) {
 				PTHREAD_MUTEX_unlock(&grace_mutex);
-				nfs_release_v4_client(gsp->ipaddr);
+				nfs_release_v4_clients(gsp->ipaddr);
 				return;
 			}
 			else {
@@ -213,7 +364,7 @@ out:
  */
 bool nfs_in_grace(void)
 {
-	return atomic_fetch_time_t(&current_grace);
+	return atomic_fetch_uint32_t(&grace_status) & GRACE_STATUS_ACTIVE;
 }
 
 /**
@@ -307,39 +458,77 @@ void nfs_try_lift_grace(void)
 {
 	bool in_grace = true;
 	int32_t rc_count = 0;
-	time_t current = atomic_fetch_time_t(&current_grace);
+	uint32_t cur, old, pro;
 
 	/* Already lifted? Just return */
-	if (!current)
+	if (!(atomic_fetch_uint32_t(&grace_status) & GRACE_STATUS_ACTIVE))
 		return;
 
 	/*
 	 * If we know there are no NLM clients, then we can consider the grace
 	 * period done when all previous clients have sent a RECLAIM_COMPLETE.
 	 */
+	PTHREAD_MUTEX_lock(&grace_mutex);
 	rc_count = atomic_fetch_int32_t(&reclaim_completes);
 	if (!nfs_param.core_param.enable_NLM)
 		in_grace = (rc_count != clid_count);
 
 	/* Otherwise, wait for the timeout */
-	if (in_grace)
-		in_grace = ((current + nfs_param.nfsv4_param.grace_period) >
-					time(NULL));
+	if (in_grace) {
+		struct timespec timeout, now;
+		int ret = clock_gettime(CLOCK_MONOTONIC, &now);
+
+		if (ret != 0) {
+			LogCrit(COMPONENT_MAIN, "Failed to get timestamp");
+			assert(0);
+		}
+
+		timeout = current_grace;
+		timeout.tv_sec += nfs_param.nfsv4_param.grace_period;
+		in_grace = gsh_time_cmp(&timeout, &now) > 0;
+	}
 
 	/*
-	 * Can we lift the grace period now? Clustered backends may need
-	 * extra checks before they can do so. If that is the case, then take
-	 * the grace_mutex and try to do it. If the backend does not implement
-	 * a try_lift_grace operation, then we assume it's always ok.
+	 * Ok, we're basically ready to lift. Ensure there are no outstanding
+	 * references to the current status of the grace period. If there are,
+	 * then set the flag saying that there is an upcoming change.
+	 */
+
+	/*
+	 * Can we lift the grace period now? If there are any outstanding refs,
+	 * then just set the grace_change_req flag to indicate that we don't
+	 * want to hand any more refs out. Otherwise, we try to lift.
+	 *
+	 * Clustered backends may need extra checks before they can do so. If
+	 * the backend does not implement a try_lift_grace operation, then we
+	 * assume there are no external conditions and that it's always ok.
 	 */
 	if (!in_grace) {
-		if (!recovery_backend->try_lift_grace ||
-		     recovery_backend->try_lift_grace()) {
-			PTHREAD_MUTEX_lock(&grace_mutex);
-			nfs_lift_grace_locked(current);
-			PTHREAD_MUTEX_unlock(&grace_mutex);
-		}
+		cur = atomic_fetch_uint32_t(&grace_status);
+		do {
+			old = cur;
+
+			/* Are we already done? Exit if so */
+			if (!(cur & GRACE_STATUS_ACTIVE)) {
+				PTHREAD_MUTEX_unlock(&grace_mutex);
+				return;
+			}
+
+			/* Record that a change has now been requested */
+			pro = old | GRACE_STATUS_CHANGE_REQ;
+			if (pro == old)
+				break;
+			cur = __sync_val_compare_and_swap(&grace_status,
+							  old, pro);
+		} while (cur != old);
+
+		/* Otherwise, go ahead and lift if we can */
+		if (!(old & GRACE_STATUS_COUNT_MASK) &&
+		    (!recovery_backend->try_lift_grace ||
+		     recovery_backend->try_lift_grace()))
+			nfs_lift_grace_locked();
 	}
+	PTHREAD_MUTEX_unlock(&grace_mutex);
 }
 
 static pthread_cond_t enforcing_cond = PTHREAD_COND_INITIALIZER;
@@ -492,24 +681,63 @@ static void nfs4_recovery_load_clids(nfs_grace_start_t *gsp)
 						nfs4_add_rfh_entry);
 }
 
-static int load_backend(const char *name)
-{
-	if (!strcmp(name, "fs"))
-		fs_backend_init(&recovery_backend);
 #ifdef USE_RADOS_RECOV
-	else if (!strcmp(name, "rados_kv"))
-		rados_kv_backend_init(&recovery_backend);
-	else if (!strcmp(name, "rados_ng"))
-		rados_ng_backend_init(&recovery_backend);
-	else if (!strcmp(name, "rados_cluster"))
-		rados_cluster_backend_init(&recovery_backend);
+static struct {
+	void *dl;
+	void (*kv_init)(struct nfs4_recovery_backend **);
+	void (*ng_init)(struct nfs4_recovery_backend **);
+	void (*cluster_init)(struct nfs4_recovery_backend **);
+	int (*load_config_from_parse)(config_file_t,
+				      struct config_error_type *);
+} rados = { NULL,};
+
+static int load_rados_recov(void)
+{
+	rados.dl = dlopen("libganesha_rados_recov.so",
+#if defined(LINUX) && !defined(SANITIZE_ADDRESS)
+			  RTLD_NOW | RTLD_LOCAL | RTLD_DEEPBIND);
+#elif defined(FREEBSD) || defined(SANITIZE_ADDRESS)
+			  RTLD_NOW | RTLD_LOCAL);
 #endif
-	else if (!strcmp(name, "fs_ng"))
-		fs_ng_backend_init(&recovery_backend);
-	else
+
+	if (rados.dl) {
+		rados.kv_init = dlsym(rados.dl, "rados_kv_backend_init");
+		rados.ng_init = dlsym(rados.dl, "rados_ng_backend_init");
+		rados.cluster_init = dlsym(rados.dl,
+					   "rados_cluster_backend_init");
+		rados.load_config_from_parse = dlsym(rados.dl,
+					   "rados_load_config_from_parse");
+
+		if (!rados.kv_init || !rados.ng_init || !rados.cluster_init ||
+		    !rados.load_config_from_parse) {
+			dlclose(rados.dl);
+			rados.dl = NULL;
+			return -1;
+		}
+	} else {
 		return -1;
+	}
 	return 0;
 }
+#endif
+
+const char *recovery_backend_str(enum recovery_backend recovery_backend)
+{
+	switch (recovery_backend) {
+	case RECOVERY_BACKEND_FS:
+		return "fs";
+	case RECOVERY_BACKEND_FS_NG:
+		return "fs_ng";
+	case RECOVERY_BACKEND_RADOS_KV:
+		return "rados_kv";
+	case RECOVERY_BACKEND_RADOS_NG:
+		return "rados_ng";
+	case RECOVERY_BACKEND_RADOS_CLUSTER:
+		return "rados_cluster";
+	}
+
+	return "Unknown recovery backend";
+}
 
 /**
  * @brief Create the recovery directory
@@ -520,10 +748,38 @@ static int load_backend(const char *name)
  */
 int nfs4_recovery_init(void)
 {
-	if (load_backend(nfs_param.nfsv4_param.recovery_backend)) {
-		LogCrit(COMPONENT_CLIENTID, "Unknown recovery backend");
+	LogInfo(COMPONENT_CLIENTID, "Recovery Backend Init for %s",
+		recovery_backend_str(nfs_param.nfsv4_param.recovery_backend));
+
+	switch (nfs_param.nfsv4_param.recovery_backend) {
+	case RECOVERY_BACKEND_FS:
+		fs_backend_init(&recovery_backend);
+		break;
+	case RECOVERY_BACKEND_FS_NG:
+		fs_ng_backend_init(&recovery_backend);
+		break;
+#ifdef USE_RADOS_RECOV
+	case RECOVERY_BACKEND_RADOS_KV:
+		rados.kv_init(&recovery_backend);
+		break;
+	case RECOVERY_BACKEND_RADOS_NG:
+		rados.ng_init(&recovery_backend);
+		break;
+	case RECOVERY_BACKEND_RADOS_CLUSTER:
+		rados.cluster_init(&recovery_backend);
+		break;
+#else
+	case RECOVERY_BACKEND_RADOS_KV:
+	case RECOVERY_BACKEND_RADOS_NG:
+	case RECOVERY_BACKEND_RADOS_CLUSTER:
+#endif
+	default:
+		LogCrit(COMPONENT_CLIENTID, "Unsupported Backend %s",
+			recovery_backend_str(
+				nfs_param.nfsv4_param.recovery_backend));
 		return -ENOENT;
 	}
+
 	return recovery_backend->recovery_init();
 }
 
@@ -537,6 +793,11 @@ void nfs4_recovery_shutdown(void)
 {
 	if (recovery_backend->recovery_shutdown)
 		recovery_backend->recovery_shutdown();
+#ifdef USE_RADOS_RECOV
+	if (rados.dl)
+		(void) dlclose(rados.dl);
+	rados.dl = NULL;
+#endif
 }
 
 /**
@@ -590,10 +851,6 @@ bool nfs4_check_deleg_reclaim(nfs_client_id_t *clid, nfs_fh4 *fhandle)
 	int b64ret;
 	bool retval = true;
 
-	/* If we aren't in grace period, then reclaim is not possible */
-	if (!nfs_in_grace())
-		return false;
-
 	/* Convert nfs_fh4_val into base64 encoded string */
 	b64ret = base64url_encode(fhandle->nfs_fh4_val, fhandle->nfs_fh4_len,
 				  rhdlstr, sizeof(rhdlstr));
@@ -639,7 +896,7 @@ static void nlm_releasecall(struct fridgethr_context *ctx)
 }
 #endif /* _USE_NLM */
 
-void extractv4(char *ipv6, char *ipv4)
+void extractv4(char *ipv6, char *ipv4, size_t size)
 {
 	char *token, *saveptr;
 	char *delim = ":";
@@ -648,19 +905,19 @@ void extractv4(char *ipv6, char *ipv4)
 	while (token != NULL) {
 		/* IPv4 delimiter is '.' */
 		if (strchr(token, '.') != NULL) {
-			(void)strcpy(ipv4, token);
+			(void) strlcpy(ipv4, token, size);
 			return;
 		}
 		token = strtok_r(NULL, delim, &saveptr);
 	}
 	/* failed, copy a null string */
-	(void)strcpy(ipv4, "");
+	ipv4[0] = '\0';
 }
 
 bool ip_str_match(char *release_ip, char *server_ip)
 {
 	bool ripv6, sipv6;
-	char ipv4[SOCK_NAME_MAX + 1];
+	char ipv4[SOCK_NAME_MAX];
 
 	/* IPv6 delimiter is ':' */
 	ripv6 = (strchr(release_ip, ':') != NULL);
@@ -671,13 +928,13 @@ bool ip_str_match(char *release_ip, char *server_ip)
 			return !strcmp(release_ip, server_ip);
 		else {
 			/* extract v4 addr from release_ip*/
-			extractv4(release_ip, ipv4);
+			extractv4(release_ip, ipv4, sizeof(ipv4));
 			return !strcmp(ipv4, server_ip);
 		}
 	} else {
 		if (sipv6) {
 			/* extract v4 addr from server_ip*/
-			extractv4(server_ip, ipv4);
+			extractv4(server_ip, ipv4, sizeof(ipv4));
 			return !strcmp(ipv4, release_ip);
 		}
 	}
@@ -698,7 +955,7 @@ static void nfs_release_nlm_state(char *release_ip)
 	struct rbt_node *pn;
 	struct hash_data *pdata;
 	state_status_t state_status;
-	char serverip[SOCK_NAME_MAX + 1];
+	char serverip[SOCK_NAME_MAX];
 	int i;
 
 	LogDebug(COMPONENT_STATE, "Release all NLM locks");
@@ -713,10 +970,10 @@ static void nfs_release_nlm_state(char *release_ip)
 		RBT_LOOP(head_rbt, pn) {
 			pdata = RBT_OPAQ(pn);
 			nlm_cp = (state_nlm_client_t *) pdata->val.addr;
-			sprint_sockip(&(nlm_cp->slc_server_addr),
-					serverip,
-					SOCK_NAME_MAX + 1);
-			if (ip_str_match(release_ip, serverip)) {
+
+			if (sprint_sockip(&nlm_cp->slc_server_addr,
+					  serverip, sizeof(serverip)) &&
+			    ip_str_match(release_ip, serverip)) {
 				nsm_cp = nlm_cp->slc_nsm_client;
 				inc_nsm_client_ref(nsm_cp);
 				state_status = fridgethr_submit(
@@ -738,24 +995,31 @@ static void nfs_release_nlm_state(char *release_ip)
 
 static int ip_match(char *ip, nfs_client_id_t *cid)
 {
-	LogDebug(COMPONENT_STATE, "NFS Server V4 match ip %s with (%s)",
-		 ip, cid->cid_client_record->cr_client_val);
+	char *haystack;
+	char *value = cid->cid_client_record->cr_client_val;
+	int len = cid->cid_client_record->cr_client_val_len;
+
+	LogDebug(COMPONENT_STATE, "NFS Server V4 match ip %s with (%.*s)",
+		 ip, len, value);
 
 	if (strlen(ip) == 0)	/* No IP all are matching */
 		return 1;
 
-	if (strstr(cid->cid_client_record->cr_client_val, ip) != NULL)
+	haystack = alloca(len + 1);
+	memcpy(haystack, value, len);
+	haystack[len] = '\0';
+	if (strstr(haystack, ip) != NULL)
 		return 1;
 
 	return 0;		/* no match */
 }
 
 /*
- * try to find a V4 client that matches the IP we are releasing.
+ * try to find a V4 clients which match the IP we are releasing.
  * only search the confirmed clients, unconfirmed clients won't
  * have any state to release.
  */
-static void nfs_release_v4_client(char *ip)
+static void nfs_release_v4_clients(char *ip)
 {
 	hash_table_t *ht = ht_confirmed_client_id;
 	struct rbt_head *head_rbt;
@@ -769,10 +1033,11 @@ static void nfs_release_v4_client(char *ip)
 
 	/* go through the confirmed clients looking for a match */
 	for (i = 0; i < ht->parameter.index_size; i++) {
-
-		PTHREAD_RWLOCK_wrlock(&ht->partitions[i].lock);
 		head_rbt = &ht->partitions[i].rbt;
 
+restart:
+		PTHREAD_RWLOCK_wrlock(&ht->partitions[i].lock);
+
 		/* go through all entries in the red-black-tree */
 		RBT_LOOP(head_rbt, pn) {
 			pdata = RBT_OPAQ(pn);
@@ -810,7 +1075,7 @@ static void nfs_release_v4_client(char *ip)
 				}
 
 				dec_client_id_ref(cp);
-				return;
+				goto restart;
 
 			} else {
 				PTHREAD_MUTEX_unlock(&cp->cid_mutex);
@@ -821,4 +1086,44 @@ static void nfs_release_v4_client(char *ip)
 	}
 }
 
+int load_recovery_param_from_conf(config_file_t parse_tree,
+				  struct config_error_type *err_type)
+{
+	switch (nfs_param.nfsv4_param.recovery_backend) {
+	case RECOVERY_BACKEND_FS:
+	case RECOVERY_BACKEND_FS_NG:
+		return 0;
+
+	case RECOVERY_BACKEND_RADOS_KV:
+	case RECOVERY_BACKEND_RADOS_NG:
+	case RECOVERY_BACKEND_RADOS_CLUSTER:
+#ifdef USE_RADOS_RECOV
+		/*
+		 * see if we actually need the rados_recovery shlib loaded
+		 *
+		 * we are here because the config (explicitly) calls
+		 * for this recovery class. If we can't do it because
+		 * the (package with the) libganesha_rados_recovery
+		 * library wasn't installed, then we should return
+		 * an error and eventually die.
+		 */
+		if (!rados.dl && load_rados_recov() < 0) {
+			LogCrit(COMPONENT_CLIENTID,
+				"Failed to load Backend %s. Please install the appropriate package",
+				recovery_backend_str(
+				       nfs_param.nfsv4_param.recovery_backend));
+			return -1;
+		}
+
+		return rados.load_config_from_parse(parse_tree, err_type);
+#endif
+	default:
+		LogCrit(COMPONENT_CLIENTID, "Unsupported Backend %s",
+			recovery_backend_str(
+				nfs_param.nfsv4_param.recovery_backend));
+	}
+
+	return -1;
+}
+
 /** @} */
