@@ -38,29 +38,41 @@ static void legacy_fs_db_migrate(void)
 		char *dname;
 
 		/* create empty tmpdir in same parent */
-		snprintf(pathbuf, sizeof(pathbuf), "%s.XXXXXX", v4_recov_link);
+		ret = snprintf(pathbuf, sizeof(pathbuf), "%s.XXXXXX",
+			       v4_recov_link);
+
+		if (unlikely(ret >= sizeof(pathbuf))) {
+			LogCrit(COMPONENT_CLIENTID,
+				"Path too long %s.XXXXXX", v4_recov_link);
+			return;
+		} else if (unlikely(ret < 0)) {
+			LogCrit(COMPONENT_CLIENTID,
+				"Unexpected return from snprintf %d error %s (%d)",
+				ret, strerror(errno), errno);
+			return;
+		}
 
 		dname = mkdtemp(pathbuf);
 		if (!dname) {
 			LogEvent(COMPONENT_CLIENTID,
-				"Failed to create temp file (%s): %s",
-				pathbuf, strerror(errno));
+				"Failed to create temp file (%s): %s (%d)",
+				pathbuf, strerror(errno), errno);
 			return;
 		}
 
 		ret = rename(v4_recov_link, dname);
 		if (ret != 0) {
 			LogEvent(COMPONENT_CLIENTID,
-				"Failed to rename v4 recovery dir (%s) to (%s): %s",
-				v4_recov_link, dname, strerror(errno));
+				"Failed to rename v4 recovery dir (%s) to (%s): %s (%d)",
+				v4_recov_link, dname, strerror(errno), errno);
 			return;
 		}
 
 		ret = symlink(basename(dname), v4_recov_link);
 		if (ret != 0) {
 			LogEvent(COMPONENT_CLIENTID,
-				"Failed to set recoverydir symlink at %s: %s",
-				dname, strerror(errno));
+				"Failed to set recoverydir symlink at %s: %s (%d)",
+				dname, strerror(errno), errno);
 			return;
 		}
 	}
@@ -75,43 +87,99 @@ static int fs_ng_create_recov_dir(void)
 	err = mkdir(NFS_V4_RECOV_ROOT, 0700);
 	if (err == -1 && errno != EEXIST) {
 		LogEvent(COMPONENT_CLIENTID,
-			 "Failed to create v4 recovery dir (%s): %s",
-			 NFS_V4_RECOV_ROOT, strerror(errno));
+			 "Failed to create v4 recovery dir (%s): %s (%d)",
+			 NFS_V4_RECOV_ROOT, strerror(errno), errno);
+	}
+
+	err = snprintf(v4_recov_dir, sizeof(v4_recov_dir), "%s/%s",
+		       NFS_V4_RECOV_ROOT, NFS_V4_RECOV_DIR);
+
+	if (unlikely(err >= sizeof(v4_recov_dir))) {
+		LogCrit(COMPONENT_CLIENTID,
+			"Path too long %s/%s",
+			NFS_V4_RECOV_ROOT, NFS_V4_RECOV_DIR);
+		return -EINVAL;
+	} else if (unlikely(err < 0)) {
+		int error = errno;
+
+		LogCrit(COMPONENT_CLIENTID,
+			"Unexpected return from snprintf %d error %s (%d)",
+			err, strerror(error), error);
+		return -error;
 	}
 
-	snprintf(v4_recov_dir, sizeof(v4_recov_dir), "%s/%s", NFS_V4_RECOV_ROOT,
-		 NFS_V4_RECOV_DIR);
 	err = mkdir(v4_recov_dir, 0700);
 	if (err == -1 && errno != EEXIST) {
 		LogEvent(COMPONENT_CLIENTID,
-			 "Failed to create v4 recovery dir(%s): %s",
-			 v4_recov_dir, strerror(errno));
+			 "Failed to create v4 recovery dir(%s): %s (%d)",
+			 v4_recov_dir, strerror(errno), errno);
 	}
 
 	/* Populate link path string */
 	if (nfs_param.core_param.clustered) {
-		snprintf(host, sizeof(host), "node%d", g_nodeid);
+		err = snprintf(host, sizeof(host), "node%d", g_nodeid);
+
+		if (unlikely(err >= sizeof(host))) {
+			LogCrit(COMPONENT_CLIENTID,
+				"node%d too long", g_nodeid);
+			return -EINVAL;
+		} else if (unlikely(err < 0)) {
+			int error = errno;
+
+			LogCrit(COMPONENT_CLIENTID,
+				"Unexpected return from snprintf %d error %s (%d)",
+				err, strerror(error), error);
+			return -error;
+		}
 	} else {
 		err = gethostname(host, sizeof(host));
 		if (err) {
 			LogEvent(COMPONENT_CLIENTID,
-				 "Failed to gethostname: %s",
-				 strerror(errno));
+				 "Failed to gethostname: %s (%d)",
+				 strerror(errno), errno);
 			return -errno;
 		}
 	}
 
-	snprintf(v4_recov_link, sizeof(v4_recov_link), "%s/%s/%s",
-		 NFS_V4_RECOV_ROOT, NFS_V4_RECOV_DIR, host);
+	err = snprintf(v4_recov_link, sizeof(v4_recov_link), "%s/%s/%s",
+		       NFS_V4_RECOV_ROOT, NFS_V4_RECOV_DIR, host);
 
-	snprintf(v4_recov_dir, sizeof(v4_recov_dir), "%s.XXXXXX",
-		 v4_recov_link);
+	if (unlikely(err >= sizeof(v4_recov_link))) {
+		LogCrit(COMPONENT_CLIENTID,
+			"Path too long %s/%s/%s",
+			NFS_V4_RECOV_ROOT, NFS_V4_RECOV_DIR, host);
+		return -EINVAL;
+	} else if (unlikely(err < 0)) {
+		int error = errno;
+
+		LogCrit(COMPONENT_CLIENTID,
+			"Unexpected return from snprintf %d error %s (%d)",
+			err, strerror(error), error);
+		return -error;
+	}
+
+	err = snprintf(v4_recov_dir, sizeof(v4_recov_dir), "%s.XXXXXX",
+		       v4_recov_link);
+
+	if (unlikely(err >= sizeof(v4_recov_dir))) {
+		LogCrit(COMPONENT_CLIENTID,
+			"Path too long %s.XXXXXX",
+			v4_recov_link);
+		return -EINVAL;
+	} else if (unlikely(err < 0)) {
+		int error = errno;
+
+		LogCrit(COMPONENT_CLIENTID,
+			"Unexpected return from snprintf %d error %s (%d)",
+			err, strerror(error), error);
+		return -error;
+	}
 
 	newdir = mkdtemp(v4_recov_dir);
 	if (newdir != v4_recov_dir) {
 		LogEvent(COMPONENT_CLIENTID,
-			 "Failed to create v4 recovery dir(%s): %s",
-			 v4_recov_dir, strerror(errno));
+			 "Failed to create v4 recovery dir(%s): %s (%d)",
+			 v4_recov_dir, strerror(errno), errno);
 	}
 
 	legacy_fs_db_migrate();
@@ -142,14 +210,13 @@ static int fs_ng_read_recov_clids_impl(const char *parent_path,
 	char temp[10];
 	int cid_len, len;
 	int segment_len;
-	int total_len;
-	int total_clid_len;
+	int clid_str_len = clid_str ? strlen(clid_str) : 0;
 
 	dp = opendir(parent_path);
 	if (dp == NULL) {
 		LogEvent(COMPONENT_CLIENTID,
-			 "Failed to open v4 recovery dir (%s): %s",
-			 parent_path, strerror(errno));
+			 "Failed to open v4 recovery dir (%s): %s (%d)",
+			 parent_path, strerror(errno), errno);
 		return -1;
 	}
 
@@ -170,24 +237,19 @@ static int fs_ng_read_recov_clids_impl(const char *parent_path,
 		 * next readdir. This recursion keeps reading the
 		 * subdirectory until reaching the end.
 		 */
+		sub_path = gsh_concat_sep(parent_path, '/', dentp->d_name);
 		segment_len = strlen(dentp->d_name);
-		total_len = segment_len + 2 + strlen(parent_path);
-		sub_path = gsh_malloc(total_len);
 
-		memset(sub_path, 0, total_len);
-
-		strcpy(sub_path, parent_path);
-		strcat(sub_path, "/");
-		strncat(sub_path, dentp->d_name, segment_len);
 		/* keep building the clientid str by recursively */
 		/* reading the directory structure */
-		total_clid_len = segment_len + 1;
+		build_clid = gsh_malloc(clid_str_len + segment_len + 1);
+
 		if (clid_str)
-			total_clid_len += strlen(clid_str);
-		build_clid = gsh_calloc(1, total_clid_len);
-		if (clid_str)
-			strcpy(build_clid, clid_str);
-		strncat(build_clid, dentp->d_name, segment_len);
+			memcpy(build_clid, clid_str, clid_str_len);
+
+		memcpy(build_clid + clid_str_len,
+		       dentp->d_name,
+		       segment_len + 1);
 
 		rc = fs_ng_read_recov_clids_impl(sub_path,
 					      build_clid,
@@ -242,8 +304,7 @@ static int fs_ng_read_recov_clids_impl(const char *parent_path,
 				gsh_free(build_clid);
 				continue;
 			}
-			strncpy(temp, ptr+1, len);
-			temp[len] = 0;
+			memcpy(temp, ptr+1, len+1);
 			cid_len = atoi(temp);
 			len = strlen(ptr2);
 			if ((len == (cid_len+2)) && (ptr2[len-1] == ')')) {
@@ -302,12 +363,23 @@ static void fs_ng_read_recov_clids(nfs_grace_start_t *gsp,
 
 	switch (gsp->event) {
 	case EVENT_TAKE_NODEID:
-		snprintf(path, sizeof(path), "%s/%s/node%d",
-			 NFS_V4_RECOV_ROOT, NFS_V4_RECOV_DIR,
-			 gsp->nodeid);
+		rc = snprintf(path, sizeof(path), "%s/%s/node%d",
+			      NFS_V4_RECOV_ROOT, NFS_V4_RECOV_DIR, gsp->nodeid);
+
+		if (unlikely(rc >= sizeof(path))) {
+			LogCrit(COMPONENT_CLIENTID,
+				"Path too long %s/%s/node%d",
+				NFS_V4_RECOV_ROOT, NFS_V4_RECOV_DIR,
+				gsp->nodeid);
+		} else if (unlikely(rc < 0)) {
+			LogCrit(COMPONENT_CLIENTID,
+				"Unexpected return from snprintf %d error %s (%d)",
+				rc, strerror(errno), errno);
+			return;
+		}
 		break;
 	default:
-		LogWarn(COMPONENT_STATE, "Recovery unknown event: %d",
+		LogWarn(COMPONENT_CLIENTID, "Recovery unknown event: %d",
 				gsp->event);
 		return;
 	}
@@ -336,14 +408,25 @@ static void fs_ng_swap_recov_dir(void)
 	old_path = realpath(v4_recov_link, old_pathbuf);
 
 	/* Make a new symlink at a temporary location, pointing to new dir */
-	snprintf(tmp_link, PATH_MAX, "%s.tmp", v4_recov_link);
+	ret = snprintf(tmp_link, sizeof(tmp_link), "%s.tmp", v4_recov_link);
+
+	if (unlikely(ret >= sizeof(tmp_link))) {
+		LogCrit(COMPONENT_CLIENTID,
+			"Path too long %s.tmp", v4_recov_link);
+		return;
+	} else if (unlikely(ret < 0)) {
+		LogCrit(COMPONENT_CLIENTID,
+			"Unexpected return from snprintf %d error %s (%d)",
+			ret, strerror(errno), errno);
+		return;
+	}
 
 	/* unlink old symlink, if any */
 	ret = unlink(tmp_link);
 	if (ret != 0 && errno != ENOENT) {
 		LogEvent(COMPONENT_CLIENTID,
-			 "Unable to remove recoverydir symlink: %s",
-			 strerror(errno));
+			 "Unable to remove recoverydir symlink: %s (%d)",
+			 strerror(errno), errno);
 		return;
 	}
 
@@ -351,8 +434,8 @@ static void fs_ng_swap_recov_dir(void)
 	ret = symlink(basename(v4_recov_dir), tmp_link);
 	if (ret != 0) {
 		LogEvent(COMPONENT_CLIENTID,
-			 "Unable to create recoverydir symlink: %s",
-			 strerror(errno));
+			 "Unable to create recoverydir symlink: %s (%d)",
+			 strerror(errno), errno);
 		return;
 	}
 
@@ -360,8 +443,8 @@ static void fs_ng_swap_recov_dir(void)
 	ret = rename(tmp_link, v4_recov_link);
 	if (ret != 0) {
 		LogEvent(COMPONENT_CLIENTID,
-			 "Unable to rename recoverydir symlink: %s",
-			 strerror(errno));
+			 "Unable to rename recoverydir symlink: %s (%d)",
+			 strerror(errno), errno);
 		return;
 	}
 
