@@ -14,13 +14,14 @@
 #include <rados/librados.h>
 #include <rados_grace.h>
 #include "recovery_rados.h"
+#include <urcu-bp.h>
 
 #define MAX_ITEMS		1024		/* relaxed */
 
 static rados_t clnt;
 rados_ioctx_t rados_recov_io_ctx;
-char rados_recov_oid[NI_MAXHOST + 6];
-char rados_recov_old_oid[NI_MAXHOST + 4];
+struct gsh_refstr *rados_recov_oid;
+struct gsh_refstr *rados_recov_old_oid;
 
 struct rados_kv_parameter rados_kv_param;
 
@@ -101,39 +102,64 @@ static int convert_opaque_val(struct display_buffer *dspbuf,
 	return b_left;
 }
 
-void rados_kv_create_key(nfs_client_id_t *clientid, char *key)
-{
-	snprintf(key, RADOS_KEY_MAX_LEN, "%lu",
-		 (uint64_t)clientid->cid_clientid);
-}
-
-void rados_kv_create_val(nfs_client_id_t *clientid, char *val)
+char *rados_kv_create_val(nfs_client_id_t *clientid, size_t *size)
 {
 	char *src = clientid->cid_client_record->cr_client_val;
 	int src_len = clientid->cid_client_record->cr_client_val_len;
 	const char *str_client_addr = "(unknown)";
-	char cidstr[PATH_MAX] = { 0, };
+	char cidstr[PATH_MAX] = { 0, }, *val;
 	struct display_buffer dspbuf = {sizeof(cidstr), cidstr, cidstr};
-	char cidstr_len[20];
-	int total_len;
+	char cidstr_lenx[5];
+	int total_len, cidstr_len, cidstr_lenx_len, str_client_addr_len;
 	int ret;
+	size_t lsize;
 
 	/* get the caller's IP addr */
 	if (clientid->gsh_client != NULL)
 		str_client_addr = clientid->gsh_client->hostaddr_str;
 
-	ret = convert_opaque_val(&dspbuf, src, src_len, PATH_MAX);
+	str_client_addr_len = strlen(str_client_addr);
+
+	ret = convert_opaque_val(&dspbuf, src, src_len, sizeof(cidstr));
 	assert(ret > 0);
 
-	snprintf(cidstr_len, sizeof(cidstr_len), "%zd", strlen(cidstr));
-	total_len = strlen(cidstr) + strlen(str_client_addr) + 5 +
-		    strlen(cidstr_len);
+	cidstr_len = display_buffer_len(&dspbuf);
+
+	cidstr_lenx_len = snprintf(cidstr_lenx, sizeof(cidstr_lenx), "%d",
+				   cidstr_len);
+
+	if (unlikely(cidstr_lenx_len >= sizeof(cidstr_lenx) ||
+		     cidstr_lenx_len < 0)) {
+		/* cidrstr can at most be PATH_MAX or 1024, so at most
+		 * 4 characters plus NUL are necessary, so we won't
+		 * overrun, nor can we get a -1 with EOVERFLOW or EINVAL
+		 */
+		LogFatal(COMPONENT_CLIENTID,
+			 "snprintf returned unexpected %d", cidstr_lenx_len);
+	}
+
+	lsize = str_client_addr_len + 2 + cidstr_lenx_len + 1 + cidstr_len + 2;
 
 	/* hold both long form clientid and IP */
-	snprintf(val, total_len, "%s-(%s:%s)",
-		 str_client_addr, cidstr_len, cidstr);
+	val = gsh_malloc(lsize);
+	memcpy(val, str_client_addr, str_client_addr_len);
+	total_len = str_client_addr_len;
+	memcpy(val + total_len, "-(", 2);
+	total_len += 2;
+	memcpy(val + total_len, cidstr_lenx, cidstr_lenx_len);
+	total_len += cidstr_lenx_len;
+	val[total_len] = ':';
+	total_len += 1;
+	memcpy(val + total_len, cidstr, cidstr_len);
+	total_len += cidstr_lenx_len;
+	memcpy(val + total_len, ")", 2);
 
 	LogDebug(COMPONENT_CLIENTID, "Created client name [%s]", val);
+
+	if (size)
+		*size = lsize;
+
+	return val;
 }
 
 int rados_kv_put(char *key, char *val, char *object)
@@ -194,7 +220,8 @@ int rados_kv_get(char *key, char *val, char *object)
 		goto out;
 	}
 
-	strlcpy(val, val_out, val_len_out+1);
+	/* All internal so buffer length is known to be ok */
+	memcpy(val, val_out, val_len_out+1);
 	LogDebug(COMPONENT_CLIENTID, "%s: key=%s val=%s", __func__, key, val);
 	rados_omap_get_end(iter_vals);
 out:
@@ -284,7 +311,7 @@ static void rados_kv_append_val_rdfh(char *val, char *rdfh, int rdfh_len)
 	strncat(val, rdfhstr, buflen);
 }
 
-int rados_kv_set_param_from_conf(config_file_t parse_tree,
+int rados_load_config_from_parse(config_file_t parse_tree,
 				 struct config_error_type *err_type)
 {
 	(void) load_config_from_parse(parse_tree,
@@ -345,6 +372,8 @@ int rados_kv_connect(rados_ioctx_t *io_ctx, const char *userid,
 
 void rados_kv_shutdown(void)
 {
+	struct gsh_refstr *recov_oid;
+
 	if (rados_recov_io_ctx) {
 		rados_ioctx_destroy(rados_recov_io_ctx);
 		rados_recov_io_ctx = NULL;
@@ -353,28 +382,61 @@ void rados_kv_shutdown(void)
 		rados_shutdown(clnt);
 		clnt = NULL;
 	}
+	recov_oid = rcu_xchg_pointer(&rados_recov_oid, NULL);
+	synchronize_rcu();
+	if (recov_oid)
+		gsh_refstr_put(recov_oid);
 }
 
 int rados_kv_init(void)
 {
 	int ret;
+	size_t len, host_len;
+	struct gsh_refstr *recov_oid = NULL, *old_oid = NULL;
 	char host[NI_MAXHOST];
 
 	if (nfs_param.core_param.clustered) {
-		snprintf(host, sizeof(host), "node%d", g_nodeid);
+		ret = snprintf(host, sizeof(host), "node%d", g_nodeid);
+
+		if (unlikely(ret >= sizeof(host))) {
+			LogCrit(COMPONENT_CLIENTID,
+				"node%d too long", g_nodeid);
+			return -ENAMETOOLONG;
+		} else if (unlikely(ret < 0)) {
+			ret = errno;
+			LogCrit(COMPONENT_CLIENTID,
+				"Unexpected return from snprintf %d error %s (%d)",
+				ret, strerror(ret), ret);
+			return -ret;
+		}
 	} else {
 		ret = gethostname(host, sizeof(host));
+
 		if (ret) {
+			ret = errno;
 			LogEvent(COMPONENT_CLIENTID,
-				 "Failed to gethostname: %s",
-				 strerror(errno));
-			return -errno;
+				 "Failed to gethostname: %s (%d)",
+				 strerror(ret), ret);
+			return -ret;
 		}
 	}
 
-	snprintf(rados_recov_old_oid, sizeof(rados_recov_old_oid),
-		 "%s_old", host);
-	snprintf(rados_recov_oid, sizeof(rados_recov_oid), "%s_recov", host);
+	host_len = strlen(host);
+	len = host_len + 6 + 1;
+	recov_oid = gsh_refstr_alloc(len);
+	gsh_refstr_get(recov_oid);
+
+	/* Can't overrun and shouldn't return EOVERFLOW or EINVAL */
+	(void) snprintf(recov_oid->gr_val, len, "%s_recov", host);
+	rcu_set_pointer(&rados_recov_oid, recov_oid);
+
+	len = host_len + 4 + 1;
+	old_oid = gsh_refstr_alloc(len);
+	gsh_refstr_get(old_oid);
+
+	/* Can't overrun and shouldn't return EOVERFLOW or EINVAL */
+	(void) snprintf(old_oid->gr_val, len, "%s_old", host);
+	rcu_set_pointer(&rados_recov_old_oid, old_oid);
 
 	ret = rados_kv_connect(&rados_recov_io_ctx, rados_kv_param.userid,
 			rados_kv_param.ceph_conf, rados_kv_param.pool,
@@ -382,70 +444,79 @@ int rados_kv_init(void)
 	if (ret < 0) {
 		LogEvent(COMPONENT_CLIENTID,
 			"Failed to connect to cluster: %d", ret);
-		return ret;
+		goto out;
 	}
 
 	rados_write_op_t op = rados_create_write_op();
 
 	rados_write_op_create(op, LIBRADOS_CREATE_EXCLUSIVE, NULL);
 	ret = rados_write_op_operate(op, rados_recov_io_ctx,
-				     rados_recov_old_oid, NULL, 0);
+				     old_oid->gr_val, NULL, 0);
 	if (ret < 0 && ret != -EEXIST) {
 		LogEvent(COMPONENT_CLIENTID, "Failed to create object");
 		rados_release_write_op(op);
 		rados_kv_shutdown();
-		return ret;
+		goto out;
 	}
 	rados_release_write_op(op);
 
 	op = rados_create_write_op();
 	rados_write_op_create(op, LIBRADOS_CREATE_EXCLUSIVE, NULL);
-	ret = rados_write_op_operate(op, rados_recov_io_ctx, rados_recov_oid,
+	ret = rados_write_op_operate(op, rados_recov_io_ctx, recov_oid->gr_val,
 				     NULL, 0);
 	if (ret < 0 && ret != -EEXIST) {
 		LogEvent(COMPONENT_CLIENTID, "Failed to create object");
 		rados_release_write_op(op);
 		rados_kv_shutdown();
-		return ret;
+		goto out;
 	}
 	rados_release_write_op(op);
 
 	LogEvent(COMPONENT_CLIENTID, "Rados kv store init done");
-	return 0;
+	ret = 0;
+out:
+	gsh_refstr_put(recov_oid);
+	gsh_refstr_put(old_oid);
+	return ret;
 }
 
 void rados_kv_add_clid(nfs_client_id_t *clientid)
 {
 	char ckey[RADOS_KEY_MAX_LEN];
 	char *cval;
+	struct gsh_refstr *recov_oid;
 	int ret;
 
-	cval = gsh_malloc(RADOS_VAL_MAX_LEN);
+	rados_kv_create_key(clientid, ckey, sizeof(ckey));
+	cval = rados_kv_create_val(clientid, NULL);
 
-	rados_kv_create_key(clientid, ckey);
-	rados_kv_create_val(clientid, cval);
-
-	ret = rados_kv_put(ckey, cval, rados_recov_oid);
+	rcu_read_lock();
+	recov_oid = gsh_refstr_get(rcu_dereference(rados_recov_oid));
+	rcu_read_unlock();
+	ret = rados_kv_put(ckey, cval, recov_oid->gr_val);
+	gsh_refstr_put(recov_oid);
 	if (ret < 0) {
 		LogEvent(COMPONENT_CLIENTID, "Failed to add clid %lu",
 			 clientid->cid_clientid);
-		goto out;
+		gsh_free(cval);
+	} else {
+		clientid->cid_recov_tag = cval;
 	}
-
-	clientid->cid_recov_tag = gsh_malloc(strlen(cval) + 1);
-	strncpy(clientid->cid_recov_tag, cval, strlen(cval) + 1);
-out:
-	gsh_free(cval);
 }
 
 void rados_kv_rm_clid(nfs_client_id_t *clientid)
 {
 	char ckey[RADOS_KEY_MAX_LEN];
+	struct gsh_refstr *recov_oid;
 	int ret;
 
-	rados_kv_create_key(clientid, ckey);
+	rados_kv_create_key(clientid, ckey, sizeof(ckey));
 
-	ret = rados_kv_del(ckey, rados_recov_oid);
+	rcu_read_lock();
+	recov_oid = gsh_refstr_get(rcu_dereference(rados_recov_oid));
+	rcu_read_unlock();
+	ret = rados_kv_del(ckey, recov_oid->gr_val);
+	gsh_refstr_put(recov_oid);
 	if (ret < 0) {
 		LogEvent(COMPONENT_CLIENTID, "Failed to del clid %lu",
 			 clientid->cid_clientid);
@@ -462,6 +533,7 @@ static void rados_kv_pop_clid_entry(char *key, char *val, size_t val_len,
 	int ret;
 	char *dupval;
 	char *cl_name, *rfh_names, *rfh_name;
+	struct gsh_refstr *old_oid;
 	clid_entry_t *clid_ent;
 	add_clid_entry_hook add_clid_entry = pop_args->add_clid_entry;
 	add_rfh_entry_hook add_rfh_entry = pop_args->add_rfh_entry;
@@ -485,8 +557,11 @@ static void rados_kv_pop_clid_entry(char *key, char *val, size_t val_len,
 		rfh_name = strtok(NULL, "#");
 	}
 
+	rcu_read_lock();
+	old_oid = gsh_refstr_get(rcu_dereference(rados_recov_old_oid));
+	rcu_read_unlock();
 	if (!old) {
-		ret = rados_kv_put(key, dupval, rados_recov_old_oid);
+		ret = rados_kv_put(key, dupval, old_oid->gr_val);
 		if (ret < 0) {
 			LogEvent(COMPONENT_CLIENTID,
 				 "Failed to move %s", key);
@@ -496,15 +571,23 @@ static void rados_kv_pop_clid_entry(char *key, char *val, size_t val_len,
 
 	if (!takeover) {
 		if (old) {
-			ret = rados_kv_del(key, rados_recov_old_oid);
+			ret = rados_kv_del(key, old_oid->gr_val);
 		} else {
-			ret = rados_kv_del(key, rados_recov_oid);
+			struct gsh_refstr *recov_oid;
+
+			rcu_read_lock();
+			recov_oid = gsh_refstr_get(
+					rcu_dereference(rados_recov_oid));
+			rcu_read_unlock();
+			ret = rados_kv_del(key, recov_oid->gr_val);
+			gsh_refstr_put(recov_oid);
 		}
 		if (ret < 0) {
 			LogEvent(COMPONENT_CLIENTID,
 				 "Failed to del %s", key);
 		}
 	}
+	gsh_refstr_put(old_oid);
 }
 
 static void
@@ -512,6 +595,7 @@ rados_kv_read_recov_clids_recover(add_clid_entry_hook add_clid_entry,
 				       add_rfh_entry_hook add_rfh_entry)
 {
 	int ret;
+	struct gsh_refstr *recov_oid, *old_oid;
 	struct pop_args args = {
 		.add_clid_entry = add_clid_entry,
 		.add_rfh_entry = add_rfh_entry,
@@ -519,8 +603,12 @@ rados_kv_read_recov_clids_recover(add_clid_entry_hook add_clid_entry,
 		.takeover = false,
 	};
 
+	rcu_read_lock();
+	old_oid = gsh_refstr_get(rcu_dereference(rados_recov_old_oid));
+	rcu_read_unlock();
 	ret = rados_kv_traverse(rados_kv_pop_clid_entry, &args,
-				rados_recov_old_oid);
+				old_oid->gr_val);
+	gsh_refstr_put(old_oid);
 	if (ret < 0) {
 		LogEvent(COMPONENT_CLIENTID,
 			 "Failed to recover, processing old entries");
@@ -528,8 +616,12 @@ rados_kv_read_recov_clids_recover(add_clid_entry_hook add_clid_entry,
 	}
 
 	args.old = false;
+	rcu_read_lock();
+	recov_oid = gsh_refstr_get(rcu_dereference(rados_recov_oid));
+	rcu_read_unlock();
 	ret = rados_kv_traverse(rados_kv_pop_clid_entry, &args,
-				rados_recov_oid);
+				recov_oid->gr_val);
+	gsh_refstr_put(recov_oid);
 	if (ret < 0) {
 		LogEvent(COMPONENT_CLIENTID,
 			 "Failed to recover, processing recov entries");
@@ -555,7 +647,18 @@ void rados_kv_read_recov_clids_takeover(nfs_grace_start_t *gsp,
 		return;
 	}
 
-	snprintf(object_takeover, NI_MAXHOST, "%s_recov", gsp->ipaddr);
+	ret = snprintf(object_takeover, sizeof(object_takeover), "%s_recov",
+		       gsp->ipaddr);
+
+	if (unlikely(ret >= sizeof(object_takeover))) {
+		LogCrit(COMPONENT_CLIENTID,
+			"object_takeover too long %s_recov", gsp->ipaddr);
+	} else if (unlikely(ret < 0)) {
+		LogCrit(COMPONENT_CLIENTID,
+			"Unexpected return from snprintf %d error %s (%d)",
+			ret, strerror(errno), errno);
+	}
+
 	ret = rados_kv_traverse(rados_kv_pop_clid_entry, &args,
 				object_takeover);
 	if (ret < 0) {
@@ -566,15 +669,20 @@ void rados_kv_read_recov_clids_takeover(nfs_grace_start_t *gsp,
 void rados_kv_cleanup_old(void)
 {
 	int ret;
+	struct gsh_refstr *old_oid;
 	rados_write_op_t write_op = rados_create_write_op();
 
+	rcu_read_lock();
+	old_oid = gsh_refstr_get(rcu_dereference(rados_recov_old_oid));
+	rcu_read_unlock();
+
 	rados_write_op_omap_clear(write_op);
 	ret = rados_write_op_operate(write_op, rados_recov_io_ctx,
-				     rados_recov_old_oid, NULL, 0);
-	if (ret < 0) {
-		LogEvent(COMPONENT_CLIENTID, "Failed to clearup old");
-	}
+				     old_oid->gr_val, NULL, 0);
+	if (ret < 0)
+		LogEvent(COMPONENT_CLIENTID, "Failed to cleanup old");
 	rados_release_write_op(write_op);
+	gsh_refstr_put(old_oid);
 }
 
 void rados_kv_add_revoke_fh(nfs_client_id_t *delr_clid, nfs_fh4 *delr_handle)
@@ -582,11 +690,15 @@ void rados_kv_add_revoke_fh(nfs_client_id_t *delr_clid, nfs_fh4 *delr_handle)
 	int ret;
 	char ckey[RADOS_KEY_MAX_LEN];
 	char *cval;
+	struct gsh_refstr *recov_oid;
 
 	cval = gsh_malloc(RADOS_VAL_MAX_LEN);
 
-	rados_kv_create_key(delr_clid, ckey);
-	ret = rados_kv_get(ckey, cval, rados_recov_oid);
+	rados_kv_create_key(delr_clid, ckey, sizeof(ckey));
+	rcu_read_lock();
+	recov_oid = gsh_refstr_get(rcu_dereference(rados_recov_oid));
+	rcu_read_unlock();
+	ret = rados_kv_get(ckey, cval, recov_oid->gr_val);
 	if (ret < 0) {
 		LogEvent(COMPONENT_CLIENTID, "Failed to get %s", ckey);
 		goto out;
@@ -597,14 +709,14 @@ void rados_kv_add_revoke_fh(nfs_client_id_t *delr_clid, nfs_fh4 *delr_handle)
 	rados_kv_append_val_rdfh(cval, delr_handle->nfs_fh4_val,
 				      delr_handle->nfs_fh4_len);
 
-	ret = rados_kv_put(ckey, cval, rados_recov_oid);
+	ret = rados_kv_put(ckey, cval, recov_oid->gr_val);
 	if (ret < 0) {
 		LogEvent(COMPONENT_CLIENTID,
 			 "Failed to add rdfh for clid %lu",
 			 delr_clid->cid_clientid);
 	}
-
 out:
+	gsh_refstr_put(recov_oid);
 	gsh_free(cval);
 }
 
