@@ -37,6 +37,7 @@
 #include "config.h"
 #include <netdb.h>
 #include <rados/librados.h>
+#include <urcu-bp.h>
 #include "log.h"
 #include "nfs_core.h"
 #include "sal_functions.h"
@@ -114,27 +115,48 @@ static int rados_ng_del(char *key, char *object)
 static int rados_ng_init(void)
 {
 	int ret;
+	size_t len;
+	struct gsh_refstr *recov_oid;
 	char host[NI_MAXHOST];
 	rados_write_op_t op;
 
 	if (nfs_param.core_param.clustered) {
-		snprintf(host, sizeof(host), "node%d", g_nodeid);
+		ret = snprintf(host, sizeof(host), "node%d", g_nodeid);
+
+		if (unlikely(ret >= sizeof(host))) {
+			LogCrit(COMPONENT_CLIENTID,
+				"node%d too long", g_nodeid);
+			return -ENAMETOOLONG;
+		} else if (unlikely(ret < 0)) {
+			ret = errno;
+			LogCrit(COMPONENT_CLIENTID,
+				"Unexpected return from snprintf %d error %s (%d)",
+				ret, strerror(ret), ret);
+			return ret;
+		}
 	} else {
 		ret = gethostname(host, sizeof(host));
 		if (ret) {
 			LogEvent(COMPONENT_CLIENTID,
-				 "Failed to gethostname: %s",
-				 strerror(errno));
+				 "Failed to gethostname: %s (%d)",
+				 strerror(errno), errno);
 			return -errno;
 		}
 	}
 
-	snprintf(rados_recov_oid, sizeof(rados_recov_oid), "%s.recov", host);
+	len = strlen(host) + 6 + 1;
+	recov_oid = gsh_refstr_alloc(len);
+	gsh_refstr_get(recov_oid);
+
+	/* Can't overrun and shouldn't return EOVERFLOW or EINVAL */
+	(void) snprintf(recov_oid->gr_val, len, "%s_recov", host);
+	rcu_set_pointer(&rados_recov_oid, recov_oid);
 
 	ret = rados_kv_connect(&rados_recov_io_ctx, rados_kv_param.userid,
 			rados_kv_param.ceph_conf, rados_kv_param.pool,
 			rados_kv_param.namespace);
 	if (ret < 0) {
+		gsh_refstr_put(recov_oid);
 		LogEvent(COMPONENT_CLIENTID,
 			"Failed to connect to cluster: %d", ret);
 		return ret;
@@ -142,8 +164,9 @@ static int rados_ng_init(void)
 
 	op = rados_create_write_op();
 	rados_write_op_create(op, LIBRADOS_CREATE_EXCLUSIVE, NULL);
-	ret = rados_write_op_operate(op, rados_recov_io_ctx, rados_recov_oid,
+	ret = rados_write_op_operate(op, rados_recov_io_ctx, recov_oid->gr_val,
 				     NULL, 0);
+	gsh_refstr_put(recov_oid);
 	if (ret < 0 && ret != -EEXIST) {
 		LogEvent(COMPONENT_CLIENTID, "Failed to create object");
 		rados_release_write_op(op);
@@ -162,38 +185,43 @@ static int rados_ng_init(void)
 
 static void rados_ng_add_clid(nfs_client_id_t *clientid)
 {
+	struct gsh_refstr *recov_oid;
 	char ckey[RADOS_KEY_MAX_LEN];
 	char *cval;
 	int ret;
 
-	cval = gsh_malloc(RADOS_VAL_MAX_LEN);
-
-	rados_kv_create_key(clientid, ckey);
-	rados_kv_create_val(clientid, cval);
+	rados_kv_create_key(clientid, ckey, sizeof(ckey));
+	cval = rados_kv_create_val(clientid, NULL);
 
 	LogDebug(COMPONENT_CLIENTID, "adding %s :: %s", ckey, cval);
-	ret = rados_ng_put(ckey, cval, rados_recov_oid);
+	rcu_read_lock();
+	recov_oid = gsh_refstr_get(rcu_dereference(rados_recov_oid));
+	rcu_read_unlock();
+	ret = rados_ng_put(ckey, cval, recov_oid->gr_val);
+	gsh_refstr_put(recov_oid);
 	if (ret < 0) {
 		LogEvent(COMPONENT_CLIENTID, "Failed to add clid %lu",
 			 clientid->cid_clientid);
-		goto out;
+		gsh_free(cval);
+	} else {
+		clientid->cid_recov_tag = cval;
 	}
-
-	clientid->cid_recov_tag = gsh_malloc(strlen(cval) + 1);
-	strncpy(clientid->cid_recov_tag, cval, strlen(cval) + 1);
-out:
-	gsh_free(cval);
 }
 
 static void rados_ng_rm_clid(nfs_client_id_t *clientid)
 {
 	char ckey[RADOS_KEY_MAX_LEN];
 	int ret;
+	struct gsh_refstr *recov_oid;
 
-	rados_kv_create_key(clientid, ckey);
+	rados_kv_create_key(clientid, ckey, sizeof(ckey));
 
 	LogDebug(COMPONENT_CLIENTID, "removing %s", ckey);
-	ret = rados_ng_del(ckey, rados_recov_oid);
+	rcu_read_lock();
+	recov_oid = gsh_refstr_get(rcu_dereference(rados_recov_oid));
+	rcu_read_unlock();
+	ret = rados_ng_del(ckey, recov_oid->gr_val);
+	gsh_refstr_put(recov_oid);
 	if (ret < 0) {
 		LogEvent(COMPONENT_CLIENTID, "Failed to del clid %lu",
 			 clientid->cid_clientid);
@@ -204,8 +232,8 @@ static void rados_ng_rm_clid(nfs_client_id_t *clientid)
 	clientid->cid_recov_tag = NULL;
 }
 
-static void rados_ng_pop_clid_entry(char *key, char *val,
-				    struct pop_args *pop_args)
+void rados_ng_pop_clid_entry(char *key, char *val, size_t val_len,
+			     struct pop_args *pop_args)
 {
 	char *dupval, *cl_name;
 	char *rfh_names, *rfh_name;
@@ -214,7 +242,9 @@ static void rados_ng_pop_clid_entry(char *key, char *val,
 	add_rfh_entry_hook add_rfh_entry = pop_args->add_rfh_entry;
 
 	/* extract clid records */
-	dupval = gsh_strdup(val);
+	dupval = gsh_malloc(val_len +  1);
+	memcpy(dupval, val, val_len);
+	dupval[val_len] = '\0';
 	cl_name = strtok(dupval, "#");
 	if (!cl_name)
 		cl_name = dupval;
@@ -234,13 +264,18 @@ rados_ng_read_recov_clids_recover(add_clid_entry_hook add_clid_entry,
 				       add_rfh_entry_hook add_rfh_entry)
 {
 	int ret;
+	struct gsh_refstr *recov_oid;
 	struct pop_args args = {
 		.add_clid_entry = add_clid_entry,
 		.add_rfh_entry = add_rfh_entry,
 	};
 
+	rcu_read_lock();
+	recov_oid = gsh_refstr_get(rcu_dereference(rados_recov_oid));
+	rcu_read_unlock();
 	ret = rados_kv_traverse(rados_ng_pop_clid_entry, &args,
-				rados_recov_oid);
+				recov_oid->gr_val);
+	gsh_refstr_put(recov_oid);
 	if (ret < 0) {
 		LogEvent(COMPONENT_CLIENTID,
 			 "Failed to recover, processing old entries");
@@ -265,11 +300,16 @@ static void rados_ng_read_recov_clids_takeover(nfs_grace_start_t *gsp,
 static void rados_ng_cleanup_old(void)
 {
 	int ret;
+	struct gsh_refstr *recov_oid;
 
 	/* Commit pregrace transaction */
 	PTHREAD_MUTEX_lock(&grace_op_lock);
+	rcu_read_lock();
+	recov_oid = gsh_refstr_get(rcu_dereference(rados_recov_oid));
+	rcu_read_unlock();
 	ret = rados_write_op_operate(grace_op, rados_recov_io_ctx,
-				     rados_recov_oid, NULL, 0);
+				     recov_oid->gr_val, NULL, 0);
+	gsh_refstr_put(recov_oid);
 	if (ret < 0) {
 		LogEvent(COMPONENT_CLIENTID,
 			 "Failed to commit grace period transactions: %s",
