@@ -52,6 +52,8 @@
 #include "server_stats.h"
 #include "fsal_up.h"
 #include "nfs_file_handle.h"
+#include "nfs_convert.h"
+#include "fsal_convert.h"
 
 /**
  * @brief Initialize new delegation state as argument for state_add()
@@ -139,6 +141,7 @@ state_status_t acquire_lease_lock(struct state_hdl *ostate,
 
 	if (status == STATE_SUCCESS) {
 		update_delegation_stats(ostate, owner);
+		reset_cbgetattr_stats(ostate->file.obj);
 	} else {
 		LogDebug(COMPONENT_STATE, "Could not set lease, error=%s",
 			 state_err_str(status));
@@ -206,6 +209,17 @@ static int advance_avg(time_t prev_avg, time_t new_time,
 	return ((prev_tot * prev_avg) + new_time) / curr_tot;
 }
 
+/*
+ * @brief reset cbgetattr struct args
+ */
+void reset_cbgetattr_stats(struct fsal_obj_handle *obj)
+{
+	cbgetattr_t *cbgetattr = &obj->state_hdl->file.cbgetattr;
+
+	cbgetattr->state = CB_GETATTR_NONE;
+	cbgetattr->modified = false;
+}
+
 /**
  * @brief Update statistics on successfully recalled delegation.
  *
@@ -427,6 +441,7 @@ nfsstat4 deleg_revoke(struct fsal_obj_handle *obj, struct state_t *deleg_state)
 	(void) nfs4_FSALToFhandle(true, &fhandle, obj, export);
 
 	deleg_heuristics_recall(obj, owner, deleg_state);
+	reset_cbgetattr_stats(obj);
 
 	/* Build op_context for state_unlock_locked */
 	init_root_op_context(&root_op_context, NULL, NULL, 0, 0,
@@ -482,6 +497,43 @@ void state_deleg_revoke(struct fsal_obj_handle *obj, state_t *state)
 	(void)deleg_revoke(obj, state);
 }
 
+/**
+ * @brief Check if the file is write delegated under state_lock
+ *
+ * Check if the file is write delegated. If yes, take a ref and return
+ * the client holding the delegation.
+ *
+ * @note: The caller should acquire state_lock before calling this
+ * function.
+ *
+ * @param[in] obj File
+ * @param[out] client holding the delegation
+ *
+ * @retval true if file is write delegated
+ * @retval false otherwise
+ */
+bool is_write_delegated(struct fsal_obj_handle *obj, nfs_client_id_t **client)
+{
+	bool write_delegated = false;
+	struct file_deleg_stats *deleg_stats;
+
+	if (obj->type != REGULAR_FILE)
+		return false;
+
+	deleg_stats = &obj->state_hdl->file.fdeleg_stats;
+
+	if (deleg_stats->fds_curr_delegations < 0)
+		return false;
+
+	write_delegated = obj->state_hdl->file.write_delegated;
+	if (write_delegated && client) {
+		*client = obj->state_hdl->file.write_deleg_client;
+		inc_client_id_ref(*client);
+	}
+
+	return write_delegated;
+}
+
 /**
  * @brief Check if an operation is conflicting with delegations.
  *
@@ -498,18 +550,25 @@ void state_deleg_revoke(struct fsal_obj_handle *obj, state_t *state)
  * @retval true if there is a conflict and the delegations have been recalled.
  * @retval false if there is no delegation conflict.
  */
-bool state_deleg_conflict(struct fsal_obj_handle *obj, bool write)
+bool state_deleg_conflict_impl(struct fsal_obj_handle *obj, bool write)
 {
 	struct file_deleg_stats *deleg_stats;
+	struct gsh_client *deleg_client = NULL;
 
 	if (obj->type != REGULAR_FILE)
 		return false;
 
 	deleg_stats = &obj->state_hdl->file.fdeleg_stats;
+
+	if (obj->state_hdl->file.write_delegated)
+		deleg_client =
+			obj->state_hdl->file.write_deleg_client->gsh_client;
+
 	if (deleg_stats->fds_curr_delegations > 0
 	    && ((deleg_stats->fds_deleg_type == OPEN_DELEGATE_READ
 		 && write)
-		|| (deleg_stats->fds_deleg_type == OPEN_DELEGATE_WRITE))
+		|| (deleg_stats->fds_deleg_type == OPEN_DELEGATE_WRITE &&
+		    deleg_client != op_ctx->client))
 	    ) {
 		LogDebug(COMPONENT_STATE,
 			 "While trying to perform a %s op, found a conflicting %s delegation",
@@ -524,6 +583,91 @@ bool state_deleg_conflict(struct fsal_obj_handle *obj, bool write)
 	return false;
 }
 
+/**
+ * @brief Acquire state_lock and check if an operation is conflicting
+ *        with delegations.
+ *
+ * @param[in] obj   File
+ * @param[in] write a boolean indicating whether the operation will read or
+ *            change the file.
+ *
+ * @retval true if there is a conflict and the delegations have been recalled.
+ * @retval false if there is no delegation conflict.
+ */
+bool state_deleg_conflict(struct fsal_obj_handle *obj, bool write)
+{
+	bool status = false;
+
+	PTHREAD_RWLOCK_rdlock(&obj->state_hdl->state_lock);
+	status = state_deleg_conflict_impl(obj, write);
+	PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
+	return status;
+}
+
+/*
+ * @brief: fetch getattr from the write_delegated client
+ *
+ * Send CB_GETATTR to the write_delegated client to fetch
+ * right attributes. If not recall delegation.
+ *
+ * @note: should be called under state_lock.
+ */
+nfsstat4 handle_deleg_getattr(struct fsal_obj_handle *obj,
+			      nfs_client_id_t *client)
+{
+	nfsstat4 status = NFS4ERR_DELAY;
+	int rc = 0;
+	enum cbgetattr_state cb_state;
+
+	/* Check for delegation conflict.*/
+	LogDebug(COMPONENT_STATE,
+		 "While trying to perform a GETATTR op, found a conflicting WRITE delegation");
+
+	/*
+	 * @todo: Provide an option for user to enable CB_GETATTR
+	 */
+
+	cb_state = obj->state_hdl->file.cbgetattr.state;
+	switch (cb_state) {
+	case CB_GETATTR_RSP_OK:
+		/* got response for CB_GETATTR */
+		status = NFS4_OK;
+		goto out;
+	case CB_GETATTR_WIP:
+		/* wait for response */
+		goto out;
+	case CB_GETATTR_FAILED:
+		goto deleg_recall;
+	default: /* CB_GETATTR_NONE */
+		goto send_request;
+	}
+send_request:
+	LogDebug(COMPONENT_STATE, "sending CB_GETATTR");
+	rc = async_cbgetattr(general_fridge, obj, client);
+	if (rc != 0) {
+		LogCrit(COMPONENT_STATE,
+			"Failed to start thread to send cb_getattr.");
+		goto deleg_recall;
+	}
+	goto out;
+deleg_recall:
+	LogDebug(COMPONENT_STATE, "CB_GETATTR is either not enabled or failed,"
+				   " recalling write delegation");
+	rc = async_delegrecall(general_fridge, obj);
+	if (rc != 0) {
+		LogCrit(COMPONENT_STATE,
+			"Failed to start thread to recall delegation from conflicting operation.");
+		goto out;
+	}
+
+out:
+	if (rc != 0) {
+		status = nfs4_Errno_status(fsalstat(posix2fsal_error(rc),
+						    rc));
+	}
+	return status;
+}
+
 bool deleg_supported(struct fsal_obj_handle *obj,
 		     struct fsal_export *fsal_export,
 		     struct export_perms *export_perms, uint32_t share_access)
