@@ -116,49 +116,6 @@ static state_status_t do_lock_op(struct fsal_obj_handle *obj,
  * @brief Blocking lock cookies
  */
 
-/**
- * Parameters used for lock cookie hash table initialization.
- *
- * @todo Switch the cookie table to something else and get rid
- * of this.
- */
-
-static hash_parameter_t cookie_param = {
-	.index_size = PRIME_STATE,
-	.hash_func_key = lock_cookie_value_hash_func,
-	.hash_func_rbt = lock_cookie_rbt_hash_func,
-	.compare_key = compare_lock_cookie_key,
-	.key_to_str = display_lock_cookie_key,
-	.val_to_str = display_lock_cookie_val,
-	.flags = HT_FLAG_NONE,
-};
-
-static hash_table_t *ht_lock_cookies;
-
-/**
- * @brief Initalize locking
- *
- * @return State status.
- */
-state_status_t state_lock_init(void)
-{
-	state_status_t status = STATE_SUCCESS;
-
-	ht_lock_cookies = hashtable_init(&cookie_param);
-	if (ht_lock_cookies == NULL) {
-		LogCrit(COMPONENT_STATE, "Cannot init NLM Client cache");
-		status = STATE_INIT_ENTRY_FAILED;
-		return status;
-	}
-
-	status = state_async_init();
-
-	state_owner_pool =
-		pool_basic_init("NFSv4 state owners", sizeof(state_owner_t));
-
-	return status;
-}
-
 /**
  * @brief Check whether a lock is from NLM
  *
@@ -1154,20 +1111,22 @@ static state_status_t subtract_lock_from_list(state_owner_t *owner,
 
 static void grant_blocked_locks(struct state_hdl *);
 
+static inline int display_lock_cookie(struct display_buffer *dspbuf,
+				      struct gsh_buffdesc *buff)
+{
+	return display_opaque_value(dspbuf, buff->addr, buff->len);
+}
+
 /**
  * @brief Display lock cookie in hash table
  *
- * @param[in]  buff Key to display
- * @param[out] str  Output buffer
- *
- * @return Length of output string.
+ * @param[in]  dspbuf display buffer to display into
+ * @param[in]  buff   Key to display
  */
-int display_lock_cookie_key(struct gsh_buffdesc *buff, char *str)
+int display_lock_cookie_key(struct display_buffer *dspbuf,
+			    struct gsh_buffdesc *buff)
 {
-	struct display_buffer dspbuf = {HASHTABLE_DISPLAY_STRLEN, str, str};
-
-	display_lock_cookie(&dspbuf, buff);
-	return display_buffer_len(&dspbuf);
+	return display_lock_cookie(dspbuf, buff);
 }
 
 /**
@@ -1229,17 +1188,13 @@ int display_lock_cookie_entry(struct display_buffer *dspbuf,
 /**
  * @brief Display lock cookie entry in hash table
  *
- * @param[in]  buff Value to display
- * @param[out] str  Output buffer
- *
- * @return Length of output string.
+ * @param[in]  dspbuf display buffer to display into
+ * @param[in]  buff   Value to display
  */
-int display_lock_cookie_val(struct gsh_buffdesc *buff, char *str)
+int display_lock_cookie_val(struct display_buffer *dspbuf,
+			    struct gsh_buffdesc *buff)
 {
-	struct display_buffer dspbuf = {HASHTABLE_DISPLAY_STRLEN, str, str};
-
-	display_lock_cookie_entry(&dspbuf, buff->addr);
-	return display_buffer_len(&dspbuf);
+	return display_lock_cookie_entry(dspbuf, buff->addr);
 }
 
 /**
@@ -1376,6 +1331,25 @@ void free_cookie(state_cookie_entry_t *cookie_entry, bool unblock)
 	gsh_free(cookie_entry);
 }
 
+/**
+ * Parameters used for lock cookie hash table initialization.
+ *
+ * @todo Switch the cookie table to something else and get rid
+ * of this.
+ */
+
+static hash_parameter_t cookie_param = {
+	.index_size = PRIME_STATE,
+	.hash_func_key = lock_cookie_value_hash_func,
+	.hash_func_rbt = lock_cookie_rbt_hash_func,
+	.compare_key = compare_lock_cookie_key,
+	.display_key = display_lock_cookie_key,
+	.display_val = display_lock_cookie_val,
+	.flags = HT_FLAG_NONE,
+};
+
+static hash_table_t *ht_lock_cookies;
+
 /**
  * @brief Add a grant cookie to a blocked lock
  *
@@ -2309,14 +2283,14 @@ state_status_t state_test(struct fsal_obj_handle *obj,
  *
  * Must hold the state_lock
  *
- * @param[in]  obj        File to lock
- * @param[in]  owner      Lock owner
- * @param[in]  state      Associated state for the lock
- * @param[in]  blocking   Blocking type
- * @param[in]  block_data Blocking lock data
- * @param[in]  lock       Lock description
- * @param[out] holder     Holder of conflicting lock
- * @param[out] conflict   Conflicting lock description
+ * @param[in]     obj        File to lock
+ * @param[in]     owner      Lock owner
+ * @param[in]     state      Associated state for the lock
+ * @param[in]     blocking   Blocking type
+ * @param[in,out] bdata      Blocking lock data
+ * @param[in]     lock       Lock description
+ * @param[out]    holder     Holder of conflicting lock
+ * @param[out]    conflict   Conflicting lock description
  *
  * @return State status.
  */
@@ -2324,7 +2298,7 @@ state_status_t state_lock(struct fsal_obj_handle *obj,
 			  state_owner_t *owner,
 			  state_t *state,
 			  state_blocking_t blocking,
-			  state_block_data_t *block_data,
+			  state_block_data_t **bdata,
 			  fsal_lock_param_t *lock,
 			  state_owner_t **holder,
 			  fsal_lock_param_t *conflict)
@@ -2338,6 +2312,7 @@ state_status_t state_lock(struct fsal_obj_handle *obj,
 	fsal_lock_op_t lock_op;
 	state_status_t status = 0;
 	bool async;
+	state_block_data_t *block_data;
 
 	if (blocking != STATE_NON_BLOCKING) {
 		/* First search for a blocked request. Client can ignore the
@@ -2429,7 +2404,10 @@ state_status_t state_lock(struct fsal_obj_handle *obj,
 				LogEntry("Conflicts with", found_entry);
 				LogList("Locks", obj,
 					&obj->state_hdl->file.lock_list);
-				copy_conflict(found_entry, holder, conflict);
+				if (blocking != STATE_NLM_BLOCKING) {
+					copy_conflict(found_entry, holder,
+						      conflict);
+				}
 				allow = false;
 				overlap = true;
 				break;
@@ -2493,7 +2471,7 @@ state_status_t state_lock(struct fsal_obj_handle *obj,
 		 */
 		lock_op = FSAL_OP_LOCK;
 	} else {
-		/* Can't do async blocking lock in FSAL and have a conflict.
+		/* This is not a blocking lock and has a conflict.
 		 * Return it.
 		 */
 		status = STATE_LOCK_CONFLICT;
@@ -2608,6 +2586,12 @@ state_status_t state_lock(struct fsal_obj_handle *obj,
 		/* Discard lock entry */
 		remove_from_locklist(found_entry);
 	} else if (status == STATE_LOCK_BLOCKED) {
+		/* We are going to use the bdata, set it to NULL so that
+		 * the caller doesn't free it!
+		 */
+		block_data = *bdata;
+		*bdata = NULL;
+
 		/* Mark entry as blocking and attach block_data */
 		found_entry->sle_block_data = block_data;
 		found_entry->sle_blocked = blocking;
@@ -3015,7 +2999,7 @@ state_status_t state_nlm_notify(state_nsm_client_t *nsmclient,
 		found_share =
 			glist_first_entry(&nsmclient->ssc_share_list,
 					  state_t,
-					  state_data.share.share_lockstates);
+					  state_data.nlm_share.share_perclient);
 
 		/* If we don't find any entries, then we are done. */
 		if (found_share == NULL) {
@@ -3058,8 +3042,9 @@ state_status_t state_nlm_notify(state_nsm_client_t *nsmclient,
 		/* Move this entry to the end of the list
 		 * (this will help if errors occur)
 		 */
+		glist_del(&found_share->state_data.nlm_share.share_perclient);
 		glist_add_tail(&nsmclient->ssc_share_list,
-			       &found_share->state_data.share.share_lockstates);
+			  &found_share->state_data.nlm_share.share_perclient);
 
 		PTHREAD_MUTEX_unlock(&nsmclient->ssc_mutex);
 
@@ -3068,8 +3053,8 @@ state_status_t state_nlm_notify(state_nsm_client_t *nsmclient,
 			 * Owner on the file (on all exports)
 			 */
 			status = state_nlm_share(obj,
-						 OPEN4_SHARE_ACCESS_NONE,
-						 OPEN4_SHARE_DENY_NONE,
+						 OPEN4_SHARE_ACCESS_ALL,
+						 OPEN4_SHARE_DENY_ALL,
 						 owner,
 						 found_share,
 						 false,
@@ -3577,4 +3562,28 @@ out:
 	release_root_op_context();
 }
 
+/**
+ * @brief Initialize locking
+ *
+ * @return State status.
+ */
+state_status_t state_lock_init(void)
+{
+	state_status_t status = STATE_SUCCESS;
+
+	ht_lock_cookies = hashtable_init(&cookie_param);
+	if (ht_lock_cookies == NULL) {
+		LogCrit(COMPONENT_STATE, "Cannot init NLM Client cache");
+		status = STATE_INIT_ENTRY_FAILED;
+		return status;
+	}
+
+	status = state_async_init();
+
+	state_owner_pool =
+		pool_basic_init("NFSv4 state owners", sizeof(state_owner_t));
+
+	return status;
+}
+
 /** @} */
