@@ -124,6 +124,7 @@ state_status_t state_nlm_share(struct fsal_obj_handle *obj,
 	unsigned int new_deny = 0;
 	struct state_nlm_share *nlm_share = &state->state_data.nlm_share;
 	int i, acount = 0, dcount = 0;
+	fsal_accessflags_t access_mask = 0;
 
 	PTHREAD_RWLOCK_wrlock(&obj->state_hdl->state_lock);
 
@@ -136,19 +137,26 @@ state_status_t state_nlm_share(struct fsal_obj_handle *obj,
 			share_access, share_deny);
 
 	if (unshare) {
-		if (nlm_share->share_access_counts[share_access] > 0)
+		/* For FREE_ALL case need to release all shares */
+		if (share_access == OPEN4_SHARE_ACCESS_ALL)
+			for (i = 0; i <= fsa_RW; i++)
+				nlm_share->share_access_counts[i] = 0;
+		else if (nlm_share->share_access_counts[share_access] > 0)
 			nlm_share->share_access_counts[share_access]--;
 		else
 			LogDebugAlt(COMPONENT_STATE, COMPONENT_NLM,
 				    "UNSHARE access %d did not match",
 				    share_access);
 
-		if (nlm_share->share_deny_counts[share_deny] > 0)
+		if (share_deny == OPEN4_SHARE_DENY_ALL)
+			for (i = 0; i <= fsm_DRW; i++)
+				nlm_share->share_deny_counts[i] = 0;
+		else if (nlm_share->share_deny_counts[share_deny] > 0)
 			nlm_share->share_deny_counts[share_deny]--;
 		else
 			LogDebugAlt(COMPONENT_STATE, COMPONENT_NLM,
 				    "UNSHARE deny %d did not match",
-				    share_access);
+				    share_deny);
 	} else {
 		nlm_share->share_access_counts[share_access]++;
 		nlm_share->share_deny_counts[share_deny]++;
@@ -170,15 +178,22 @@ state_status_t state_nlm_share(struct fsal_obj_handle *obj,
 		}
 	}
 
-	LogFullDebugAlt(COMPONENT_STATE, COMPONENT_NLM,
-			"%s share_access_counts[%d] = %d, total = %d, share_deny_counts[%d] = %d, total = %d",
-			unshare ? "UNSHARE" : "SHARE",
-			share_access,
-			nlm_share->share_access_counts[share_access],
-			acount,
-			share_deny,
-			nlm_share->share_deny_counts[share_deny],
-			dcount);
+	if (share_access == OPEN4_SHARE_ACCESS_ALL ||
+	    share_deny == OPEN4_SHARE_DENY_ALL) {
+		LogFullDebugAlt(COMPONENT_STATE, COMPONENT_NLM,
+				"%s share access total = %d, share deny total = %d",
+				unshare ? "UNSHARE" : "SHARE", acount, dcount);
+	} else {
+		LogFullDebugAlt(COMPONENT_STATE, COMPONENT_NLM,
+				"%s share_access_counts[%d] = %d, total = %d, share_deny_counts[%d] = %d, total = %d",
+				unshare ? "UNSHARE" : "SHARE",
+				share_access,
+				nlm_share->share_access_counts[share_access],
+				acount,
+				share_deny,
+				nlm_share->share_deny_counts[share_deny],
+				dcount);
+	}
 
 	if (new_access == old_access && new_deny == old_deny) {
 		/* The share or unshare did not affect the union of shares so
@@ -226,10 +241,31 @@ state_status_t state_nlm_share(struct fsal_obj_handle *obj,
 	if (reclaim)
 		openflags |= FSAL_O_RECLAIM;
 
+
+	if (openflags & FSAL_O_READ)
+		access_mask |= FSAL_READ_ACCESS;
+
+	if (openflags & FSAL_O_WRITE)
+		access_mask |= FSAL_WRITE_ACCESS;
+
+	/* The access check must be same as the read and write calls.
+	 * Use owner_skip for the access checks
+	 * The reopen2 call does not use the owner_skip
+	 * perform test_access first and then call reopen2.
+	 */
+	fsal_status = obj->obj_ops->test_access(obj, access_mask,
+						NULL, NULL, true);
+	if (FSAL_IS_ERROR(fsal_status)) {
+		LogDebug(COMPONENT_STATE,
+			 "test_access failed with %s",
+			 fsal_err_txt(fsal_status));
+		goto out_unlock;
+	}
+
 	/* Use reopen2 to open or re-open the file and check for share
 	 * conflict.
 	 */
-	fsal_status = fsal_reopen2(obj, state, openflags, true);
+	fsal_status = fsal_reopen2(obj, state, openflags, false);
 
 	if (FSAL_IS_ERROR(fsal_status)) {
 		LogDebugAlt(COMPONENT_STATE, COMPONENT_NLM,
@@ -359,8 +395,8 @@ void state_export_unshare_all(void)
 
 		/* Remove all shares held by this Owner on this export */
 		status = state_nlm_share(obj,
-					 OPEN4_SHARE_ACCESS_BOTH,
-					 OPEN4_SHARE_DENY_BOTH,
+					 OPEN4_SHARE_ACCESS_ALL,
+					 OPEN4_SHARE_DENY_ALL,
 					 owner,
 					 state,
 					 false,
