@@ -890,8 +890,10 @@ static bool do_block_init(struct config_node *blk_node,
 						  gai_strerror(rc));
 				errors++;
 			}
-			if (res != NULL)
+			if (res != NULL) {
 				freeaddrinfo(res);
+				res = NULL;
+			}
 			break;
 		case CONFIG_BLOCK:
 			(void) item->u.blk.init(NULL, param_addr);
@@ -984,7 +986,7 @@ static int do_block_load(struct config_node *blk,
 	void *param_addr;
 	struct config_node *node, *term_node, *next_node = NULL;
 	struct glist_head *ns;
-	int errors = 0;
+	int errors = 0, prev_errors = err_type->errors;
 
 	for (item = params; item->name != NULL; item++) {
 		uint64_t num64;
@@ -1259,6 +1261,16 @@ static int do_block_load(struct config_node *blk,
 			node = next_node;
 		}
 	}
+
+	/* Check for any errors in parsing params.
+	 * It will set to default value if parsing fails.
+	 * For params like Export_Id if parsing fails, we
+	 * will end up setting it to 1, which could cause issue
+	 * if we set it to 1 for multiple exports.
+	 */
+	if (err_type->errors > prev_errors)
+		errors = err_type->errors;
+
 	if (relax)
 		return errors;
 
@@ -1674,9 +1686,8 @@ static char *parse_expr(char *expr, struct expr_parse **expr_node)
 {
 	char *sp;
 	struct expr_parse *node = NULL, *prev_node = NULL, *root_node = NULL;
-	char *lexpr = alloca(strlen(expr) + 1);
+	char *lexpr = gsh_strdupa(expr);
 
-	strcpy(lexpr, expr);
 	sp = lexpr;
 	while (sp != NULL && *sp != '\0') {
 		sp = parse_block(sp, &node);	/* block is name ( ... ) */
