@@ -406,12 +406,15 @@ struct _9p_rdma_priv {
 #endif
 
 struct _9p_request_data {
+	struct glist_head req_q;	/* chaining of pending requests */
 	char *_9pmsg;
 	struct _9p_conn *pconn;
 #ifdef _USE_9P_RDMA
 	msk_data_t *data;
 #endif
 	struct _9p_flush_hook flush_hook;
+	pthread_mutex_t *mutex;
+	pthread_cond_t *cond;
 };
 
 typedef int (*_9p_function_t) (struct _9p_request_data *req9p,
@@ -634,6 +637,9 @@ do {                                                   \
  */
 
 struct _9p_param {
+	/** Number of worker threads.  Set to NB_WORKER_DEFAULT by
+	    default and changed with the Nb_Worker option. */
+	uint32_t nb_worker;
 	/** TCP port for 9p operations.  Defaults to _9P_TCP_PORT,
 	    settable by _9P_TCP_Port */
 	uint16_t _9p_tcp_port;
@@ -821,4 +827,11 @@ int _9p_xattrwalk(struct _9p_request_data *req9p, u32 *plenout, char *preply);
 int _9p_rerror(struct _9p_request_data *req9p, u16 *msgtag, u32 err,
 	       u32 *plenout, char *preply);
 
+/* Expects to already be size checked */
+static inline void _9p_get_fname(char *name, int len, const char *str)
+{
+	memcpy(name, str, len);
+	name[len] = '\0';
+}
+
 #endif				/* _9P_H */
