@@ -268,17 +268,6 @@ typedef fsal_errors_t (*helper_readdir_cb)
 	 uint64_t cookie,
 	 enum cb_state cb_state);
 
-/**
- * Indicate whether this is a read or write operation, for fsal_rdwr.
- */
-
-typedef enum io_direction__ {
-	FSAL_IO_READ = 1,		/*< Reading */
-	FSAL_IO_WRITE = 2,		/*< Writing */
-	FSAL_IO_READ_PLUS = 3,	/*< Reading plus */
-	FSAL_IO_WRITE_PLUS = 4	/*< Writing plus */
-} fsal_io_direction_t;
-
 /**
  * @brief Type of callback for fsal_readdir
  *
@@ -546,7 +535,7 @@ static inline void fsal_copy_attrs(struct attrlist *dest,
 		src->valid_mask &= ~ATTR4_SEC_LABEL;
 	} else if (dest->sec_label.slai_data.slai_data_val != NULL &&
 		((save_request_mask & ATTR4_SEC_LABEL) != 0)) {
-		dest->sec_label.slai_data.slai_data_val =
+		dest->sec_label.slai_data.slai_data_val = (char *)
 			gsh_memdup(dest->sec_label.slai_data.slai_data_val,
 				   dest->sec_label.slai_data.slai_data_len);
 	} else {
@@ -571,7 +560,7 @@ fsal_get_changeid4(struct fsal_obj_handle *obj)
 	fsal_status_t status;
 	changeid4 change;
 
-	fsal_prepare_attrs(&attrs, ATTR_CHANGE | ATTR_CHGTIME);
+	fsal_prepare_attrs(&attrs, ATTR_CHANGE);
 
 	status = obj->obj_ops->getattrs(obj, &attrs);
 
@@ -663,5 +652,27 @@ struct fsal_stats {
 	struct fsal_op_stats *op_stats;
 };
 
+/* Async Processes that will be made synchronous */
+struct async_process_data {
+	/** Return from process */
+	fsal_status_t ret;
+	/** Indicator callback is done. */
+	bool done;
+	/** Mutex to protect done and condition variable. */
+	pthread_mutex_t *mutex;
+	/** Condition variable to signal callback is done. */
+	pthread_cond_t *cond;
+};
+
+extern void fsal_read(struct fsal_obj_handle *obj_hdl,
+		      bool bypass,
+		      struct fsal_io_arg *arg,
+		      struct async_process_data *data);
+
+extern void fsal_write(struct fsal_obj_handle *obj_hdl,
+		       bool bypass,
+		       struct fsal_io_arg *arg,
+		       struct async_process_data *data);
+
 #endif				/* !FSAL_H */
 /** @} */
