@@ -464,6 +464,7 @@ struct attrlist {
 				   read) */
 	fsal_fsid_t fsid;	/*< Filesystem on which this object is
 				   stored */
+	uint64_t fsid3;		/*< Squashed fsid for NFS v3 */
 	fsal_acl_t *acl;	/*< ACL for this object */
 	uint64_t fileid;	/*< Unique identifier for this object within
 				   the scope of the fsid, (e.g. inode number) */
@@ -521,11 +522,13 @@ struct attrlist {
  *          FSAL extended attributes management.
  ******************************************************/
 
+#define XATTR_NAME_SIZE (MAXNAMLEN + 1)
+
 /** An extented attribute entry */
 typedef struct fsal_xattrent {
 	uint64_t xattr_id;	/*< xattr index */
 	uint64_t xattr_cookie;	/*< cookie for the next entry */
-	char xattr_name[MAXNAMLEN + 1];	/*< attribute name  */
+	char xattr_name[XATTR_NAME_SIZE];	/*< attribute name  */
 } fsal_xattrent_t;
 
 /* generic definitions for extended attributes */
@@ -611,7 +614,6 @@ typedef enum {
 
 typedef uint16_t fsal_openflags_t;
 
-/* ACCESS flags */
 #define FSAL_O_CLOSED     0x0000  /* Closed */
 #define FSAL_O_READ       0x0001  /* read */
 #define FSAL_O_WRITE      0x0002  /* write */
@@ -619,21 +621,20 @@ typedef uint16_t fsal_openflags_t;
 						     * explicitly or'd together
 						     * so that FSAL_O_RDWR can
 						     * be used as a mask */
-
-/* Internal flags, not really NFS open flags! */
+#define FSAL_O_RECLAIM         0x0008  /* open reclaim */
 #define FSAL_O_ANY             0x0020  /* any open file descriptor is usable */
 #define FSAL_O_TRUNC           0x0040  /* Truncate file on open */
-
-/* DENY flags */
 #define FSAL_O_DENY_READ       0x0100
 #define FSAL_O_DENY_WRITE      0x0200
 #define FSAL_O_DENY_WRITE_MAND 0x0400  /* Mandatory deny-write (i.e. NFSv4) */
 #define FSAL_O_DENY_NONE       0x0000
 
+/* NFS open flags */
+#define FSAL_O_OPENFLAGS (FSAL_O_RDWR | FSAL_O_DENY_READ | \
+		 FSAL_O_DENY_WRITE | FSAL_O_DENY_WRITE_MAND)
+
 /* Extract NFS open flags */
-#define FSAL_O_NFS_FLAGS(flags) ((flags) & \
-		(FSAL_O_RDWR | FSAL_O_DENY_READ | \
-		 FSAL_O_DENY_WRITE | FSAL_O_DENY_WRITE_MAND))
+#define FSAL_O_NFS_FLAGS(flags) ((flags) & (FSAL_O_OPENFLAGS))
 
 enum fsal_create_mode {
 	FSAL_NO_CREATE = 0,
@@ -673,6 +674,7 @@ typedef enum enum_fsal_fsinfo_options {
 	fso_compute_readdir_cookie,
 	fso_whence_is_name,
 	fso_readdir_plus,
+	fso_compliant_eof_behavior,
 } fsal_fsinfo_options_t;
 
 /* The largest maxread and maxwrite value */
@@ -719,6 +721,8 @@ typedef struct fsal_staticfsinfo_t {
 	bool compute_readdir_cookie;
 	bool whence_is_name;
 	bool readdir_plus;	/*< FSAL supports readdir_plus */
+	bool compliant_eof_behavior;	/* FSAL compliant to end-of-file NFS
+					  v3 read behavior. */
 	int32_t expire_time_parent; /*< Expiration time interval in
 				       seconds for parent handle.
 				       If FS gives information about parent
