@@ -398,9 +398,7 @@ LogFullDebugOpaque(component, format, buf_size, value, length, args...) \
 						 __LINE__, \
 						 __func__, \
 						 NIV_EVENT, \
-						 "%s: EVENT: " format, \
-						 LogComponents[component] \
-						     .comp_str, ## args); \
+						 format, ## args); \
 		} \
 	} while (0)
 
@@ -418,9 +416,7 @@ LogFullDebugOpaque(component, format, buf_size, value, length, args...) \
 						 __LINE__, \
 						 __func__, \
 						 NIV_INFO, \
-						 "%s: INFO: " format, \
-						 LogComponents[component] \
-						     .comp_str, ## args); \
+						 format, ## args); \
 		} \
 	} while (0)
 
@@ -438,9 +434,7 @@ LogFullDebugOpaque(component, format, buf_size, value, length, args...) \
 						 __LINE__, \
 						 __func__, \
 						 NIV_DEBUG, \
-						 "%s: DEBUG: " format, \
-						 LogComponents[component] \
-						     .comp_str, ## args); \
+						 format, ## args); \
 		} \
 	} while (0)
 
@@ -458,9 +452,7 @@ LogFullDebugOpaque(component, format, buf_size, value, length, args...) \
 						 __LINE__, \
 						 __func__, \
 						 NIV_MID_DEBUG, \
-						 "%s: MID DEBUG: " format, \
-						 LogComponents[component] \
-						     .comp_str, ## args); \
+						 format, ## args); \
 		} \
 	} while (0)
 
@@ -478,9 +470,7 @@ LogFullDebugOpaque(component, format, buf_size, value, length, args...) \
 						 __LINE__, \
 						 __func__, \
 						 NIV_FULL_DEBUG, \
-						 "%s: FULLDEBUG: " format, \
-						 LogComponents[component] \
-						     .comp_str, ## args); \
+						 format, ## args); \
 		} \
 	} while (0)
 
@@ -493,4 +483,41 @@ void rpc_warnx(/* const */ char *fmt, ...);
 extern struct gsh_dbus_interface log_interface;
 #endif
 
+/* Rate limited logging */
+struct ratelimit_state {
+	pthread_mutex_t mutex;
+	int interval;
+	int burst;
+	int printed;
+	int missed;
+	time_t begin;
+};
+bool _ratelimit(struct ratelimit_state *rs, int *missed);
+
+#define RATELIMIT_STATE_INIT(interval_init, burst_init) {         \
+		.mutex          = PTHREAD_MUTEX_INITIALIZER,      \
+		.interval       = interval_init,                  \
+		.burst          = burst_init,                     \
+	}
+
+#define DEFINE_RATELIMIT_STATE(name, interval_init, burst_init)   \
+	struct ratelimit_state name =                             \
+		RATELIMIT_STATE_INIT(interval_init, burst_init)   \
+
+#define DEFAULT_RATELIMIT_INTERVAL    30 /* 30 seconds */
+#define DEFAULT_RATELIMIT_BURST       2
+
+#define LogEventLimited(comp, fmt, args...) ({                             \
+	int missed;                                                        \
+									   \
+	static DEFINE_RATELIMIT_STATE(_rs,                                 \
+			DEFAULT_RATELIMIT_INTERVAL,                        \
+			DEFAULT_RATELIMIT_BURST);                          \
+	if (_ratelimit(&(_rs), &missed)) {                                    \
+		if (missed)                                                \
+			LogEvent(comp, "message missed %d times", missed); \
+		LogEvent(comp, fmt, ## args);                              \
+	}                                                                  \
+})
+
 #endif
