@@ -104,6 +104,16 @@ struct export_stats {
 	struct gsh_export export;
 };
 
+/**
+ * @brief Auth stats information
+ */
+struct auth_stats {
+	uint64_t total;
+	uint64_t latency;
+	uint64_t max;
+	uint64_t min;
+};
+
 #ifdef USE_DBUS
 
 /* Bits for introspect arg structures
@@ -135,6 +145,58 @@ struct export_stats {
 	.direction = "out" \
 }
 
+#define CEIOSTATS_REPLY    \
+{                          \
+	.name = "read",    \
+	.type = "(ttdt)",  \
+	.direction = "out" \
+},                         \
+{                          \
+	.name = "write",   \
+	.type = "(ttdt)",  \
+	.direction = "out" \
+},                         \
+{                          \
+	.name = "other",   \
+	.type = "(ttd)",   \
+	.direction = "out" \
+}
+
+#define CELOSTATS_REPLY    \
+{                          \
+	.name = "layout",  \
+	.type = "(ttt)",  \
+	.direction = "out" \
+}
+
+#define CE_STATS_REPLY      \
+{                           \
+	.name = "clnt_v3",  \
+	.type = "b",        \
+	.direction = "out"  \
+},                          \
+CEIOSTATS_REPLY,            \
+{                           \
+	.name = "clnt_v40", \
+	.type = "b",        \
+	.direction = "out"  \
+},                          \
+CEIOSTATS_REPLY,            \
+{                           \
+	.name = "clnt_v41", \
+	.type = "b",        \
+	.direction = "out"  \
+},                          \
+CEIOSTATS_REPLY,            \
+CELOSTATS_REPLY,            \
+{                           \
+	.name = "clnt_v42", \
+	.type = "b",        \
+	.direction = "out"  \
+},                          \
+CEIOSTATS_REPLY,            \
+CELOSTATS_REPLY
+
 #define TRANSPORT_REPLY    \
 {                          \
 	.name = "rx_bytes",\
@@ -211,23 +273,34 @@ struct export_stats {
 	.name = "v4_full_status",  \
 	.type = "b(tt)",        \
 	.direction = "out"      \
+},				\
+{                               \
+	.name = "auth_status",  \
+	.type = "b(tt)",        \
+	.direction = "out"      \
 }
 
-
 #define V3_FULL_REPLY		\
 {				\
 	.name = "v3_full_stats",	\
-	.type = "a(stttdddddd)",	\
+	.type = "a(stttddd)",	\
 	.direction = "out"	\
 }
 
 #define V4_FULL_REPLY		\
 {				\
 	.name = "v4_full_stats",	\
-	.type = "a(sttdddddd)",	\
+	.type = "a(sttddd)",	\
 	.direction = "out"	\
 }
 
+#define AUTH_REPLY		\
+{                               \
+	.name = "auth",		\
+	.type = "a(tdddtddd)",  \
+	.direction = "out"      \
+}
+
 #define LAYOUTS_REPLY		\
 {				\
 	.name = "getdevinfo",	\
@@ -289,8 +362,15 @@ struct export_stats {
 	.direction = "out"  \
 }
 
+extern struct timespec auth_stats_time;
+extern struct timespec v3_full_stats_time;
+extern struct timespec v4_full_stats_time;
 
 void server_stats_summary(DBusMessageIter * iter, struct gsh_stats *st);
+void server_dbus_client_io_ops(DBusMessageIter *iter,
+				struct gsh_client *client);
+void server_dbus_export_details(DBusMessageIter *iter,
+				struct gsh_export *g_export);
 void server_dbus_v3_iostats(struct nfsv3_stats *v3p, DBusMessageIter *iter);
 void server_dbus_v40_iostats(struct nfsv40_stats *v40p, DBusMessageIter *iter);
 void server_dbus_v41_iostats(struct nfsv41_stats *v41p, DBusMessageIter *iter);
@@ -313,6 +393,7 @@ void reset_client_stats(void);
 void reset_gsh_stats(struct gsh_stats *st);
 void reset_v3_full_stats(void);
 void reset_v4_full_stats(void);
+void reset_auth_stats(void);
 
 #ifdef _USE_9P
 void server_dbus_9p_iostats(struct _9p_stats *_9pp, DBusMessageIter *iter);
