@@ -1,4 +1,4 @@
-#!/usr/bin/python2
+#!/usr/bin/python3
 #
 # ganesha_mgr_utils.py - commandline tool utils for managing nfs-ganesha.
 #
@@ -26,8 +26,6 @@ except ImportError:
     from gi.repository import GObject as gobject
 
 import sys
-import time
-import traceback
 import dbus.mainloop.glib
 import dbus
 from dbus import glib
@@ -58,7 +56,7 @@ class ClientMgr():
         self.bus = dbus.SystemBus()
         try:
             self.dbusobj = self.bus.get_object(self.dbus_service_name,
-                                self.dbus_path)
+                                               self.dbus_path)
         except:
             sys.exit("Error: Can't talk to ganesha service on d-bus." \
                      " Looks like Ganesha is down")
@@ -67,9 +65,9 @@ class ClientMgr():
         add_client_method = self.dbusobj.get_dbus_method("AddClient",
                                                          self.dbus_interface)
         try:
-           reply = add_client_method(ipaddr)
-        except dbus.exceptions.DBusException as e:
-           return False, e
+            reply = add_client_method(ipaddr)
+        except dbus.exceptions.DBusException as ex:
+            return False, ex
 
         status = reply[0]
         msg = reply[1]
@@ -79,9 +77,9 @@ class ClientMgr():
         remove_client_method = self.dbusobj.get_dbus_method("RemoveClient",
                                                             self.dbus_interface)
         try:
-           reply = remove_client_method(ipaddr)
-        except dbus.exceptions.DBusException as e:
-           return False, e
+            reply = remove_client_method(ipaddr)
+        except dbus.exceptions.DBusException as ex:
+            return False, ex
 
         status = reply[0]
         msg = reply[1]
@@ -91,32 +89,31 @@ class ClientMgr():
         show_client_method = self.dbusobj.get_dbus_method("ShowClients",
                                                           self.dbus_interface)
         try:
-           reply = show_client_method()
-        except dbus.exceptions.DBusException as e:
-           return False, e, []
+            reply = show_client_method()
+        except dbus.exceptions.DBusException as ex:
+            return False, ex, []
 
         time = reply[0]
         client_array = reply[1]
 
-        ts = (time[0],
-              time[1])
+        ts_ = (time[0], time[1])
         clients = []
         for client in client_array:
-            cl = client
-            lasttime = cl[9]
-            clt = Client(ClientIP = str(cl[0]),
-                         HasNFSv3 = cl[1],
-                         HasMNT = cl[2],
-                         HasNLM4 = cl[3],
-                         HasRQUOTA = cl[4],
-                         HasNFSv40 = cl[5],
-                         HasNFSv41 = cl[6],
-                         HasNFSv42 = cl[7],
-                         Has9P = cl[8],
-                         LastTime = (lasttime[0],
-                                     lasttime[1]))
+            cl_ = client
+            lasttime = cl_[9]
+            clt = Client(ClientIP=str(cl_[0]),
+                         HasNFSv3=cl_[1],
+                         HasMNT=cl_[2],
+                         HasNLM4=cl_[3],
+                         HasRQUOTA=cl_[4],
+                         HasNFSv40=cl_[5],
+                         HasNFSv41=cl_[6],
+                         HasNFSv42=cl_[7],
+                         Has9P=cl_[8],
+                         LastTime=(lasttime[0],
+                                   lasttime[1]))
             clients.append(clt)
-        return True, "Done", [ts, clients]
+        return True, "Done", [ts_, clients]
 
 
 
@@ -157,7 +154,7 @@ class ExportMgr():
         self.bus = dbus.SystemBus()
         try:
             self.dbusobj = self.bus.get_object(self.dbus_service_name,
-                                self.dbus_path)
+                                               self.dbus_path)
         except:
             sys.exit("Error: Can't talk to ganesha service on d-bus." \
                      " Looks like Ganesha is down")
@@ -166,9 +163,9 @@ class ExportMgr():
         add_export_method = self.dbusobj.get_dbus_method("AddExport",
                                                          self.dbus_interface)
         try:
-           msg = add_export_method(conf_path, exp_expr)
-        except dbus.exceptions.DBusException as e:
-           return False, e
+            msg = add_export_method(conf_path, exp_expr)
+        except dbus.exceptions.DBusException as ex:
+            return False, ex
 
         return True, "Done: "+msg
 
@@ -176,9 +173,9 @@ class ExportMgr():
         update_export_method = self.dbusobj.get_dbus_method("UpdateExport",
                                                             self.dbus_interface)
         try:
-           msg = update_export_method(conf_path, exp_expr)
-        except dbus.exceptions.DBusException as e:
-           return False, e
+            msg = update_export_method(conf_path, exp_expr)
+        except dbus.exceptions.DBusException as ex:
+            return False, ex
 
         return True, "Done: "+msg
 
@@ -186,68 +183,67 @@ class ExportMgr():
         rm_export_method = self.dbusobj.get_dbus_method("RemoveExport",
                                                         self.dbus_interface)
         try:
-           rm_export_method(int(exp_id))
-        except dbus.exceptions.DBusException as e:
-           return False, e
+            rm_export_method(int(exp_id))
+        except dbus.exceptions.DBusException as ex:
+            return False, ex
         return True, "Done"
 
     def DisplayExport(self, exp_id):
         display_export_method = self.dbusobj.get_dbus_method("DisplayExport",
                                                              self.dbus_interface)
         try:
-           id, fullpath, pseudopath, tag, clients_array = \
+            id_, fullpath, pseudopath, tag, clients_array = \
                 display_export_method(int(exp_id))
-        except dbus.exceptions.DBusException as e:
-           return False, e, []
+        except dbus.exceptions.DBusException as ex:
+            return False, ex, []
 
         export_clients = []
         for client in clients_array:
-            c = ExportClient(Client_type = client[0],
-                             CIDR_version = client[1],
-                             CIDR_address = client[2],
-                             CIDR_mask = client[3],
-                             CIDR_proto = client[4],
-                             Anonymous_uid = client[5],
-                             Anonymous_gid = client[6],
-                             Expire_time_attr = client[7],
-                             Options = client[8],
-                             Set = client[9])
+            c_ = ExportClient(Client_type=client[0],
+                              CIDR_version=client[1],
+                              CIDR_address=client[2],
+                              CIDR_mask=client[3],
+                              CIDR_proto=client[4],
+                              Anonymous_uid=client[5],
+                              Anonymous_gid=client[6],
+                              Expire_time_attr=client[7],
+                              Options=client[8],
+                              Set=client[9])
 
-            export_clients.append(c)
+            export_clients.append(c_)
 
-        return True, "Done", [id, fullpath, pseudopath, tag, export_clients]
+        return True, "Done", [id_, fullpath, pseudopath, tag, export_clients]
 
     def ShowExports(self):
         show_export_method = self.dbusobj.get_dbus_method("ShowExports",
                                                           self.dbus_interface)
         try:
-           reply = show_export_method()
-        except dbus.exceptions.DBusException as e:
-           return False, e, []
+            reply = show_export_method()
+        except dbus.exceptions.DBusException as ex:
+            return False, ex, []
 
         time = reply[0]
         export_array = reply[1]
 
-        ts = (time[0],
-              time[1])
+        ts_ = (time[0], time[1])
         exports = []
         for export in export_array:
             ex = export
             lasttime = ex[10]
-            exp = Export(ExportID = ex[0],
-                         ExportPath = str(ex[1]),
-                         HasNFSv3 = ex[2],
-                         HasMNT = ex[3],
-                         HasNLM4 = ex[4],
-                         HasRQUOTA = ex[5],
-                         HasNFSv40 = ex[6],
-                         HasNFSv41 = ex[7],
-                         HasNFSv42 = ex[8],
-                         Has9P = ex[9],
-                         LastTime = (lasttime[0],
-                                     lasttime[1]))
+            exp = Export(ExportID=ex[0],
+                         ExportPath=str(ex[1]),
+                         HasNFSv3=ex[2],
+                         HasMNT=ex[3],
+                         HasNLM4=ex[4],
+                         HasRQUOTA=ex[5],
+                         HasNFSv40=ex[6],
+                         HasNFSv41=ex[7],
+                         HasNFSv42=ex[8],
+                         Has9P=ex[9],
+                         LastTime=(lasttime[0],
+                                   lasttime[1]))
             exports.append(exp)
-        return True, "Done", [ts, exports]
+        return True, "Done", [ts_, exports]
 
 class AdminInterface():
     '''
@@ -270,9 +266,9 @@ class AdminInterface():
         grace_method = self.dbusobj.get_dbus_method("grace",
                                                     self.dbus_interface)
         try:
-           reply = grace_method(ipaddr)
-        except dbus.exceptions.DBusException as e:
-           return False, e
+            reply = grace_method(ipaddr)
+        except dbus.exceptions.DBusException as ex:
+            return False, ex
 
         status = reply[0]
         msg = reply[1]
@@ -282,9 +278,9 @@ class AdminInterface():
         shutdown_method = self.dbusobj.get_dbus_method("shutdown",
                                                        self.dbus_interface)
         try:
-           reply = shutdown_method()
-        except dbus.exceptions.DBusException as e:
-           return False, e
+            reply = shutdown_method()
+        except dbus.exceptions.DBusException as ex:
+            return False, ex
 
         status = reply[0]
         msg = reply[1]
@@ -294,9 +290,9 @@ class AdminInterface():
         method = self.dbusobj.get_dbus_method("purge_netgroups",
                                               self.dbus_interface)
         try:
-           reply = method()
-        except dbus.exceptions.DBusException as e:
-           return False, e
+            reply = method()
+        except dbus.exceptions.DBusException as ex:
+            return False, ex
 
         status = reply[0]
         msg = reply[1]
@@ -306,9 +302,9 @@ class AdminInterface():
         method = self.dbusobj.get_dbus_method("purge_idmapper_cache",
                                               self.dbus_interface)
         try:
-           reply = method()
-        except dbus.exceptions.DBusException as e:
-           return False, e
+            reply = method()
+        except dbus.exceptions.DBusException as ex:
+            return False, ex
 
         status = reply[0]
         msg = reply[1]
@@ -318,25 +314,31 @@ class AdminInterface():
         method = self.dbusobj.get_dbus_method("purge_gids",
                                               self.dbus_interface)
         try:
-           reply = method()
-        except dbus.exceptions.DBusException as e:
-           return False, e
+            reply = method()
+        except dbus.exceptions.DBusException as ex:
+            return False, ex
 
         status = reply[0]
         msg = reply[1]
         return status, msg
 
+    def GetAll(self):
+        method = self.dbusobj.get_dbus_method("GetAll",
+                                              "org.freedesktop.DBus.Properties")
+        try:
+            dictionary = method(self.dbus_interface)
+        except dbus.exceptions.DBusException as ex:
+            return False, ex, {}
 
-IDMapper = namedtuple('IDMapper',
-                     ['Name',
-                      'UID',
-                      'HasGID',
-                      'GID'])
+        prop_dict = {}
+        for key in dictionary.keys():
+            prop_dict[key] = dictionary[key]
+        return True, "Done", prop_dict
 
-FileSys = namedtuple('FileSys',
-                     ['Path',
-                      'MajorDevId',
-                      'MinorDevId'])
+
+IDMapper = namedtuple('IDMapper', ['Name', 'UID', 'HasGID', 'GID'])
+
+FileSys = namedtuple('FileSys', ['Path', 'MajorDevId', 'MinorDevId'])
 
 class CacheMgr():
     '''
@@ -350,7 +352,7 @@ class CacheMgr():
         self.bus = dbus.SystemBus()
         try:
             self.dbusobj = self.bus.get_object(self.dbus_service_name,
-                                self.dbus_path)
+                                               self.dbus_path)
         except:
             sys.exit("Error: Can't talk to ganesha service on d-bus." \
                      " Looks like Ganesha is down")
@@ -359,47 +361,45 @@ class CacheMgr():
         show_filesys_method = self.dbusobj.get_dbus_method("showfs",
                                                            self.dbus_interface)
         try:
-           reply = show_filesys_method()
-        except dbus.exceptions.DBusException as e:
-           return False, e, []
+            reply = show_filesys_method()
+        except dbus.exceptions.DBusException as ex:
+            return False, ex, []
 
         time = reply[0]
         fs_array = reply[1]
 
-        ts = (time[0],
-              time[1])
+        ts_ = (time[0], time[1])
 
         fss = []
-        for fs in fs_array:
-            filesys1 = FileSys(Path = str(fs[0]),
-                               MajorDevId = fs[1],
-                               MinorDevId = fs[2])
+        for fs_ in fs_array:
+            filesys1 = FileSys(Path=str(fs_[0]),
+                               MajorDevId=fs_[1],
+                               MinorDevId=fs_[2])
             fss.append(filesys1)
-        return True, "Done", [ts, fss]
+        return True, "Done", [ts_, fss]
 
 
     def ShowIdmapper(self):
         show_id_method = self.dbusobj.get_dbus_method("showidmapper",
                                                       self.dbus_interface)
         try:
-           reply = show_id_method()
-        except dbus.exceptions.DBusException as e:
-           return False, e, []
+            reply = show_id_method()
+        except dbus.exceptions.DBusException as ex:
+            return False, ex, []
 
         time = reply[0]
         id_array = reply[1]
 
-        ts = (time[0],
-              time[1])
+        ts_ = (time[0], time[1])
 
         ids = []
         for entry in id_array:
-            entry1 = IDMapper(Name = str(entry[0]),
-                              UID = entry[1],
-                              HasGID = entry[2],
-                              GID = entry[3])
+            entry1 = IDMapper(Name=str(entry[0]),
+                              UID=entry[1],
+                              HasGID=entry[2],
+                              GID=entry[3])
             ids.append(entry1)
-        return True, "Done", [ts, ids]
+        return True, "Done", [ts_, ids]
 
 
 LOGGER_PROPS = 'org.ganesha.nfsd.log.component'
@@ -426,31 +426,31 @@ class LogManager():
         getall_method = self.dbusobj.get_dbus_method("GetAll",
                                                      self.dbus_interface)
         try:
-           dictionary = getall_method(LOGGER_PROPS)
-        except dbus.exceptions.DBusException as e:
-           return False, e, {}
+            dictionary = getall_method(LOGGER_PROPS)
+        except dbus.exceptions.DBusException as ex:
+            return False, ex, {}
 
         prop_dict = {}
         for key in dictionary.keys():
-                prop_dict[key] = dictionary[key]
+            prop_dict[key] = dictionary[key]
         return True, "Done", prop_dict
 
-    def Get(self, property):
+    def Get(self, prop):
         get_method = self.dbusobj.get_dbus_method("Get",
                                                   self.dbus_interface)
         try:
-           level = get_method(LOGGER_PROPS, property)
-        except dbus.exceptions.DBusException as e:
-           return False, e, 0
+            level = get_method(LOGGER_PROPS, prop)
+        except dbus.exceptions.DBusException as ex:
+            return False, ex, 0
 
         return True, "Done", level
 
-    def Set(self, property, setval):
+    def Set(self, prop, setval):
         set_method = self.dbusobj.get_dbus_method("Set",
                                                   self.dbus_interface)
         try:
-           set_method(LOGGER_PROPS, property, setval)
-        except dbus.exceptions.DBusException as e:
-           return False, e
+            set_method(LOGGER_PROPS, prop, setval)
+        except dbus.exceptions.DBusException as ex:
+            return False, ex
 
         return True, "Done"
