@@ -18,6 +18,13 @@ if [ -r ${CONFIGFILE} ]; then
 	. ${CONFIGFILE}
 	[ -x ${EPOCH_EXEC} ] &&  EPOCHVALUE=`${EPOCH_EXEC}`
 
+	NOFILE_CONF=/lib/systemd/system/nfs-ganesha.service.d/10-nofile.conf
+	if [ -n "$NOFILE" ]; then
+		mkdir -p $(command dirname ${NOFILE_CONF})
+		printf "[Service]\nLimitNOFILE=$NOFILE\n" > $NOFILE_CONF
+		systemctl daemon-reload
+	fi
+
 	mkdir -p $(command dirname ${RUNCONFIG} 2>/dev/null)
 	{
 		cat ${CONFIGFILE}
