@@ -90,7 +90,7 @@ static struct client_by_ip client_by_ip;
  *
  * @return The computed offset.
  */
-static inline uint32_t eip_cache_offsetof(struct client_by_ip *eid, uint32_t k)
+static inline int eip_cache_offsetof(struct client_by_ip *eid, uint64_t k)
 {
 	return k % eid->cache_sz;
 }
@@ -110,10 +110,8 @@ static int client_ip_cmpf(const struct avltree_node *lhs,
 
 	lk = avltree_container_of(lhs, struct gsh_client, node_k);
 	rk = avltree_container_of(rhs, struct gsh_client, node_k);
-	if (lk->addr.len != rk->addr.len)
-		return (lk->addr.len < rk->addr.len) ? -1 : 1;
-	else
-		return memcmp(lk->addr.addr, rk->addr.addr, lk->addr.len);
+
+	return sockaddr_cmpf(&lk->cl_addrbuf, &rk->cl_addrbuf, true);
 }
 
 /**
@@ -135,61 +133,21 @@ struct gsh_client *get_gsh_client(sockaddr_t *client_ipaddr, bool lookup_only)
 	struct gsh_client *cl;
 	struct server_stats *server_st;
 	struct gsh_client v;
-	char hoststr[SOCK_NAME_MAX];
-	uint8_t *addr = NULL;
-	uint32_t ipaddr;
-	int addr_len = 0;
 	void **cache_slot;
-
-	switch (client_ipaddr->ss_family) {
-	case AF_INET:
-		addr =
-		    (uint8_t *) &((struct sockaddr_in *)client_ipaddr)->
-		    sin_addr;
-		addr_len = 4;
-		memcpy(&ipaddr,
-		       (uint8_t *) &((struct sockaddr_in *)client_ipaddr)->
-		       sin_addr, sizeof(ipaddr));
-		break;
-	case AF_INET6:
-		addr =
-		    (uint8_t *) &((struct sockaddr_in6 *)client_ipaddr)->
-		    sin6_addr;
-		addr_len = 16;
-		memcpy(&ipaddr,
-		       (uint8_t *) &((struct sockaddr_in6 *)client_ipaddr)->
-		       sin6_addr, sizeof(ipaddr));
-		break;
-#ifdef RPC_VSOCK
-	case AF_VSOCK:
-	{
-		struct sockaddr_vm *svm; /* XXX checkpatch bs */
-
-		svm = (struct sockaddr_vm *)client_ipaddr;
-		addr = (uint8_t *)&(svm->svm_cid);
-		addr_len = sizeof(svm->svm_cid);
-		ipaddr = svm->svm_cid;
-	}
-	break;
-#endif /* VSOCK */
-	default:
-		assert(0);
-	}
-	v.addr.addr = addr;
-	v.addr.len = addr_len;
+	uint64_t hash = hash_sockaddr(client_ipaddr, true);
 
 	PTHREAD_RWLOCK_rdlock(&client_by_ip.lock);
 
 	/* check cache */
 	cache_slot = (void **)
-	    &(client_by_ip.cache[eip_cache_offsetof(&client_by_ip, ipaddr)]);
+	    &(client_by_ip.cache[eip_cache_offsetof(&client_by_ip, hash)]);
 	node = (struct avltree_node *)atomic_fetch_voidptr(cache_slot);
 	if (node) {
 		if (client_ip_cmpf(&v.node_k, node) == 0) {
 			/* got it in 1 */
 			LogDebug(COMPONENT_HASHTABLE_CACHE,
 				 "client_mgr cache hit slot %d",
-				 eip_cache_offsetof(&client_by_ip, ipaddr));
+				 eip_cache_offsetof(&client_by_ip, hash));
 			cl = avltree_container_of(node, struct gsh_client,
 						  node_k);
 			goto out;
@@ -209,27 +167,31 @@ struct gsh_client *get_gsh_client(sockaddr_t *client_ipaddr, bool lookup_only)
 	}
 	PTHREAD_RWLOCK_unlock(&client_by_ip.lock);
 
-	server_st = gsh_calloc(1, (sizeof(struct server_stats) + addr_len));
+	server_st = gsh_calloc(1, sizeof(*server_st));
 
 	cl = &server_st->client;
-	memcpy(cl->addrbuf, addr, addr_len);
-	cl->addr.addr = cl->addrbuf;
-	cl->addr.len = addr_len;
-	cl->refcnt = 0;		/* we will hold a ref starting out... */
-	sprint_sockip(client_ipaddr, hoststr, SOCK_NAME_MAX);
-	cl->hostaddr_str = gsh_strdup(hoststr);
+	cl->cl_addrbuf = *client_ipaddr;
+	cl->refcnt = 0;
+
+	if (!sprint_sockip(client_ipaddr, cl->hostaddr_str,
+			   sizeof(cl->hostaddr_str))) {
+		(void) strlcpy(cl->hostaddr_str, "<unknown>",
+			       sizeof(cl->hostaddr_str));
+	}
 
 	PTHREAD_RWLOCK_wrlock(&client_by_ip.lock);
 	node = avltree_insert(&cl->node_k, &client_by_ip.t);
 	if (node) {
-		gsh_free(cl->hostaddr_str);
 		gsh_free(server_st);	/* somebody beat us to it */
 		cl = avltree_container_of(node, struct gsh_client, node_k);
 	} else {
 		PTHREAD_RWLOCK_init(&cl->lock, NULL);
+		/* update cache */
+		atomic_store_voidptr(cache_slot, &cl->node_k);
 	}
 
  out:
+	/* we will hold a ref starting out... */
 	inc_gsh_client_refcount(cl);
 	PTHREAD_RWLOCK_unlock(&client_by_ip.lock);
 	return cl;
@@ -266,48 +228,9 @@ int remove_gsh_client(sockaddr_t *client_ipaddr)
 	struct gsh_client *cl = NULL;
 	struct server_stats *server_st;
 	struct gsh_client v;
-	uint8_t *addr = NULL;
-	uint32_t ipaddr;
-	int addr_len = 0;
 	int removed = 0;
 	void **cache_slot;
-
-	switch (client_ipaddr->ss_family) {
-	case AF_INET:
-		addr =
-		    (uint8_t *) &((struct sockaddr_in *)client_ipaddr)->
-		    sin_addr;
-		addr_len = 4;
-		memcpy(&ipaddr,
-		       (uint8_t *) &((struct sockaddr_in *)client_ipaddr)->
-		       sin_addr, sizeof(ipaddr));
-		break;
-	case AF_INET6:
-		addr =
-		    (uint8_t *) &((struct sockaddr_in6 *)client_ipaddr)->
-		    sin6_addr;
-		addr_len = 16;
-		memcpy(&ipaddr,
-		       (uint8_t *) &((struct sockaddr_in6 *)client_ipaddr)->
-		       sin6_addr, sizeof(ipaddr));
-		break;
-#ifdef RPC_VSOCK
-	case AF_VSOCK:
-	{
-		struct sockaddr_vm *svm; /* XXX checkpatch horror */
-
-		svm = (struct sockaddr_vm *)client_ipaddr;
-		addr = (uint8_t *)&(svm->svm_cid);
-		addr_len = sizeof(svm->svm_cid);
-		ipaddr = svm->svm_cid;
-	}
-	break;
-#endif /* VSOCK */
-	default:
-		assert(0);
-	}
-	v.addr.addr = addr;
-	v.addr.len = addr_len;
+	uint64_t hash = hash_sockaddr(client_ipaddr, true);
 
 	PTHREAD_RWLOCK_wrlock(&client_by_ip.lock);
 	node = avltree_lookup(&v.node_k, &client_by_ip.t);
@@ -319,7 +242,7 @@ int remove_gsh_client(sockaddr_t *client_ipaddr)
 		}
 		cache_slot = (void **)
 		    &(client_by_ip.cache[eip_cache_offsetof(
-						&client_by_ip, ipaddr)]);
+						&client_by_ip, hash)]);
 		cnode = (struct avltree_node *)atomic_fetch_voidptr(cache_slot);
 		if (node == cnode)
 			atomic_store_voidptr(cache_slot, NULL);
@@ -332,8 +255,6 @@ int remove_gsh_client(sockaddr_t *client_ipaddr)
 	if (removed == 0) {
 		server_st = container_of(cl, struct server_stats, client);
 		server_stats_free(&server_st->st);
-		if (cl->hostaddr_str != NULL)
-			gsh_free(cl->hostaddr_str);
 		gsh_free(server_st);
 	}
 	return removed;
@@ -377,7 +298,7 @@ int foreach_gsh_client(bool(*cb) (struct gsh_client *cl, void *state),
 static bool arg_ipaddr(DBusMessageIter *args, sockaddr_t *sp, char **errormsg)
 {
 	char *client_addr;
-	unsigned char addrbuf[16];
+	unsigned char cl_addrbuf[sizeof(struct in6_addr)];
 	bool success = true;
 
 	/* XXX AF_VSOCK addresses are not self-describing--and one might
@@ -392,14 +313,14 @@ static bool arg_ipaddr(DBusMessageIter *args, sockaddr_t *sp, char **errormsg)
 		*errormsg = "arg not a string";
 	} else {
 		dbus_message_iter_get_basic(args, &client_addr);
-		if (inet_pton(AF_INET, client_addr, addrbuf) == 1) {
+		if (inet_pton(AF_INET, client_addr, cl_addrbuf) == 1) {
 			sp->ss_family = AF_INET;
-			memcpy(&(((struct sockaddr_in *)sp)->sin_addr), addrbuf,
-			       4);
-		} else if (inet_pton(AF_INET6, client_addr, addrbuf) == 1) {
+			memcpy(&((struct sockaddr_in *)sp)->sin_addr,
+			       cl_addrbuf, sizeof(struct in_addr));
+		} else if (inet_pton(AF_INET6, client_addr, cl_addrbuf) == 1) {
 			sp->ss_family = AF_INET6;
-			memcpy(&(((struct sockaddr_in6 *)sp)->sin6_addr),
-			       addrbuf, 16);
+			memcpy(&((struct sockaddr_in6 *)sp)->sin6_addr,
+			       cl_addrbuf, sizeof(struct in6_addr));
 		} else {
 			success = false;
 			*errormsg = "can't decode client address";
@@ -444,7 +365,7 @@ static bool gsh_client_addclient(DBusMessageIter *args,
 			errormsg = "No memory to insert client";
 		}
 	}
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	return true;
 }
 
@@ -482,7 +403,7 @@ static bool gsh_client_removeclient(DBusMessageIter *args,
 			errormsg = "Unexpected error";
 		}
 	}
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	return true;
 }
 
@@ -503,22 +424,21 @@ static bool client_to_dbus(struct gsh_client *cl_node, void *state)
 	struct showclients_state *iter_state =
 	    (struct showclients_state *)state;
 	struct server_stats *cl;
-	char ipaddr[64];
-	const char *addrp;
-	int addr_type;
+	char *ipaddr = alloca(SOCK_NAME_MAX);
 	DBusMessageIter struct_iter;
 	struct timespec last_as_ts = nfs_ServerBootTime;
 
 	cl = container_of(cl_node, struct server_stats, client);
-	addr_type = (cl_node->addr.len == 4) ? AF_INET : AF_INET6;
-	addrp =
-	    inet_ntop(addr_type, cl_node->addr.addr, ipaddr, sizeof(ipaddr));
+
+	if (!sprint_sockip(&cl_node->cl_addrbuf, ipaddr, SOCK_NAME_MAX))
+		(void) strlcpy(ipaddr, "<unknown>", SOCK_NAME_MAX);
+
 	timespec_add_nsecs(cl_node->last_update, &last_as_ts);
 	dbus_message_iter_open_container(&iter_state->client_iter,
 					 DBUS_TYPE_STRUCT, NULL, &struct_iter);
-	dbus_message_iter_append_basic(&struct_iter, DBUS_TYPE_STRING, &addrp);
+	dbus_message_iter_append_basic(&struct_iter, DBUS_TYPE_STRING, &ipaddr);
 	server_stats_summary(&struct_iter, &cl->st);
-	dbus_append_timestamp(&struct_iter, &last_as_ts);
+	gsh_dbus_append_timestamp(&struct_iter, &last_as_ts);
 	dbus_message_iter_close_container(&iter_state->client_iter,
 					  &struct_iter);
 	return true;
@@ -535,7 +455,7 @@ static bool gsh_client_showclients(DBusMessageIter *args,
 	now(&timestamp);
 	/* create a reply from the message */
 	dbus_message_iter_init_append(reply, &iter);
-	dbus_append_timestamp(&iter, &timestamp);
+	gsh_dbus_append_timestamp(&iter, &timestamp);
 	dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY,
 					 "(sbbbbbbbb(tt))",
 					 &iter_state.client_iter);
@@ -612,6 +532,45 @@ static struct gsh_client *lookup_client(DBusMessageIter *args, char **errormsg)
 	return client;
 }
 
+/**
+ * DBUS method to detailed client statistics
+ */
+static bool gsh_client_io_ops(DBusMessageIter *args,
+				  DBusMessage *reply,
+				  DBusError *error)
+{
+	char *errormsg = "OK";
+	struct gsh_client *client = NULL;
+	bool success = true;
+	DBusMessageIter iter;
+
+	dbus_message_iter_init_append(reply, &iter);
+	client = lookup_client(args, &errormsg);
+	if (client == NULL) {
+		success = false;
+		errormsg = "Client IP address not found";
+	}
+
+	gsh_dbus_status_reply(&iter, success, errormsg);
+	if (success)
+		server_dbus_client_io_ops(&iter, client);
+
+	if (client != NULL)
+		put_gsh_client(client);
+
+	return true;
+}
+
+static struct gsh_dbus_method cltmgr_client_io_ops = {
+	.name = "GetClientIOops",
+	.method = gsh_client_io_ops,
+	.args = {IPADDR_ARG,
+		 STATUS_REPLY,
+		 TIMESTAMP_REPLY,
+		 CE_STATS_REPLY,
+		 END_ARG_LIST}
+};
+
 /**
  * DBUS method to report NFSv3 I/O statistics
  *
@@ -642,7 +601,7 @@ static bool get_nfsv3_stats_io(DBusMessageIter *args,
 			errormsg = "Client does not have any NFSv3 activity";
 		}
 	}
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	if (success)
 		server_dbus_v3_iostats(server_st->st.nfsv3, &iter);
 
@@ -691,7 +650,7 @@ static bool get_nfsv40_stats_io(DBusMessageIter *args,
 			errormsg = "Client does not have any NFSv4.0 activity";
 		}
 	}
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	if (success)
 		server_dbus_v40_iostats(server_st->st.nfsv40, &iter);
 
@@ -740,7 +699,7 @@ static bool get_nfsv41_stats_io(DBusMessageIter *args,
 			errormsg = "Client does not have any NFSv4.1 activity";
 		}
 	}
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	if (success)
 		server_dbus_v41_iostats(server_st->st.nfsv41, &iter);
 
@@ -790,7 +749,7 @@ static bool get_nfsv41_stats_layouts(DBusMessageIter *args,
 			errormsg = "Client does not have any NFSv4.1 activity";
 		}
 	}
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	if (success)
 		server_dbus_v41_layouts(server_st->st.nfsv41, &iter);
 
@@ -836,7 +795,7 @@ static bool get_stats_delegations(DBusMessageIter *args,
 		}
 	}
 
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	if (success)
 		server_dbus_delegations(server_st->st.deleg, &iter);
 
@@ -885,7 +844,7 @@ static bool get_9p_stats_io(DBusMessageIter *args,
 			errormsg = "Client does not have any 9p activity";
 		}
 	}
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	if (success)
 		server_dbus_9p_iostats(server_st->st._9p, &iter);
 
@@ -932,7 +891,7 @@ static bool get_9p_stats_trans(DBusMessageIter *args,
 			errormsg = "Client does not have any 9p activity";
 		}
 	}
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	if (success)
 		server_dbus_9p_transstats(server_st->st._9p, &iter);
 
@@ -981,7 +940,7 @@ static bool get_9p_client_op_stats(DBusMessageIter *args,
 	dbus_message_iter_next(args);
 	if (success)
 		success = arg_9p_op(args, &opcode, &errormsg);
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	if (success)
 		server_dbus_9p_opstats(server_st->st._9p, opcode, &iter);
 
@@ -1009,6 +968,7 @@ static struct gsh_dbus_method *cltmgr_stats_methods[] = {
 	&cltmgr_show_v41_io,
 	&cltmgr_show_v41_layouts,
 	&cltmgr_show_delegations,
+	&cltmgr_client_io_ops,
 #ifdef _USE_9P
 	&cltmgr_show_9p_io,
 	&cltmgr_show_9p_trans,
