@@ -64,11 +64,13 @@
 #include "nfs_exports.h"
 #include "nfs_proto_functions.h"
 #include "pnfs_utils.h"
+#include "idmapper.h"
 
 struct timespec nfs_stats_time;
 struct timespec fsal_stats_time;
 struct timespec v3_full_stats_time;
 struct timespec v4_full_stats_time;
+struct timespec auth_stats_time;
 /**
  * @brief Exports are stored in an AVL tree with front-end cache.
  *
@@ -1099,12 +1101,12 @@ static bool gsh_export_addexport(DBusMessageIter *args,
 				msg_size += (strlen(conf_errs.buf)
 					     + strlen(". Errors found:\n"));
 				message = gsh_calloc(1, msg_size);
-				snprintf(message, msg_size,
+				(void) snprintf(message, msg_size,
 					 "%d exports added. Errors found:\n%s",
 					 exp_cnt, conf_errs.buf);
 			} else {
 				message = gsh_calloc(1, msg_size);
-				snprintf(message, msg_size,
+				(void) snprintf(message, msg_size,
 					 "%d exports added", exp_cnt);
 			}
 			dbus_message_iter_init_append(reply, &iter);
@@ -1441,7 +1443,7 @@ static bool export_to_dbus(struct gsh_export *exp_node, void *state)
 				       &exp_node->export_id);
 	dbus_message_iter_append_basic(&struct_iter, DBUS_TYPE_STRING, &path);
 	server_stats_summary(&struct_iter, &exp->st);
-	dbus_append_timestamp(&struct_iter, &last_as_ts);
+	gsh_dbus_append_timestamp(&struct_iter, &last_as_ts);
 	dbus_message_iter_close_container(&iter_state->export_iter,
 					  &struct_iter);
 	return true;
@@ -1453,12 +1455,10 @@ static bool gsh_export_showexports(DBusMessageIter *args,
 {
 	DBusMessageIter iter;
 	struct showexports_state iter_state;
-	struct timespec timestamp;
 
-	now(&timestamp);
 	/* create a reply from the message */
 	dbus_message_iter_init_append(reply, &iter);
-	dbus_append_timestamp(&iter, &timestamp);
+	gsh_dbus_append_timestamp(&iter, &nfs_stats_time);
 	dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY,
 					 "(qsbbbbbbbb(tt))",
 					 &iter_state.export_iter);
@@ -1480,6 +1480,41 @@ static struct gsh_dbus_method export_show_exports = {
 		 END_ARG_LIST}
 };
 
+/**
+ *   DBUS method to detailed client statistics
+ */
+static bool gsh_export_details(DBusMessageIter *args,
+				DBusMessage *reply,
+				DBusError *error)
+{
+	char *errormsg = "OK";
+	bool success = true;
+	DBusMessageIter iter;
+	struct gsh_export *export = NULL;
+
+	dbus_message_iter_init_append(reply, &iter);
+	export = lookup_export(args, &errormsg);
+	if (export == NULL) {
+		success = false;
+		errormsg = "Export ID not found";
+	}
+	gsh_dbus_status_reply(&iter, success, errormsg);
+	if (success) {
+		server_dbus_export_details(&iter, export);
+		put_gsh_export(export);
+	}
+	return true;
+}
+
+static struct gsh_dbus_method export_details = {
+	.name = "GetExportDetails",
+	.method = gsh_export_details,
+	.args = {ID_ARG,
+		STATUS_REPLY,
+		TIMESTAMP_REPLY,
+		CE_STATS_REPLY,
+		END_ARG_LIST}
+};
 /**
  * @brief Reset stat counters for all exports
  */
@@ -1619,12 +1654,12 @@ static bool gsh_export_update_export(DBusMessageIter *args,
 				msg_size += (strlen(conf_errs.buf)
 					     + strlen(". Errors found:\n"));
 				message = gsh_calloc(1, msg_size);
-				snprintf(message, msg_size,
+				(void) snprintf(message, msg_size,
 					 "%d exports updated. Errors found:\n%s",
 					 exp_cnt, conf_errs.buf);
 			} else {
 				message = gsh_calloc(1, msg_size);
-				snprintf(message, msg_size,
+				(void) snprintf(message, msg_size,
 					 "%d exports updated", exp_cnt);
 			}
 			dbus_message_iter_init_append(reply, &iter);
@@ -1733,7 +1768,7 @@ static bool get_nfsv3_export_io(DBusMessageIter *args,
 			errormsg = "Export does not have any NFSv3 activity";
 		}
 	}
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	if (success)
 		server_dbus_v3_iostats(export_st->st.nfsv3, &iter);
 
@@ -1781,7 +1816,7 @@ static bool get_nfsv40_export_io(DBusMessageIter *args,
 			errormsg = "Export does not have any NFSv4.0 activity";
 		}
 	}
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	if (success)
 		server_dbus_v40_iostats(export_st->st.nfsv40, &iter);
 
@@ -1829,7 +1864,7 @@ static bool get_nfsv41_export_io(DBusMessageIter *args,
 			errormsg = "Export does not have any NFSv4.1 activity";
 		}
 	}
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	if (success)
 		server_dbus_v41_iostats(export_st->st.nfsv41, &iter);
 
@@ -1877,7 +1912,7 @@ static bool get_nfsv41_export_layouts(DBusMessageIter *args,
 			errormsg = "Export does not have any NFSv4.1 activity";
 		}
 	}
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	if (success)
 		server_dbus_v41_layouts(export_st->st.nfsv41, &iter);
 
@@ -1907,13 +1942,13 @@ static bool get_nfsv_export_total_ops(DBusMessageIter *args,
 	export = lookup_export(args, &errormsg);
 	if (export != NULL) {
 		export_st = container_of(export, struct export_stats, export);
-		dbus_status_reply(&iter, success, errormsg);
+		gsh_dbus_status_reply(&iter, success, errormsg);
 		server_dbus_total_ops(export_st, &iter);
 		put_gsh_export(export);
 	} else {
 		success = false;
 		errormsg = "Export does not have any activity";
-		dbus_status_reply(&iter, success, errormsg);
+		gsh_dbus_status_reply(&iter, success, errormsg);
 	}
 	return true;
 }
@@ -1929,7 +1964,7 @@ static bool get_nfsv_global_total_ops(DBusMessageIter *args,
 	dbus_message_iter_init_append(reply, &iter);
 	if (!nfs_param.core_param.enable_NFSSTATS)
 		errormsg = "NFS stat counting disabled";
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	if (success)
 		global_dbus_total_ops(&iter);
 
@@ -1947,7 +1982,7 @@ static bool get_nfsv_global_fast_ops(DBusMessageIter *args,
 	dbus_message_iter_init_append(reply, &iter);
 	if (!nfs_param.core_param.enable_NFSSTATS)
 		errormsg = "NFS stat counting disabled";
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	if (success)
 		server_dbus_fast_ops(&iter);
 
@@ -1963,7 +1998,8 @@ static bool show_cache_inode_stats(DBusMessageIter *args,
 	DBusMessageIter iter;
 
 	dbus_message_iter_init_append(reply, &iter);
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_append_timestamp(&iter, &nfs_ServerBootTime);
 
 	mdcache_dbus_show(&iter);
 
@@ -2012,13 +2048,18 @@ static bool stats_reset(DBusMessageIter *args,
 	struct timespec timestamp;
 
 	dbus_message_iter_init_append(reply, &iter);
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	now(&timestamp);
-	dbus_append_timestamp(&iter, &timestamp);
+	gsh_dbus_append_timestamp(&iter, &timestamp);
 
 	reset_fsal_stats();
 	reset_server_stats();
+	reset_auth_stats();
 
+	/* update the stats counting time */
+	now(&nfs_stats_time);
+	fsal_stats_time = v3_full_stats_time = v4_full_stats_time =
+		auth_stats_time = nfs_stats_time;
 	return true;
 }
 
@@ -2046,10 +2087,10 @@ static bool stats_v3_full(DBusMessageIter *args,
 	if (!nfs_param.core_param.enable_FULLV3STATS) {
 		success = false;
 		errormsg = "v3_full stats disabled";
-		dbus_status_reply(&iter, success, errormsg);
+		gsh_dbus_status_reply(&iter, success, errormsg);
 		return true;
 	}
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	server_dbus_v3_full_stats(&iter);
 
 	return true;
@@ -2080,10 +2121,10 @@ static bool stats_v4_full(DBusMessageIter *args,
 	if (!nfs_param.core_param.enable_FULLV4STATS) {
 		success = false;
 		errormsg = "v4_full stats disabled";
-		dbus_status_reply(&iter, success, errormsg);
+		gsh_dbus_status_reply(&iter, success, errormsg);
 		return true;
 	}
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	server_dbus_v4_full_stats(&iter);
 
 	return true;
@@ -2109,18 +2150,18 @@ static bool stats_status(DBusMessageIter *args,
 	bool success = true;
 	char *errormsg = "OK";
 	DBusMessageIter iter, nfsstatus, fsalstatus;
-	DBusMessageIter v3_full_status, v4_full_status;
+	DBusMessageIter v3_full_status, v4_full_status, authstatus;
 	dbus_bool_t value;
 
 	dbus_message_iter_init_append(reply, &iter);
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 
 	/* Send info about NFS server stats */
 	dbus_message_iter_open_container(&iter, DBUS_TYPE_STRUCT, NULL,
 					 &nfsstatus);
 	value = nfs_param.core_param.enable_NFSSTATS;
 	dbus_message_iter_append_basic(&nfsstatus, DBUS_TYPE_BOOLEAN, &value);
-	dbus_append_timestamp(&nfsstatus, &nfs_stats_time);
+	gsh_dbus_append_timestamp(&nfsstatus, &nfs_stats_time);
 	dbus_message_iter_close_container(&iter, &nfsstatus);
 
 	/* Send info about FSAL stats */
@@ -2128,7 +2169,7 @@ static bool stats_status(DBusMessageIter *args,
 					 &fsalstatus);
 	value = nfs_param.core_param.enable_FSALSTATS;
 	dbus_message_iter_append_basic(&fsalstatus, DBUS_TYPE_BOOLEAN, &value);
-	dbus_append_timestamp(&fsalstatus, &fsal_stats_time);
+	gsh_dbus_append_timestamp(&fsalstatus, &fsal_stats_time);
 	dbus_message_iter_close_container(&iter, &fsalstatus);
 
 	/* Send info about NFSv3 Detailed stats */
@@ -2137,7 +2178,7 @@ static bool stats_status(DBusMessageIter *args,
 	value = nfs_param.core_param.enable_FULLV3STATS;
 	dbus_message_iter_append_basic(&v3_full_status, DBUS_TYPE_BOOLEAN,
 					&value);
-	dbus_append_timestamp(&v3_full_status, &v3_full_stats_time);
+	gsh_dbus_append_timestamp(&v3_full_status, &v3_full_stats_time);
 	dbus_message_iter_close_container(&iter, &v3_full_status);
 
 	/* Send info about NFSv4 Detailed stats */
@@ -2146,9 +2187,18 @@ static bool stats_status(DBusMessageIter *args,
 	value = nfs_param.core_param.enable_FULLV4STATS;
 	dbus_message_iter_append_basic(&v4_full_status, DBUS_TYPE_BOOLEAN,
 					&value);
-	dbus_append_timestamp(&v4_full_status, &v4_full_stats_time);
+	gsh_dbus_append_timestamp(&v4_full_status, &v4_full_stats_time);
 	dbus_message_iter_close_container(&iter, &v4_full_status);
 
+	/* Send info about auth stats */
+	dbus_message_iter_open_container(&iter, DBUS_TYPE_STRUCT, NULL,
+						&authstatus);
+	value = nfs_param.core_param.enable_AUTHSTATS;
+	dbus_message_iter_append_basic(&authstatus, DBUS_TYPE_BOOLEAN,
+						&value);
+	gsh_dbus_append_timestamp(&authstatus, &auth_stats_time);
+	dbus_message_iter_close_container(&iter, &authstatus);
+
 	return true;
 }
 
@@ -2191,6 +2241,7 @@ static bool stats_disable(DBusMessageIter *args,
 		nfs_param.core_param.enable_FSALSTATS = false;
 		nfs_param.core_param.enable_FULLV3STATS = false;
 		nfs_param.core_param.enable_FULLV4STATS = false;
+		nfs_param.core_param.enable_AUTHSTATS = false;
 		LogEvent(COMPONENT_CONFIG,
 			 "Disabling NFS server statistics counting");
 		LogEvent(COMPONENT_CONFIG,
@@ -2199,6 +2250,10 @@ static bool stats_disable(DBusMessageIter *args,
 		reset_fsal_stats();
 		/* resetting server stats includes v3_full & v4_full stats */
 		reset_server_stats();
+		LogEvent(COMPONENT_CONFIG,
+				"Disabling auth statistics counting");
+		/* reset auth counters */
+		reset_auth_stats();
 	}
 	if (strcmp(stat_type, "nfs") == 0) {
 		nfs_param.core_param.enable_NFSSTATS = false;
@@ -2231,12 +2286,20 @@ static bool stats_disable(DBusMessageIter *args,
 		reset_v4_full_stats();
 	}
 
-	dbus_status_reply(&iter, true, errormsg);
+	if (strcmp(stat_type, "auth") == 0) {
+		nfs_param.core_param.enable_AUTHSTATS = false;
+		LogEvent(COMPONENT_CONFIG,
+			"Disabling auth statistics counting");
+		/* reset auth counters */
+		reset_auth_stats();
+	}
+
+	gsh_dbus_status_reply(&iter, true, errormsg);
 	now(&timestamp);
-	dbus_append_timestamp(&iter, &timestamp);
+	gsh_dbus_append_timestamp(&iter, &timestamp);
 	return true;
 error:
-	dbus_status_reply(&iter, false, errormsg);
+	gsh_dbus_status_reply(&iter, false, errormsg);
 	return true;
 }
 
@@ -2299,6 +2362,13 @@ static bool stats_enable(DBusMessageIter *args,
 				 "Enabling NFSv4 Detailed statistics counting");
 			now(&v4_full_stats_time);
 		}
+		if (!nfs_param.core_param.enable_AUTHSTATS) {
+			nfs_param.core_param.enable_AUTHSTATS = true;
+			LogEvent(COMPONENT_CONFIG,
+					"Enabling auth statistics counting");
+			now(&auth_stats_time);
+		}
+
 	}
 	if (strcmp(stat_type, "nfs") == 0 &&
 			!nfs_param.core_param.enable_NFSSTATS) {
@@ -2338,12 +2408,21 @@ static bool stats_enable(DBusMessageIter *args,
 			now(&v4_full_stats_time);
 		}
 	}
-	dbus_status_reply(&iter, true, errormsg);
+
+	if (strcmp(stat_type, "auth") == 0 &&
+	    !nfs_param.core_param.enable_AUTHSTATS) {
+		nfs_param.core_param.enable_AUTHSTATS = true;
+		LogEvent(COMPONENT_CONFIG,
+				"Enabling auth statistics counting");
+		now(&auth_stats_time);
+	}
+
+	gsh_dbus_status_reply(&iter, true, errormsg);
 	now(&timestamp);
-	dbus_append_timestamp(&iter, &timestamp);
+	gsh_dbus_append_timestamp(&iter, &timestamp);
 	return true;
 error:
-	dbus_status_reply(&iter, false, errormsg);
+	gsh_dbus_status_reply(&iter, false, errormsg);
 	return true;
 }
 
@@ -2403,11 +2482,12 @@ static bool stats_fsal(DBusMessageIter *args,
 		errormsg = "FSAL stats disabled";
 		goto error;
 	}
-	dbus_status_reply(&iter, true, errormsg);
+	gsh_dbus_status_reply(&iter, true, errormsg);
+	gsh_dbus_append_timestamp(&iter, &fsal_stats_time);
 	fsal_hdl->m_ops.fsal_extract_stats(fsal_hdl, &iter);
 	return true;
 error:
-	dbus_status_reply(&iter, false, errormsg);
+	gsh_dbus_status_reply(&iter, false, errormsg);
 	return true;
 }
 
@@ -2452,7 +2532,7 @@ static bool get_9p_export_io(DBusMessageIter *args,
 			errormsg = "Export does not have any 9p activity";
 		}
 	}
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	if (success)
 		server_dbus_9p_iostats(export_st->st._9p, &iter);
 
@@ -2500,7 +2580,7 @@ static bool get_9p_export_op_stats(DBusMessageIter *args,
 	dbus_message_iter_next(args);
 	if (success)
 		success = arg_9p_op(args, &opcode, &errormsg);
-	dbus_status_reply(&iter, success, errormsg);
+	gsh_dbus_status_reply(&iter, success, errormsg);
 	if (success)
 		server_dbus_9p_opstats(export_st->st._9p, opcode, &iter);
 
@@ -2583,16 +2663,14 @@ static bool get_nfs_io(DBusMessageIter *args,
 	bool success = true;
 	char *errormsg = "OK";
 	DBusMessageIter reply_iter, array_iter;
-	struct timespec timestamp;
 
 	/* create a reply iterator from the message */
 	dbus_message_iter_init_append(message, &reply_iter);
 	if (!nfs_param.core_param.enable_NFSSTATS)
 		errormsg = "NFS stat counting disabled";
 	/* status and timestamp reply */
-	dbus_status_reply(&reply_iter, success, errormsg);
-	now(&timestamp);
-	dbus_append_timestamp(&reply_iter, &timestamp);
+	gsh_dbus_status_reply(&reply_iter, success, errormsg);
+	gsh_dbus_append_timestamp(&reply_iter, &nfs_stats_time);
 
 	/* create an array container iterator and loop over all exports */
 	dbus_message_iter_open_container(&reply_iter, DBUS_TYPE_ARRAY,
@@ -2635,6 +2713,8 @@ static struct gsh_dbus_method *export_stats_methods[] = {
 	&status_stats,
 	&v3_full_statistics,
 	&v4_full_statistics,
+	&auth_statistics,
+	&export_details,
 	NULL
 };
 
