@@ -178,6 +178,8 @@ static struct config_item core_params[] = {
 		       nfs_core_param, enable_FULLV3STATS),
 	CONF_ITEM_BOOL("Enable_FULLV4_Stats", false,
 		       nfs_core_param, enable_FULLV4STATS),
+	CONF_ITEM_BOOL("Enable_AUTH_Stats", false,
+		       nfs_core_param, enable_AUTHSTATS),
 	CONF_ITEM_BOOL("Short_File_Handle", false,
 		       nfs_core_param, short_file_handle),
 	CONF_ITEM_I64("Manage_Gids_Expiration", 0, 7*24*60*60, 30*60,
@@ -251,6 +253,16 @@ static struct config_item_list minor_versions[] = {
 	CONFIG_LIST_TOK("2", NFSV4_MINOR_VERSION_TWO),
 	CONFIG_LIST_EOL
 };
+
+static struct config_item_list recovery_backend_types[] = {
+	CONFIG_LIST_TOK("fs",			RECOVERY_BACKEND_FS),
+	CONFIG_LIST_TOK("fs_ng",		RECOVERY_BACKEND_FS_NG),
+	CONFIG_LIST_TOK("rados_kv",		RECOVERY_BACKEND_RADOS_KV),
+	CONFIG_LIST_TOK("rados_ng",		RECOVERY_BACKEND_RADOS_NG),
+	CONFIG_LIST_TOK("rados_cluster",	RECOVERY_BACKEND_RADOS_CLUSTER),
+	CONFIG_LIST_EOL
+};
+
 static struct config_item version4_params[] = {
 	CONF_ITEM_BOOL("Graceless", false,
 		       nfs_version4_parameter, graceless),
@@ -277,13 +289,15 @@ static struct config_item version4_params[] = {
 		       nfs_version4_parameter, pnfs_mds),
 	CONF_ITEM_BOOL("PNFS_DS", true,
 		       nfs_version4_parameter, pnfs_ds),
-	CONF_ITEM_STR("RecoveryBackend", 1, MAXPATHLEN,
-		      RECOVERY_BACKEND_DEFAULT,
-		      nfs_version4_parameter, recovery_backend),
+	CONF_ITEM_TOKEN("RecoveryBackend", RECOVERY_BACKEND_DEFAULT,
+			recovery_backend_types, nfs_version4_parameter,
+			recovery_backend),
 	CONF_ITEM_LIST("minor_versions", NFSV4_MINOR_VERSION_ALL,
 		       minor_versions, nfs_version4_parameter, minor_versions),
 	CONF_ITEM_UI32("slot_table_size", 1, 1024, NFS41_NB_SLOTS_DEF,
 		       nfs_version4_parameter, nb_slots),
+	CONF_ITEM_BOOL("Enforce_UTF8_Validation", false,
+		       nfs_version4_parameter, enforce_utf8_vld),
 	CONFIG_EOL
 };
 
